/* fmap_cvt - convert a fiber map between big- and little-endian 
 *
 * USAGE: fmap_cvt map x y
 *
 *	Marty Olevitch, Dec 1996
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <math.h>
#include <softcris.h>

char *Progname;

int
main(int argc, char *argv[])
{
    unsigned char *cp;
    short d;
    float f;
    struct sc_fmapElem *fmapp;
    struct sc_fmapElem *fmap[SC_FMAPX];
    int i;
    char *infile;
    int j;
    char *outfile;
    FILE *outfp;
    unsigned long *lp;
    int ret;

    Progname = argv[0];

    if (argc != 3) {
	fprintf(stderr, "USAGE: %s inmap outmap\n", Progname);
	exit(1);
    }

    infile = argv[1];
    outfile = argv[2];

    outfp = fopen(outfile, "w");
    if (outfp == NULL) {
	fprintf(stderr,"%s: can't open '%s' (%s)\n", Progname, outfile,
	    strerror(errno));
	exit(1);
    }

    for (i=0; i<SC_FMAPX; i++) {
	fmap[i] = (struct sc_fmapElem *)
	    malloc(SC_FMAPY * sizeof(struct sc_fmapElem));
	if (fmap[i] == (struct sc_fmapElem *)NULL) {
	    fprintf(stderr, "%s: can't malloc map (tab %d)\n", 
	    	Progname, i);
	    exit(1);
	}
    }

    ret = sc_read_fmap_by_name(infile, fmap);
    if (ret) {
	exit(ret);
    }

    for (i=0; i<SC_FMAPX; i++) {
	for (j=0; j<SC_FMAPY; j++) {
	    fmapp = &fmap[i][j];
	    f = fmapp->fiber;
	    cp = (unsigned char *)&f;
	    lp = (unsigned long *)&fmapp->fiber;
	    sc_little_endian_long(cp, lp);
	    d = fmapp->layer;
	    cp = (unsigned char *)&d;
	    swab((char *)&d, (char *)&fmapp->layer, sizeof(short));
#ifdef NOTDEF
	    printf("%d %d %f %f %02x %d\n", 
		i, j, f, fmapp->fiber, fmapp->ypos, fmapp->layer);
#endif
	}
    }

    for (i=0; i<SC_FMAPX; i++) {
	ret = fwrite((char *)fmap[i], sizeof(struct sc_fmapElem), 
	    SC_FMAPY, outfp);
	if (ret != SC_FMAPY) {
	    fprintf(stderr, "%s: bad write, tab %d (%s)\n", Progname,
	    	i, strerror(errno));
	    exit(1);
	}
    }

    fclose(outfp);

    exit(0);
}
