/* softcris.h - header file for the WU Lexas soft/cris library routines
 *
 *	Marty Olevitch
 *	Washington U in St Louis
 *	Physics Dept
 *	Lab for Experimental Astrophysics
 *
 *	    Jan 1994, Nov 1996
 */

#ifndef _SOFTCRIS_H
#define _SOFTCRIS_H

static char softcris_h_cvsid[] = "$Id: softcris.h,v 1.2 1999/05/27 17:47:32 marty Exp $";

#include <stdio.h>
#include <sys/types.h>	/* for u_char, etc. */

#ifdef _WINDOWS
#define SC_VERSION "5.3f"
#endif /* _WINDOWS */

#ifdef LIB_MAKE
    /* compiling the library */
#   include "crisevt.h"
#else
    /* compiling applications */
#   define NO_LEVEL1 1	/* #define this if no L1 level1 library */
#   include <crisevt.h>
#endif

#ifndef NO_LEVEL1		/* don't want (have) level1 routines */
#   ifndef _TRAJ_C_QSORT_QUIRK	/* see traj.c for explanation */
#	include <L1.h>		/* level1 macros and prototypes */
#    endif
#endif

#ifndef CHOP
    /* CHOP - replace last character of a string */
#   define CHOP(s, c)	((s)[strlen(s)-1] = (c))
#endif

#ifndef SOFT_STREQ
#   define SOFT_STREQ(a,b)	(strcmp((a),(b)) == 0)
#endif

#ifndef SOFT_STREQN
#   define SOFT_STREQN(a,b,n)	(strncmp((a),(b),(n)) == 0)
#endif

/* SC_MSU_MAXBLOBS - Never more blobs per frame than this in an MSU event */
#define SC_MSU_MAXBLOBS		50

/* The "CE_" and "ce_" prefixes come from crisevt.h file. */
#define SC_CRIS_MAXBLOBS	CE_CRIS_MAXBLOBS
#define SC_CRIS_NCTRPH	CE_CRIS_NCTRPH
#define SC_CRIS_NGUARD	CE_CRIS_NGUARD
#define SC_CRIS_AB	CE_CRIS_AB
#define SC_CRIS_CD	CE_CRIS_CD
#define sc_crisHodoData	ce_crisHodoData
#define sc_crisStimData ce_crisStimData
#define sc_crisEvt	ce_crisEvt

#define SC_TEL_A	0
#define SC_TEL_B	1
#define SC_TEL_C	2
#define SC_TEL_D	3

/* SC_PROC_CRIS_MAXBLOBS - This is the maximum number of blobs that CRIS
 * will process on board the spacecraft. I.e. a frame with more than this
 * number of blobs will be rejected as an error. However, fewer than this
 * number of blobs should actually be in the telemetered data stream. */
#define SC_PROC_CRIS_MAXBLOBS	89

/* SC_MAXBLOBS - the largest of all the different MAXBLOBS values - used to
 * size arrays in blobify routines in blob.c */
#define SC_MAXBLOBS	SC_PROC_CRIS_MAXBLOBS

/* sc_msuDisc - discriminator, etc stuff */
struct sc_msuDisc {
    u_char camdisc;	    /* camera discriminator */
    u_char trig1, trig2;    /* trigger discrims */
    u_char iigain;	    /* image intensifier gain */
    u_char coinen;	    /* coincidence enable mask */
    u_char cointhr1, cointhr2, cointhr3;    /* coincidence thresholds */
    u_char camdiscb;	    /* cam discrim vid system 2 */
    u_char trig1b, trig2b;  /* trigger discrims vid sys 2 */
    u_char iigainb;	    /* image intensifier gain vid sys 2 */
    u_short scalars[6];
};

/* sc_msuPhas - pha data. The raw items do not have the parity and hazard
 * bits masked out. x and y are calculated from x1, x2, y1, and y2. */
struct sc_msuPhas {
    u_short s1, x1, x2, y1, y2, pd1, pd2, par;
    u_short s1_raw, x1_raw, x2_raw, y1_raw, y2_raw, pd1_raw, pd2_raw;
    u_short x, y;
    u_short s2, x1b, x2b, y1b, y2b;
    u_short s2_raw, x1b_raw, x2b_raw, y1b_raw, y2b_raw;
    u_short xb, yb;
};

/* codes for the "valid" variable in the sc_msuEvt struct */
#define SC_MSU_INVALID	0   /* data in this "event" is not valid */
#define SC_MSU_EVT	1   /* event was some kind of event */
#define SC_MSU_SPILL	2   /* event was a spill marker */

/* sc_msuEvt - information about each event as it is processed */
struct sc_msuEvt {
    u_char *ptr;	/* pointer to start of raw data for this evt */
    u_long spill;	/* current spill number */
    u_long size;	/* evt size (bytes) (includes any alignment bytes) */
    u_long abs;		/* absolute evt no. (corrected abs evt no for ID 6) */
    u_long abs_orig;	/* original (uncorrected) absolute evt no. for ID 6 */
    u_long evtno;	/* evt no for this type of evt */
    u_long vidsize;	/* size (bytes) of video data for vid system 1 */
    u_long vidsize2;	/* size (bytes) of video data for vid system 2 */
    u_long clock;	/* PC clock tick count */
    u_long blobI[SC_MSU_MAXBLOBS];	/* blob intensities */
    u_long blobIb[SC_MSU_MAXBLOBS];	/* blob intensities  vid sys 2*/
    u_short type;	/* event type */
    u_short exp_id;	/* experiment id */
    u_short nblobs;	/* no. of blob centroids in this evt */
    u_short blobx[SC_MSU_MAXBLOBS];	/* blob x coordinates */
    u_short bloby[SC_MSU_MAXBLOBS];	/* blob y coordinates */
    u_short nblobsb;	/* no. of blob centroids in this evt; vid sys 2 */
    u_short blobxb[SC_MSU_MAXBLOBS];	/* blob x coordinates vid sys 2 */
    u_short blobyb[SC_MSU_MAXBLOBS];	/* blob y coordinates vid sys 2 */
    u_char *vidptr;	/* pointer to vid sys 1 raw video data (START_HDR) */
    u_char *vidptr2;	/* pointer to vid sys 2 raw video data (START_HDR) */
    char *reply;	/* pointer to reply string for reply evts */
    struct sc_msuPhas phas;	/* pha data */
    struct sc_msuDisc disc;	/* discriminator, etc. data */
    int valid;		/* nonzero if this data is valid (see codes above) */
};

int sc_msu_init(FILE *infile, FILE *errfile, int verb, char *progname);
int sc_msu_next_evt(struct sc_msuEvt *e);
int sc_msu_next_pix(u_short *x, u_short *y, u_short *val, int vidsys);

int sc_msu_fix_blobs(struct sc_msuEvt *e);
int sc_msu_blobify(struct sc_msuEvt *e, int *x_cen, int *y_cen, char *errmesg,
    u_short *blob_pix_ptr, int *blob_npix, long *blob_int, int *pix_per_blob,
    int min_pix, double *Fx_cen, double *Fy_cen, int vidsys);
int sc_cris_blobify(u_char *dataptr, int *x_cen, int *y_cen, char *errmesg,
    u_short *blob_pix_ptr, int *blob_npix, long *blob_int,
    int *pix_per_blob, int min_pix, double *Fx_cen, double *Fy_cen);
void sc_blob_segments(int *nblobs, int *blobsegs, int *totblobsegs, 
    int *totsegs);
int sc_blob_totpix(void);
int sc_check_soft_video(u_short *ptr, short maxwds);
void sc_version(char *str);
int sc_msu_finito(void);
long sc_soft_img_convert(u_short *rawp, u_short *vidp);
int sc_soft_next_pix(u_short *xp, u_short *yp, u_short *Ip, u_short *imgp);
int sc_too_many_blobs(void);
int sc_set_maxblobs(int n);
int sc_get_maxblobs(void);
int sc_set_pix_discrim(int val);
int sc_get_pix_discrim(void);

int sc_msu_reorder_std(u_char *ptr, u_char *swapbuf);
int sc_msu_reorder_reply(u_char *ptr, u_char *swapbuf);
int sc_msu_reorder_real(u_char *ptr, u_char *swapbuf, long *sizep, 
    long *vidsizep, long *vidsizep2, u_short *exp_id);
void sc_msu_reorder_spill(u_char *ptr, u_char *swapbuf);
int sc_msu_reorder_pha_only(u_char *ptr, u_char *swapbuf, long *sizep);
void sc_msu_reorder_dl(u_char *ptr, u_char *swapbuf);
void sc_little_endian_long(u_char *srcp, u_long *destp);
int sc_reorder_soft(u_short *ptr, u_short *swapbuf, int max);

/* lookup stuff to convert from pixel to fiber coords. For SOFT data.
 *
 *  Marty Olevitch, May 1994
 *	(originally based on Dave Crary's stuff)
 */

#define SC_NLUTS	5   /* no. of lookup tables usable at one time */
#define SC_NLUTPTS	6   /* no. of lookup table reference pts for each tab */
#define SC_NTABS	66  /* max. no. of tabs - there may be fewer */
#define SC_NLAYERS	6   /* no. of layers */

int sc_lu_read_table(char *lutfile);
int sc_lu_tab(int lut, double x, double y);
int sc_lu_layer(int lut, int tab);
int sc_lu_pos(int lut, double *pos, int layer, int tab, double x, double y);
int sc_lu_pos_extra(int lut, double *pos, int layer, int tab, double x, double y,
   double *prevpos, int *prevlayer, int *prevtab, double *px, double *py,
   double *nextpos, int *nextlayer, int *nexttab, double *nx, double *ny);
char *sc_lu_report_layer(int lut, int layer);
int sc_lu_write_binary_table(int lut, char *filename);
int sc_lu_read_binary_table(char *filename);
int sc_lu_pix_to_real_pos(int lut, double x, double y, int *tab, int *layer,
    double *rpos);

void sc_lu_print_reflen(int lut);    /* for debugging */

/* here are the macros for the pixel-to-fiber map routines */
#define SC_FMAPX 1060	/* number of discrete x positions */
#define SC_FMAPY 512	/* number of discrete y positions */

/* each element of our centroid-to-fiber map */
struct sc_fmapElem {
    float fiber;	/* fiber "number" + relative x position */
    char ypos;		/* relative y position from tab line */
    short layer;	/* -1 (NOLAYER) = not mapped;
    			 * else Y3=0, X3=1, Y2=2, X2=3, Y1=4, X1=5 */
};

/* NOTE: the order of these SC_[XY][123] macros is the order that the
 * layers appear in pixel space (with the origin at the top), i.e: Y3 is on
 * top, followed by X3, Y2, X2, Y1, X1. However, in reality the actual
 * fiber planes are such that X1 is on top, followed by Y1, X2, Y2, X3, Y3.
 * The software takes care of all this. Don't be confused! (I once was...)
 */
#define SC_Y3	0
#define SC_X3	1
#define SC_Y2	2
#define SC_X2	3
#define SC_Y1	4
#define SC_X1	5
#define SC_NOLAYER	-1

#define IS_X_LAYER(x)		(x == SC_X3 || x == SC_X2 || x == SC_X1)
#define IS_Y_LAYER(y)		(y == SC_Y3 || y == SC_Y2 || y == SC_Y1)

#define IS_HODO_LAYER_1(s)	(s == SC_X1 || s == SC_Y1)
#define IS_HODO_LAYER_2(s)	(s == SC_X2 || s == SC_Y2)
#define IS_HODO_LAYER_3(s)	(s == SC_X3 || s == SC_Y3)
#define HODO_LAYER(s)	(IS_HODO_LAYER_1(s) ? 1 : \
			 (IS_HODO_LAYER_2(s) ? 2 : \
			  (IS_HODO_LAYER_3(s) ? 3 : SC_NOLAYER)))

#define SC_CAM1	1
#define SC_CAM2	2

/* centroid to fiber stuff */
struct sc_fmapElem *sc_centr_to_fiber(int cam, int x, int y);
int sc_read_fmap(int cam, struct sc_fmapElem *fmapp[SC_FMAPX]);
int sc_read_fmap_by_name(char *filename, struct sc_fmapElem *fmapp[SC_FMAPX]);

/* real position stuff */
int sc_fiber_to_real(int c, int l, float xf, float yf, float *xi, float *yi);
void sc_fiber_inches(void);
void sc_fiber_mm(void);

#define SC_SOFT_WIDTH    550
#define SC_SOFT_HEIGHT   244

/* video data headers */
#define SC_START_HDR		0x1000
#define SC_ALT_START_HDR	0x0010	/* other endian START_HDR */
#define SC_END_HDR		0x1c00
#define SC_ALT_END_HDR		0x001c	/* other endian END_HDR */
#define SC_Y_HDR		0x1400
#define SC_X_HDR		0x1800

#define SC_IS_VIDEO_HDR(h)	((h) & 0x1000)
#define SC_IS_Y_HDR(h)		((h) & 0x0400)
#define SC_IS_X_HDR(h)		((h) & 0x0800)

#define SC_Y_VAL(h)	(unsigned short)((h) & 0x00ff)
#define SC_X_VAL(h)	(unsigned short)((h) & 0x03ff)
#define SC_VID_DATA(h)	(unsigned short)((h) & 0x0fff)

/* macros to unpack the fields in the normal event */
#define SC_CRIS_IS_STIM(p)	(*(p+1) & 0x80)
#define SC_CRIS_PRIM_TEL_PAIR(p)((*(p+1) & 0x40) >> 6)
#define SC_CRIS_EVT_BUF_ID(p)	(*(p+1) & 0x3f)

#define SC_CRIS_GUARD_HI_OR(p)	(*(p+2) & 0x80)
#define SC_CRIS_HAZARD(p)	(*(p+2) & 0x40)
#define SC_CRIS_G1(p)		((*(p+2) >> 3) & 0x07)
#define SC_CRIS_R1(p)		((*(p+2) & 0x07) + 2)

#define SC_CRIS_GOT_TEL2(p)	(*(p+3) & 0x80)
#define SC_CRIS_GOT_GUARD(p)	(*(p+3) & 0x40)
#define SC_CRIS_CENTROID_ERR(p)	(*(p+3) & 0x20)
#define SC_CRIS_SOFT_NBLOBS(p)	(*(p+3) & 0x1f)

#define SC_CRIS_G2(p)	((SC_CRIS_GOT_TEL2((p))) ? ((*(p+6) >> 3) & 0x07) : 0)
#define SC_CRIS_R2(p)	((SC_CRIS_GOT_TEL2((p))) ? ((*(p+6) & 0x07) + 2) : 0)

#define SC_CRIS_NDET	32	/* no. of CRIS "detector" names */
#define SC_STACKSIZE	72	/* size (bytes) of raw stack data */
#define SC_SOFT_MAXSIZE	6000	/* software limit (bytes) on size of SOFT
				 * data transmitted in a diagnostic evt */

/* sc_crisRawStack is used inside the sc_diagEvt struct */
struct sc_crisRawStack {
    u_short tags[4];		/* tag information */
    u_short ph[SC_CRIS_NDET];	/* pulse heights - only lowest 12 bits used */
};

/* sc_diagEvt contains the data for a diagnostic event */
struct sc_diagEvt {
    struct sc_crisEvt e;	/* cris normal data */
    struct sc_crisRawStack s;	/* raw stack data */
    u_short softp[SC_SOFT_MAXSIZE/2];	/* raw soft image data */
    u_short softwds;		/* no. of 16-bit words of soft image data */
};

/* CRIS raw stack data (unpacked). These structures are the same as Robert
 * Radocinski's L1_1CrisRaw... structs. I didn't want to use the same names
 * as he did. These structures is used by sc_cris_stack_unpack(). */
struct sc_crisRawTag
{
    u_char stim_puls_tag;
    u_char stim_off_tag;
    u_char hazard;
    u_char ghor;
    u_char mor;
    u_char hor;
    u_char soft_trig0;
    u_char soft_trig1;
    u_char hy_en;
    u_char he_en;
};

struct sc_crisRawTelescopeTag
{
    u_char z1;
    u_char z2;
    u_char z_gt_2;
    u_char adc2;
    u_char min_range;
    u_char e1_0, e1_1;
    u_char e2, e3, e4, e5, e6, e7, e8, e9, g2, g3, g4, g5, g6, g7;
};

struct sc_crisRawTelescope
{
    struct sc_crisRawTelescopeTag tag;
    u_short e1_0, e1_1;
    u_short e2, e3, e4, e5, e6, e7, e8, e9, g2, g3, g4, g5, g6, g7;
};

struct sc_crisRawEvent
{
    struct sc_crisRawTelescope ab;
    struct sc_crisRawTelescope cd;
    struct sc_crisRawTag tag;
    u_char CycleCount;
};

int sc_cris_allow_1_byte_evts(int set);
int sc_cris_get_cam(void);
int sc_cris_set_cam(int cam);
void sc_cris_set_secs1996(unsigned long sec);
int sc_cris_unpack_cam(u_char *p, struct sc_crisEvt *c);
int sc_cris_unpack(u_char *p, struct sc_crisEvt *c, int cam);
void sc_cris_unpack_raw(u_short *dp, struct sc_crisRawStack *p);
void sc_cris_unpack_stack(struct sc_crisRawStack *s, struct sc_crisRawEvent *e);
int sc_diag_unpack(u_char *p, struct sc_diagEvt *d, int cam);
int sc_diag_unpack_cam(u_char *p, struct sc_diagEvt *d);
int sc_chk_cris_norm( u_char *p, struct sc_crisEvt *c);
int sc_search_norm_start(u_char *p, int max);

void sc_msu_blob_sort(struct sc_msuEvt *ssrc, struct sc_msuEvt *sdest);
void sc_cris_blob_sort(struct sc_crisEvt *csrc, struct sc_crisEvt *cdest);
void sc_raw_blob_sort( int *xsrc, int *ysrc, long *Isrc, int *xdest,
   int *ydest, long *Idest, int nblobs);

struct sc_softCoords {
    float x1, y1, x2, y2, x3, y3;	/* real x,y in mm for each XY layer */
    float fiber[SC_NLAYERS];		/* fiber no. in the layer */
    float ypos[SC_NLAYERS];		/* y-position in the layer */
    int cam;				/* camera (CAM1 or CAM2) */
    short xpix[SC_NLAYERS];		/* x pixel value in the layer */
    short ypix[SC_NLAYERS];		/* y pixel value in the layer */
    unsigned long I[SC_NLAYERS];	/* Intensity value in the layer */
};

int sc_cris_coords_cam(struct sc_crisEvt *csrcp, struct sc_softCoords *kp);
int sc_cris_coords(struct sc_crisEvt *csrcp, struct sc_softCoords *kp, int cam);
int sc_msu_coords(struct sc_msuEvt *srcp, struct sc_softCoords *kp1,
    struct sc_softCoords *kp2);
int sc_raw_coords_cam(int *xsrcp, int *ysrcp, long *Isrcp, int nblobs,
    struct sc_softCoords *kp);
int sc_raw_coords(int *xsrcp, int *ysrcp, long *Isrcp, int nblobs,
    int cam, struct sc_softCoords *kp);
#define sc_blob_select_method(m) sc_traj_select_method(m) /* back compatible */
int sc_traj_select_method(int m);
char *sc_describe_all_traj_methods(void);
char *sc_describe_traj_method(int m);
void sc_set_z_coord(float x1, float y1, float x2, float y2, float x3, float y3);
void sc_get_z_coord(float *x1, float *y1, float *x2, float *y2,
    float *x3, float *y3);
void sc_set_soft_stack_sep(float sep);
float sc_get_soft_stack_sep(void);
float * sc_get_stack_z(int *n);

/* offsets into sc_crisRawStack.ph pulse heights */
enum {
     SC_E1A,  SC_E1B, SC_E2AB, SC_E3AB, SC_E4AB, SC_E5AB, SC_E6AB, SC_E7AB,
    SC_E8AB, SC_E9AB, SC_G2AB, SC_G3AB, SC_G4AB, SC_G5AB, SC_G6AB, SC_G7AB,
     SC_E1C,  SC_E1D, SC_E2CD, SC_E3CD, SC_E4CD, SC_E5CD, SC_E6CD, SC_E7CD,
    SC_E8CD, SC_E9CD, SC_G2CD, SC_G3CD, SC_G4CD, SC_G5CD, SC_G6CD, SC_G7CD
};

/* SC_CRIS_RAW_PH will return pulse height no. n value pointed to by p */
#define SC_CRIS_RAW_PH(p, n)	((p)->ph[(n)])

int sc_cris_det_name(int n, char *s);

#define SC_NSTACK	21	/* number of stack layers - includes E1,
				 * E2, E3_1, E3_2, etc. */

/* stack layer ID macros */
#define SC_E1	0
#define SC_E2	1
#define SC_E3_1	2
#define SC_E3_2	3
#define SC_E3	4
#define SC_E4_1	5
#define SC_E4_2	6
#define SC_E4	7
#define SC_E5_1	8
#define SC_E5_2	9
#define SC_E5	10
#define SC_E6_1	11
#define SC_E6_2	12
#define SC_E6	13
#define SC_E7_1	14
#define SC_E7_2	15
#define SC_E7	16
#define SC_E8_1	17
#define SC_E8_2	18
#define SC_E8	19
#define SC_E9	20

struct sc_softTraj {
    double layer1x, layer1y;	/* (x,y) in hodo layer 1 (mm) */
    double layer2x, layer2y;	/* (x,y) in hodo layer 2 (mm) */
    double layer3x, layer3y;	/* (x,y) in hodo layer 3 (mm) */
    double stackx[SC_NSTACK];	/* x at top of each stack (mm) */
    double stacky[SC_NSTACK];	/* y at top of each stack (mm) */
    double theta;		/* (degrees) */
    double phi;			/* (degrees) */
    double chisqx;		/* chisq for fit in XZ plane */
    double chisqy;		/* chisq for fit in YZ plane */
};

struct sc_trajQual {
    float fit_param;		/* measure of fit */
    int hop_param;		/* measure of hopping */
    float Iratio[SC_NLAYERS];	/* ratio of I to maxI in layer */
    float qual_factor;		/* quality factor */
    short raw_dx_1000;		/* dx by x_fibers * 1000 */
    short raw_dy_1000;		/* dy by y_fibers * 1000 */
    long lreserved;
    double reserved;
};

struct sc_evt_params {
    float nextqual;		/* qual_param for trajectory following last */
    u_char nxtrack;		/* number of x_tracks */
    u_char nytrack;		/* number of y_tracks */
    u_char ntraj;		/* number of traj.s found (XXX ntraj) */
    char ret_code;		/* return code from trajectory routines */
    char tel;
    char creserved[3];
};

/* sc_Coords_Traj - "super" structure containing coords, trajectories, etc */
struct sc_Coords_Traj {
    struct sc_softCoords C;
    struct sc_softTraj T;
    struct sc_trajQual Q;
    u_char blobid[SC_NLAYERS];
    short sreserved;
    double reserved;
};

int sc_check_bundle(int doit);
void sc_check_chisq(int x, int y);
void sc_set_max_traj_chisq(float x, float y);
int sc_check_dev(int doit);
int sc_check_e1(int doit);
int sc_check_telescope(int doit);
int sc_cris_traj(struct sc_crisEvt *e, struct sc_Coords_Traj *t, int *ntraj);
long * sc_layer_maxI(void);
int sc_msu_traj(struct sc_msuEvt *e, struct sc_Coords_Traj *t1,
    struct sc_Coords_Traj *t2, int *ntraj1, int *ntraj2);
int sc_raw_traj(int *x, int *y, long *I, int nblobs,
    struct sc_Coords_Traj *t, int *ntraj);
int sc_hop(int dir, int cam, int xpix, int ypix, float fiberno,
    int *dyp, struct sc_fmapElem *fp);
int sc_do_hopping(int doit);
int sc_short_traj(struct sc_softCoords *c, struct sc_softTraj *t);
int sc_telescope_hit(struct sc_crisEvt *c, struct sc_softTraj *t,
    int stack, int r);
void sc_get_fit_limits(float *xfac, float *xlo, float *xhi, float *yfac,
    float *ylo, float *yhi);
void sc_set_fit_limits(float xfac, float xlo, float xhi, float yfac, 
    float ylo, float yhi);
void sc_get_evt_params(struct sc_evt_params *ep);
int sc_set_maxtraj(int n);
int sc_get_maxtraj(void);
int sc_set_maxtracks(int n);
int sc_get_maxtracks(void);
void sc_get_x_dev_limits(float *limlo, float *limhi, float *factor);
void sc_get_y_dev_limits(float *limlo, float *limhi, float *factor);
void sc_set_x_dev_limits(float limlo, float limhi, float factor);
void sc_set_y_dev_limits(float limlo, float limhi, float factor);
int sc_x_dev_ok(float x1, float x2, float x3, float *xdev);
int sc_y_dev_ok(float y1, float y2, float y3, float *ydev);
float sc_x_dev(float x1, float x2, float x3);
float sc_y_dev(float y1, float y2, float y3);

void sc_get_traj_mb(double *mx, double *bx, double *my, double *by);
int sc_trajectory( struct sc_softCoords *coordsPtr, struct sc_softTraj *trajPtr);
double sc_layer_z(int layer);

/* Stuff for handling cris telemetry minor frame data */

/* number of bytes in the cris telemetry minor frame. This is equivalent to
 * the NUM_CRIS_TELEMETRY_BYTES seen in some non-Lexas programs. */
#define SC_CRIS_TELEMBYTES	58

/* sc_crisSubset - this is the cris portion of the spacecraft telemetry
 * data. From Robert Radocynski. */
typedef struct {
    u_int PacketSecondaryHeader;
    u_int TimeOfDay;
    u_short DayOfYear;
    u_short year;
    u_short ErrorFlags;
    u_short PhaseAngle;
    u_char facility;
    u_char subfacility;
    u_char assembly;
    u_char DataNature;
    u_char VirtualChannelId;
    u_char MajorFrameCount;
    u_char MinorFrameCount;
    u_char SpacecraftId;
    u_char FormatId;
    u_char MainBusVoltage;
    u_char MainBusCurrent;
    u_char CrisInternalTemperature1;
    u_char CrisInternalTemperature2;
    u_char CrisInterfaceTemperature;
    u_char CrisCurrent;
    u_char CrisData[SC_CRIS_TELEMBYTES];
} sc_crisSubset;

#define SC_TLM_SYNC	0x80
#define SC_TLM_CMD	0x40
#define SC_TLM_DIAG	0x20

#define SC_NEVTS_MINOR	10

struct sc_minorFrame {
    sc_crisSubset s;		/* the raw "cris subset" data */
    int major_frame_start;	/* 1 if beginning of major frame */
    int seq;			/* sequence no. of this minor frm in major fr */
    int lost_prev_frame;	/* 1 if last minor frame missing */
    u_char hsk[3];		/* housekeeping bytes (including SYNC, CMD,
				 * and DIAG bits */
    u_char cmd[SC_CRIS_TELEMBYTES-3];	/* cmd string */
    int diag;			/* no. bytes of diagnostic data */
    u_char diagdata[SC_CRIS_TELEMBYTES-3];	/* the diag data */
    int nevt;			/* no. cris normal evts in this minor frame */
    struct sc_crisEvt e[SC_NEVTS_MINOR];	/* the cris normal evt data */
};

void sc_reorder_cris_subset(sc_crisSubset *ptr, sc_crisSubset *swapbuf);

int sc_tlm_next_minor(struct sc_minorFrame *f, FILE *fp);
int sc_tlm_sync_major(FILE *fp);
int sc_tlm_next_evt(struct sc_crisEvt *c, struct sc_diagEvt *d,
    FILE *fp, int *norm);
int sc_tlm_next_cris_evt(struct sc_crisEvt *c, FILE *fp);
int sc_tlm_next_diag(struct sc_diagEvt *d, FILE *fp);
int sc_tlm_hsk(int idx);
int sc_tlm_hsk_trg0rat(void);
int sc_tlm_hsk_trg1rat(void);
int sc_tlm_hsk_trg0and1(void);
void sc_tlm_hsk_trig( int *trig0, int *trig1, int *trig0and1);
int sc_tlm_hsk_mcp1(void);
int sc_tlm_hsk_mcp2(void);
int sc_tlm_cam_guess(void);
void sc_most_recent_cris_subset(sc_crisSubset *s);
void sc_most_recent_minor(struct sc_minorFrame *f);
int sc_tlm_save_minor(int v);
int sc_tlm_next_raw_diag(u_char *ptr, int *n, FILE *fp);

long sc_tlm_sec1970(sc_crisSubset *c);
long sc_tlm_sec1996(sc_crisSubset *c);

/* stuff for dealing with the GSI '96 "chapter 2" data */

/* sc_ch2 - data at the beginning of the chap 2 (up to the user data) */
struct sc_ch2 {
    u_char 	*p;			/* ptr to raw chap 2 data */
    short	key;
    short	spare;
    short	tags1;			/* verse 1 (tagbits) */
    short	tags0;
    u_char	ext_tags;
    u_char	cal_dac_hi_bytes;
    short	cal_dac_lo_bytes;
    u_long	time_seq;		/* verse 2 (time) */
    u_long	time;
    u_long	live_time;
    short	base_peak_time[24][3];	/* verse 3 (evt data) */
    short	user_data_words;	/* verse 4 (user data) */
    short	stage[8];
    u_char	*userp;			/* ptr to raw user data */
    long	seq_no;			/* verse 5 (stop) */
    long	sync;
};

#define SC_CH2_SYNCWORD	0x8dd4a59f	/* sync word at end of each chap 2 */

/* return codes */
#define SC_GSI_OK		0	/* all well */
#define SC_GSI_READ_ERR		-1	/* read error on input file */
#define SC_GSI_EOF		-2	/* EOF on input file */
#define SC_GSI_BAD_SYNC		-3	/* bad sync word in this event */
#define SC_GSI_BAD_KEY		-4	/* chap 2 key != 2 */
#define SC_GSI_FAIL		-5	/* general failure code */
#define SC_GSI_BAD_USER_DATA	-6	/* garbage in user data */
#define SC_GSI_ZEROES		-7	/* zero bytes at start of user data */
#define SC_GSI_BAD_NORM		-8	/* bad normal event */
#define SC_GSI_BAD_SOFT		-9	/* no END_HDR in soft data */

void sc_gsi_finito(void);
int sc_gsi_next_evt(FILE *fp, struct sc_ch2 *ch2, struct sc_diagEvt *d,
    char *filename);
int sc_gsi_resync(FILE *fp, char *filename);
u_char *sc_gsi_search_for_soft(u_char *startp, int maxwds, int *bytesrd);
u_char *sc_gsi_skip_zeroes(u_char *startp, int max);

/* charge stuff based on Mike Thayer's Z algorithm */
struct sc_crisZ {
    double MeV[SC_CRIS_NCTRPH - 1];
    double zbest;
    int range;
};

int sc_z(struct sc_crisEvt *c, struct sc_softTraj *t, struct sc_crisZ *z);

int *sc_get_linecnt(void);
int sc_get_err_code(void);
char * sc_get_err_mesg(void);

#ifndef NO_LEVEL1		/* don't want (have) level1 routines */
#ifndef _TRAJ_C_QSORT_QUIRK	/* see traj.c for explanation */
    /* level1 routines */
    u_long sc_day96_to_sec96(int day);
    int sc_day_of_year_96(int yr, int doy);
    int sc_L1diag_cvt(struct L1CrisDiagnosticEvent *d1, 
    	struct sc_diagEvt *d, int *nevt);
    int sc_L1_eof(void);
    void sc_L1_set_day(int startday, int endday);
    void sc_L1_set_start_end_sec(int startsec, int endsec);
    int sc_L1_get_day(void);
    int sc_L1_next_cmd(struct L1CrisCommandEcho *p, int *day1996);
    int sc_L1_next_cmd_sec(struct L1CrisCommandEcho *p, int *day1996);
    int sc_L1_next_cris_evt(struct sc_crisEvt *p, int *day1996);
    int sc_L1_next_cris_evt_sec(struct sc_crisEvt *p, int *day1996);
    int sc_L1_next_diag(struct sc_diagEvt *p, int *day1996);
    int sc_L1_next_diag_sec(struct sc_diagEvt *p, int *day1996);
    int sc_L1_next_hipri_rate(struct L1CrisHighPriorityRate *p, int *day1996);
    int sc_L1_next_hipri_rate_sec(struct L1CrisHighPriorityRate *p, int *day1996);
    int sc_L1_next_hsk_raw(struct L1CrisHskp *p, int *day1996);
    int sc_L1_next_hsk_raw_sec(struct L1CrisHskp *p, int *day1996);
    int sc_L1_next_hsk(struct L1_1CrisHskp *p, int *day1996);
    int sc_L1_next_hsk_sec(struct L1_1CrisHskp *p, int *day1996);
    int sc_L1_next_lopri_rate_raw(struct L1CrisLowPriorityRate *p, int *day1996);
    int sc_L1_next_lopri_rate_raw_sec(struct L1CrisLowPriorityRate *p, int *day1996);
    int sc_L1_next_lopri_rate(struct L1_1CrisLowPriorityRate *p, int *day1996);
    int sc_L1_next_lopri_rate_sec(struct L1_1CrisLowPriorityRate *p, int *day1996);
    void sc_reject_diags_with_bad_normal_evts(int doit);
#endif /* ! _TRAJ_C_QSORT_QUIRK */
#endif /* ! NO_LEVEL1 */

#endif	/* _SOFTCRIS_H */
