/* t_cris_blobify.c - sc_cris_blobify(). */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <softcris.h>

char *Progname;

int correct_x_cen[] = {
    414, 406, 879, 399, 892, 883, 
};
int correct_y_cen[] = {
    86, 221, 120, 364, 257, 401, 
};
long correct_blob_int[] = {
    8610, 10560, 15666, 10236, 14828, 2857, 
};

double correct_Fx_cen[] = {
    413.68687572590011, 405.54791666666665, 879.25545767905021, 
    398.57581086361859, 891.60844348529804, 883.06195309765485, 
};
double correct_Fy_cen[] = {
    85.591869918699189, 220.7621212121212, 119.54308693986978, 
    363.57463853067605, 257.21176153223632, 400.83794189709488, 
};

int correct_pix_per_blob[] = {
    30, 38, 39, 30, 48, 17, 
};

u_char D[1200];

int
main(int argc, char *argv[])
{
    double Fx_cen[SC_MSU_MAXBLOBS];
    double Fy_cen[SC_MSU_MAXBLOBS];
    int gotevt = 0;
    int i;
    FILE *ifp;
    int ret;
    long blob_int[SC_MSU_MAXBLOBS];
    int blob_npix;
    char errormesg[1000];
    int pix_per_blob[SC_MSU_MAXBLOBS];
    int x_cen[SC_MSU_MAXBLOBS], y_cen[SC_MSU_MAXBLOBS];

    Progname = argv[0];

    ifp = fopen("Input/gsi.run264.evt1", "rb");
    if (ifp == (FILE *)NULL) {
	fprintf(stderr, "%s: can't fopen 'Input/gsi.run264.evt1' (%s)\n",
	    Progname, strerror(errno));
	return -1;
    }

    ret = fread(D, 1, 1192, ifp);
    if (ret != 1192) {
	fprintf(stderr, "%s: fread problem (%s)\n", Progname,
	    strerror(errno));
	return -1;
    }

    sc_cris_set_cam(SC_CAM1);

    ret = sc_cris_blobify(D, x_cen, y_cen, errormesg, NULL,
	&blob_npix, blob_int, NULL, 5, NULL, NULL);
    if (ret != 6) {
	fprintf(stderr,
	    "%s: incorrect no. of blobs (want %d, got %d)\n", 
	    Progname, 6, ret);
	return -1;
    }

#ifdef NOTDEF
    printf("int correct_x_cen[] = {\n\t");
    for (i=0; i<ret; i++) {
	printf("%d, ", x_cen[i]);
    }
    printf("\n};\n");
    printf("int correct_y_cen[] = {\n\t");
    for (i=0; i<ret; i++) {
	printf("%d, ", y_cen[i]);
    }
    printf("\n};\n");
    printf("long correct_blob_int[] = {\n\t");
    for (i=0; i<ret; i++) {
	printf("%ld, ", blob_int[i]);
    }
    printf("\n};\n");
#endif

    for (i=0; i<ret; i++) {
	if (x_cen[i] != correct_x_cen[i]) {
	    fprintf(stderr,
		"%s: bad x centroid x[%d] (want %d, got %d)\n",
		Progname, i, correct_x_cen[i], x_cen[i]);
	    return -1;
	}
	if (y_cen[i] != correct_y_cen[i]) {
	    fprintf(stderr,
		"%s: bad y centroid y[%d] (want %d, got %d)\n",
		Progname, i, correct_y_cen[i], y_cen[i]);
	    return -1;
	}
	if (blob_int[i] != correct_blob_int[i]) {
	    fprintf(stderr,
		"%s: bad intensity blob_int[%d] (want %ld, got %ld)\n",
		Progname, i, correct_blob_int[i], blob_int[i]);
	    return -1;
	}
    }

    /* now try it with floating point values and pix_per_blob */
    ret = sc_cris_blobify(D, x_cen, y_cen, errormesg, NULL,
	&blob_npix, blob_int, pix_per_blob, 5, Fx_cen, Fy_cen);
    if (ret != 6) {
	fprintf(stderr,
	    "%s: incorrect no. of blobs (want %d, got %d)\n", 
	    Progname, 6, ret);
	return -1;
    }
#ifdef NOTDEF
    printf("double correct_Fx_cen[] = {\n\t");
    for (i=0; i<ret; i++) {
	printf("%f, ", Fx_cen[i]);
    }
    printf("\n};\n");
    printf("double correct_Fy_cen[] = {\n\t");
    for (i=0; i<ret; i++) {
	printf("%f, ", Fy_cen[i]);
    }
    printf("\n};\n");
    printf("int correct_pix_per_blob[] = {\n\t");
    for (i=0; i<ret; i++) {
	printf("%d, ", pix_per_blob[i]);
    }
    printf("\n};\n");
#endif
    for (i=0; i<ret; i++) {
	if (Fx_cen[i] != correct_Fx_cen[i]) {
	    fprintf(stderr,
		"%s: bad x centroid Fx[%d] (want %f, got %f)\n",
		Progname, i, correct_Fx_cen[i], Fx_cen[i]);
	    return -1;
	}
	if (Fy_cen[i] != correct_Fy_cen[i]) {
	    fprintf(stderr,
		"%s: bad y centroid Fy[%d] (want %f, got %f)\n",
		Progname, i, correct_Fy_cen[i], Fy_cen[i]);
	    return -1;
	}
	if (pix_per_blob[i] != correct_pix_per_blob[i]) {
	    fprintf(stderr,
		"%s: bad pix_per_blob[%d] (want %d, got %d)\n",
		Progname, i, correct_pix_per_blob[i], pix_per_blob[i]);
	    return -1;
	}
    }
    return 0;
}
