/* t_cris_unpack.c - sc_cris_unpack() */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <softcris.h>

char *Progname;

struct sc_crisHodoData hodo[] = {
    879,    120,    12288,
    892,    257,    12288,
    406,    221,    8192,
    399,    364,    8192,
    414,    86,     8192,
    883,    401,    2048,
};

int
main(int argc, char *argv[])
{
    u_char buf[1000];
    struct sc_crisEvt c;
    u_char *cp;
    char filename[100];
    FILE *fp;
    int i;
    u_char nbytes;
    int ret;

    Progname = argv[0];

    strcpy(filename, "Input/cris.normal");
    fp = fopen(filename, "rb");
    if (fp == NULL) {
	fprintf(stderr, "%s: can't open '%s' (%s)\n", Progname, filename,
	    strerror(errno));
	return -1;
    }

    cp = buf;
    ret = fread(cp, 1, 1, fp);
    if (ret != 1) {
	fprintf(stderr, "%s: bad first read\n", Progname);
	return -1;
    }

    nbytes = *cp;
    cp++;
    ret = fread(cp, 1, (int)nbytes-1, fp);
    if (ret != nbytes-1) {
	fprintf(stderr, "%s: bad read got %d wanted %d\n", Progname, ret,
	    nbytes-1);
	return -1;
    }
    fclose(fp);

    ret = sc_cris_unpack(buf, &c, SC_CAM2);
    if (ret) {
	fprintf(stderr, "%s: bad return = %d\n", ret);
	return -1;
    }

#ifndef NOTDEF
    if (c.nbytes != 38 || c.R1 != 8 || c.nblobs != 6) {
	fprintf(stderr, "%s: one of the following is wrong:\n", Progname);
	fprintf(stderr, "\tnbytes was %d, should be 38\n", c.nbytes);
	fprintf(stderr, "\tR1 was %d, should be 8\n", c.R1);
	fprintf(stderr, "\tnblobs was %d, should be 6\n", c.nblobs);
	return -1;
    }
    for (i=0; i<c.nblobs; i++) {
	if (c.hodo[i].x != hodo[i].x || c.hodo[i].y != hodo[i].y ||
		c.hodo[i].I != hodo[i].I) {
	    fprintf(stderr, "%s: bad hodo value below (i=%d)\n", Progname, i);
	    fprintf(stderr, "\tx was %d, wanted %d\n", 
	    	c.hodo[i].x, hodo[i].x);
	    fprintf(stderr, "\ty was %d, wanted %d\n", 
	    	c.hodo[i].y, hodo[i].y);
	    fprintf(stderr, "\tI was %d, wanted %d\n", 
	    	c.hodo[i].I, hodo[i].I);
	    return -1;
	}
    }

#else
    printf("nbytes %d   R1 %d   nblobs %d\n", c.nbytes, c.R1, c.nblobs);
    for (i=0; i<c.nblobs; i++) {
	printf("\t%d,\t%d,\t%d\n", c.hodo[i].x, c.hodo[i].y,
	    c.hodo[i].I);
    }
#endif

    return 0;
}
