/* t_diag_unpack.c - sc_diag_unpack() */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <softcris.h>

char *Progname;

void errdisclaimer(void);

struct sc_crisHodoData hodo[] = {
        70,     460,    65536,
        314,    456,    32768,
        541,    453,    32768,
        536,    28,     24576,
        304,    29,     16384,
        775,    454,    8192,
        1010,   22,     8192,
        82,     25,     8192,
        112,    435,    4096,
        763,    27,     3072,
};

u_short stack[] = {
        3215, 3167, 3152, 3086, 
        3204, 3073, 3083, 3170, 
        3209, 3448, 3388, 3168, 
        3262, 3243, 3245, 3094, 
        3122, 3250, 3155, 3211, 
        3194, 3118, 3278, 3073, 
        3166, 3124, 3045, 3360, 
        3002, 3169, 3034, 3127, 
};

int
main(int argc, char *argv[])
{
    u_char buf[5000];
    struct sc_diagEvt c;
    u_char *cp;
    char filename[100];
    FILE *fp;
    int i, j;
    u_char nbytes;
    int ret;

    Progname = argv[0];

    strcpy(filename, "Input-src/diagevt");
    fp = fopen(filename, "rb");
    if (fp == NULL) {
	fprintf(stderr, "%s: can't open '%s' (%s)\n", Progname, filename,
	    strerror(errno));
	errdisclaimer();
	return -1;
    }

    cp = buf;
    ret = fread(cp, 1, 2480, fp);
    if (ret != 2480) {
	fprintf(stderr, "%s: bad read got %d wanted %d\n", 
	    Progname, ret, 2480);
	errdisclaimer();
	return -1;
    }

    ret = sc_diag_unpack(buf, &c, SC_CAM2);
    if (ret == -2) {
	/* may be an other-endian image */
	fprintf(stderr, "%s: I suspect you are running this\n", Progname);
	fprintf(stderr, "    test on a little endian system. If so, we\n");
	fprintf(stderr, "    can't deal with that. However, it could be\n");
	fprintf(stderr, "    a real problem of a missing or garbled\n");
	fprintf(stderr, "    START_HDR (0x%04x) at the start of the frame\n",
	    SC_START_HDR);
	errdisclaimer();
	return -1;
    } else if (ret != 0) {
	fprintf(stderr, "%s: bad return = %d\n", Progname, ret);
	errdisclaimer();
	return -1;
    }

#ifndef NOTDEF
    if (c.e.nbytes != 99 || c.e.R1 != 9 || c.e.nblobs != 10) {
	fprintf(stderr, "%s: one of the following is wrong:\n", Progname);
	fprintf(stderr, "\tnbytes was %d, should be 99\n", c.e.nbytes);
	fprintf(stderr, "\tR1 was %d, should be 9\n", c.e.R1);
	fprintf(stderr, "\tnblobs was %d, should be 10\n", c.e.nblobs);
	errdisclaimer();
	return -1;
    }
    for (i=0; i<c.e.nblobs; i++) {
	if (c.e.hodo[i].x != hodo[i].x || c.e.hodo[i].y != hodo[i].y ||
		c.e.hodo[i].I != hodo[i].I) {
	    fprintf(stderr, "%s: bad hodo value below (i=%d)\n", Progname, i);
	    fprintf(stderr, "\tx was %d, wanted %d\n", 
	    	c.e.hodo[i].x, hodo[i].x);
	    fprintf(stderr, "\ty was %d, wanted %d\n", 
	    	c.e.hodo[i].y, hodo[i].y);
	    fprintf(stderr, "\tI was %d, wanted %d\n", 
	    	c.e.hodo[i].I, hodo[i].I);
	    errdisclaimer();
	    return -1;
	}
    }
    for (i=0; i<SC_CRIS_NDET; i++) {
	if (c.s.ph[i] != stack[i]) {
	    fprintf(stderr, "%s: bad stack value (i=%d)\n", Progname, i);
	    fprintf(stderr, "\twanted %d, got %d\n", stack[i], c.s.ph[i]);
	    errdisclaimer();
	    return -1;
	}
    }

    if (*c.softp != 0x1000 || *(c.softp + 1) != 0x1404 || 
	*(c.softp + 105) != 0x0081 || *(c.softp + 202) != 0x013e) {
	    fprintf(stderr, "%s: one or more of the values below is wrong:\n", 
	    	Progname);
	    fprintf(stderr, "SOFT word 0    was %04x wanted %04x\n",
	    	*c.softp, 0x1000);
	    fprintf(stderr, "     word 1    was %04x wanted %04x\n",
	    	*(c.softp+1), 0x1404);
	    fprintf(stderr, "     word 105  was %04x wanted %04x\n",
	    	*(c.softp+105), 0x0081);
	    fprintf(stderr, "     word 202 was %04x wanted %04x\n",
	    	*(c.softp+202), 0x013e);
	    errdisclaimer();
	    return -1;
    }
#else
    printf("nbytes %d   R1 %d   nblobs %d\n", c.e.nbytes, c.e.R1, c.e.nblobs);
    for (i=0; i<c.e.nblobs; i++) {
	printf("\t%d,\t%d,\t%d,\n", c.e.hodo[i].x, c.e.hodo[i].y,
	    c.e.hodo[i].I);
    }
    printf("raw\n");
    for (i=0; i<SC_CRIS_NDET; ) {
	printf("\t");
	for (j=0; j<4; j++) {
	    printf("%d, ", c.s.ph[i]);
	    i++;
	}
	printf("\n");
    }
    printf("\n");
    printf("0x%04x 0x%04x 0x%04x 0x%04x\n", *(c.softp),
    	*(c.softp + 1), *(c.softp + 105), *(c.softp + 202));
#endif

    return 0;
}

void
errdisclaimer(void)
{
#ifdef NOTDEF
    fprintf(stderr,"NOTE: this test program sometimes falsely reports an\n");
    fprintf(stderr,"      error. Try running it again just to be sure.\n");
#endif
}
