/* get_traj_mb_test.c - sc_get_traj_mb() */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <sys/stat.h>
#include <softcris.h>

char *Progname;

#define NTRAJ	20

double Mx[] = {
	0.1037165, 0.1038173, 0.1038741, 0.1039749, 0.0999415, 0.1000422,
	0.1000991, 0.1001999, 0.1014874, 0.1014077, 0.1029746, 0.1030754,
	0.1031322, 0.1032330, 0.0991996, 0.0993003, 0.0993572, 0.0994580,
};
double Bx[] = {
	87.7018391, 87.7006430, 87.6999674, 87.6987713, 87.9285018,
	87.9273057, 87.9266301, 87.9254339, 64.3437760, 64.3447228,
	88.0333335, 88.0321373, 88.0314617, 88.0302656, 88.2599962,
	88.2588000, 88.2581244, 88.2569283,
};
double My[] = {
	-1.4837586, -1.4930583, -1.4837565, -1.4930562, -1.4837816,
	-1.4930813, -1.4837795, -1.4930792, -1.4839690, -1.4932582,
	-1.4837669, -1.4930666, -1.4837648, -1.4930645, -1.4837899,
	-1.4930896, -1.4837878, -1.4930875,
};
double By[] = {
	134.4913875, 134.5999430, 134.4913634, 134.5999189, 134.4927627,
	134.6013182, 134.4927386, 134.6012941, 134.3261827, 134.4346163,
	134.4951047, 134.6036602, 134.4950806, 134.6036361, 134.4964799,
	134.6050354, 134.4964558, 134.6050113,
};
int
main(int argc, char *argv[])
{
    double bx[NTRAJ], by[NTRAJ];
    struct sc_crisEvt c;
    char filename[100];
    FILE *fp;
    int i;
    double mx[NTRAJ], my[NTRAJ];
    u_char nbytes;
    int ntraj = NTRAJ;
    int ret;
    struct stat statbuf;
    struct sc_Coords_Traj t[NTRAJ];
    char tmpstr[20];

    int dbg = 0;

    if (argc > 1) {
	if (strcmp(argv[1], "-d") == 0) {
	    dbg = 1;
	}
    }

    Progname = argv[0];

    /* crisevt.357.100 is the sc_crisEvt struct for event 100 from day 357
     * of year 1997 */
    strcpy(filename, "Input/crisevt.357.100");
    ret = stat(filename, &statbuf);
    if (ret != 0) {
	fprintf(stderr, "%s: can't stat '%s'\n", Progname, filename);
	return -1;
    }
    fp = fopen(filename, "rb");
    if (fp == NULL) {
	fprintf(stderr, "%s: can't open '%s' (%s)\n", Progname, filename,
	    strerror(errno));
	return -1;
    }

    ret = fread(&c, 1, statbuf.st_size, fp);
    if (ret != statbuf.st_size) {
	fprintf(stderr, "%s: bad read\n", Progname);
	return -1;
    }

    sc_cris_set_cam(SC_CAM2);
    sc_traj_select_method(8);
    ret = sc_cris_traj(&c, t, &ntraj);

    if (ret < 0) {
	fprintf(stderr, "%s: bad return from sc_cris_traj() = %d\n",
	    Progname, ret);
	fprintf(stderr, "    \"%s\"\n", sc_get_err_mesg());
	return -1;
    }

    sc_get_traj_mb(mx, bx, my, by); 


    if (!dbg) {
	ret = 0;
	for (i=0; i<ntraj; i++) {
	    sprintf(tmpstr, "mx[%d]", i);
	    ret += checkit(Mx[i], mx[i], tmpstr);
	    sprintf(tmpstr, "bx[%d]", i);
	    ret += checkit(Bx[i], bx[i], tmpstr);
	    sprintf(tmpstr, "my[%d]", i);
	    ret += checkit(My[i], my[i], tmpstr);
	    sprintf(tmpstr, "by[%d]", i);
	    ret += checkit(By[i], by[i], tmpstr);
	}
	if (ret) {
	    return -1;
	}
    } else {
	printf("ntraj: %d\n", ntraj);
	printf("double Mx[] = {\n");
	for(i=0; i<ntraj; i++) {
	    printf("\t%.7f,\n", mx[i]);
	}
	printf("};\n");
	printf("double Bx[] = {\n");
	for(i=0; i<ntraj; i++) {
	    printf("\t%.7f,\n", bx[i]);
	}
	printf("};\n");
	printf("double My[] = {\n");
	for(i=0; i<ntraj; i++) {
	    printf("\t%.7f,\n", my[i]);
	}
	printf("};\n");
	printf("double By[] = {\n");
	for(i=0; i<ntraj; i++) {
	    printf("\t%.7f,\n", by[i]);
	}
	printf("};\n");
    }

    return 0;
}

int
checkit(want, got, phrase)
    float want, got;
    char *phrase;
{
    static float epsilon = 0.0003;
    if (want - epsilon <= got && got <= want + epsilon) {
	return 0;
    } else {
	fprintf(stderr," %s: got %f, wanted %f\n", phrase, got, want);
	return 1;
    }
}

