/* get_traj_mb_test.c - sc_get_traj_mb() */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <softcris.h>

char *Progname;
double Mx = 0.00914614779488893796;
double Bx = 82.73154578735824316027;
double My = 0.00513904231375978716;
double By = 203.65033086189984601333;

int
main(int argc, char *argv[])
{
    u_char buf[1000];
    double bx, by;
    struct sc_crisEvt c;
    struct sc_softCoords coords;
    u_char *cp;
    char filename[100];
    FILE *fp;
    int i;
    double mx, my;
    u_char nbytes;
    int ret;
    struct sc_softTraj traj;

    int dbg = 0;

    if (argc > 1) {
	if (strcmp(argv[1], "-d") == 0) {
	    dbg = 1;
	}
    }

    Progname = argv[0];

    strcpy(filename, "Input/cris.normal");
    fp = fopen(filename, "rb");
    if (fp == NULL) {
	fprintf(stderr, "%s: can't open '%s' (%s)\n", Progname, filename,
	    strerror(errno));
	return -1;
    }

    cp = buf;
    ret = fread(cp, 1, 1, fp);
    if (ret != 1) {
	fprintf(stderr, "%s: bad first read\n", Progname);
	return -1;
    }

    nbytes = *cp;
    cp++;
    ret = fread(cp, 1, (int)nbytes-1, fp);
    if (ret != nbytes-1) {
	fprintf(stderr, "%s: bad read got %d wanted %d\n", Progname, ret,
	    nbytes-1);
	return -1;
    }
    fclose(fp);

    sc_traj_select_method(4);
    ret = sc_cris_unpack(buf, &c, SC_CAM2);
    if (ret) {
	fprintf(stderr, "%s: bad return = %d\n", ret);
	return -1;
    }

    ret = sc_cris_coords(&c, &coords, SC_CAM2);
    if (ret) {
	fprintf(stderr, "%s: bad return from sc_cris_coords() = %d\n",
	    Progname, ret);
	return -1;
    }

    ret = sc_trajectory(&coords, &traj);
    if (ret) {
	fprintf(stderr, "%s: bad return from sc_trajectory() = %d\n",
	    Progname, ret);
	return -1;
    }

    sc_get_traj_mb(&mx, &bx, &my, &by); 


    if (!dbg) {
	if (mx != Mx || bx != Bx || my != My || by != By) {
	    fprintf(stderr,"One of the following is wrong:\n");
	    fprintf(stderr,"\tmx: got %.20f wanted %.20f\n", mx, Mx);
	    fprintf(stderr,"\tbx: got %.20f wanted %.20f\n", bx, Bx);
	    fprintf(stderr,"\tmy: got %.20f wanted %.20f\n", my, My);
	    fprintf(stderr,"\tby: got %.20f wanted %.20f\n", by, By);
	    return -1;
	}
    } else {
	printf("double Mx = %.20f;\n", mx);
	printf("double Bx = %.20f;\n", bx);
	printf("double My = %.20f;\n", my);
	printf("double By = %.20f;\n", by);
    }

    return 0;
}
