/* gsi_next_evt_test.c - sc_gsi_next_evt() */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <softcris.h>

char *Progname;

struct sc_crisHodoData h0[] = {
        879, 120, 12288,
        892, 257, 12288,
        406, 221, 8192,
        399, 364, 8192,
        414, 86, 8192,
        883, 401, 2048,
};

struct sc_crisHodoData h1[] = {
        910, 256, 16384,
        903, 119, 12288,
        302, 86, 8192,
        288, 365, 8192,
        294, 221, 8192,
        906, 401, 8192,
};

struct sc_crisHodoData h2[] = {
        914, 256, 16384,
        906, 401, 12288,
        487, 86, 8192,
        903, 119, 8192,
        480, 221, 8192,
        476, 364, 2048,
        956, 120, 1024,
        902, 109, 768,
        881, 400, 512,
};

int
main(int argc, char *argv[])
{
    char filename[100];
    FILE *fp;
    int i, j, k;
    struct sc_ch2 ch2;
    struct sc_diagEvt d;
    int ret;

    Progname = argv[0];

    strcpy(filename, "Input/gsi.264.frag");
    fp = fopen(filename, "rb");
    if (fp == NULL) {
	fprintf(stderr, "%s: can't open '%s' (%s)\n", Progname, filename,
	    strerror(errno));
	return -1;
    }

#ifndef NOTDEF
    for (k=0; k<3; k++) {
	ret = sc_gsi_next_evt(fp, &ch2, &d, filename);
	if (ret != SC_GSI_OK) {
	    fprintf(stderr, "OOPS! ret is not SC_GSI_OK (0), but %d\n", ret);
	    return -1;
	}

	if (*d.softp != SC_START_HDR) {
	    fprintf(stderr, 
	    	"SOFT start is 0x%04x, should be 0x%04x (SC_START_HDR)\n", 
	    	*d.softp, SC_START_HDR);
	    return -1;
	}

	if (k != 1) {
	    continue;
	}

	if (d.e.nbytes != 38) {
	    fprintf(stderr,"%s: (k=%d) bad nbytes. wanted 38, got %d\n",
		Progname, k, d.e.nbytes);
	    return -1;
	}

	for (j=0; j<d.e.nblobs; j++) {
	    if (d.e.hodo[j].x != h1[j].x || d.e.hodo[j].y != h1[j].y ||
		    d.e.hodo[j].I != h1[j].I) {
		fprintf(stderr,"%s: (j=%d) one of these is wrong:\n",
		    Progname, j);
		fprintf(stderr,"\tx (got %d, want %d)\n", d.e.hodo[j].x, h1[j].x);
		fprintf(stderr,"\ty (got %d, want %d)\n", d.e.hodo[j].y, h1[j].y);
		fprintf(stderr,"\tI (got %d, want %d)\n", d.e.hodo[j].I, h1[j].I);
		return -1;
	    }
	}
    }

    ret = sc_gsi_next_evt(fp, &ch2, &d, filename);
    if (ret != SC_GSI_EOF) {
	fprintf(stderr,"%s: expected %d, but got ret = %d\n", Progname,
	    SC_GSI_EOF, ret);
	return -1;
    }
#else
    for (k=0; k<5; k++) {
	ret = sc_gsi_next_evt(fp, &ch2, &d, filename);
	if (ret != SC_GSI_OK) {
	    fprintf(stderr, "OOPS! ret is not SC_GSI_OK (0), but %d\n", ret);
	    return -1;
	}
	printf("\n%d) size = %d\n", k, d.e.nbytes);
	printf("\nstruct sc_crisHodoData h%d[] = {\n", k);
	for (j=0; j<d.e.nblobs; j++) {
	    printf("\t%d, %d, %u,\n", d.e.hodo[j].x,
		d.e.hodo[j].y, d.e.hodo[j].I);
	}
	printf("};\n");
	if (*d.softp != SC_START_HDR) {
	    printf("SOFT start is 0x%04x\n", *d.softp);
	} else {
	    printf("SOFT OK\n");
	}
    }
#endif

    return 0;
}
