/* layer_maxI_test.c - sc_layer_maxI() */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <sys/stat.h>
#include <softcris.h>

char *Progname;

#define NCHECK	3
int Ntraj = 18;
long Intensity[] = { 4096L, 8192L, 8192L, 8192L, 8192L, 6144L,};

#define NTRAJ	20

int
main(int argc, char *argv[])
{
    struct sc_crisEvt c;
    char filename[100];
    FILE *fp;
    long *I;
    int i;
    int j;
    int ntraj = NTRAJ;
    int ret;
    struct stat statbuf;
    struct sc_Coords_Traj t[NTRAJ];
    char tmpstr[20];

    int dbg = 0;

    if (argc > 1) {
	if (strcmp(argv[1], "-d") == 0) {
	    dbg = 1;
	}
    }

    Progname = argv[0];

    /* crisevt.357.100 is the sc_crisEvt struct for event 100 from day 357
     * of year 1997 */
    strcpy(filename, "Input/crisevt.357.100");
    ret = stat(filename, &statbuf);
    if (ret != 0) {
	fprintf(stderr, "%s: can't stat '%s'\n", Progname, filename);
	return -1;
    }
    fp = fopen(filename, "rb");
    if (fp == NULL) {
	fprintf(stderr, "%s: can't open '%s' (%s)\n", Progname, filename,
	    strerror(errno));
	return -1;
    }

    ret = fread(&c, 1, statbuf.st_size, fp);
    if (ret != statbuf.st_size) {
	fprintf(stderr, "%s: bad read\n", Progname);
	return -1;
    }

    sc_cris_set_cam(SC_CAM2);
    sc_traj_select_method(8);
    ret = sc_cris_traj(&c, t, &ntraj);
    I = sc_layer_maxI();

    if (ret < 0) {
	fprintf(stderr, "%s: bad return from sc_cris_traj() = %d\n",
	    Progname, ret);
	fprintf(stderr, "    \"%s\"\n", sc_get_err_mesg());
	return -1;
    }

    if (!dbg) {
	for (i=0; i<SC_NLAYERS; i++) {
	    if (I[i] != Intensity[i]) {
		fprintf(stderr, "Intensity[%d] = %ld, should be %ld\n",
		    i, I[i], Intensity[i]);
		return -1;
	    }
	}
    } else {
	printf("long Intensity[] = {");
	for (i=0; i<SC_NLAYERS; i++) {
	    printf(" %ldL,", I[i]);
	}
	printf("};\n");
    }

    return 0;
}
