/* t_lu_pos_extra.c - sc_lu_pos_extra(). */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <softcris.h>

char *Progname;

struct {
    double x;
    double y;
    int tab;
    int layer;
    double pos;
    double ppos;
    int player;
    int ptab;
    double px;
    double py;
    double npos;
    int nlayer;
    int ntab;
    double nx;
    double ny;
} vals[] = {
    100.00, 100.00, 27, 2, 38500.00, 30979.00, 2, 26, 100.00, 99.00, 
    	46053.00, 2, 28, 100.00, 105.00,
    200.00, 100.00, 27, 2, 40100.00, 32579.00, 2, 26, 200.00, 99.00, 
    	47653.00, 2, 28, 200.00, 105.00,
    300.00, 105.00, 28, 2, 49253.00, 41700.00, 2, 27, 300.00, 101.00, 
    	56838.00, 2, 29, 300.00, 107.00,
    400.00, 119.00, 33, 3, 1905.00, 80451.00, 2, 32, 400.00, 116.00, 
    	9538.00, 3, 34, 400.00, 122.00,
    500.00, 150.00, 43, 3, 76055.00, 68422.00, 3, 42, 500.00, 148.00, 
    	7285.00, 4, 44, 500.00, 156.00,
    500.00, 220.00, 64, 5, 69002.00, 61320.00, 5, 63, 500.00, 217.00, 
    	76731.00, 5, 65, 500.00, 223.00,
    500.00, 20.00, 2, 0, 22320.00, 14794.00, 0, 1, 500.00, 18.00, 
    	29871.00, 0, 3, 500.00, 25.00,
    400.00, 150.00, 43, 3, 77655.00, 70022.00, 3, 42, 400.00, 148.00, 
    	5685.00, 4, 44, 400.00, 155.00,
};

#define NVALS (sizeof(vals)/sizeof(vals[0]))

int 
main(int argc, char *argv[])
{
    double d;
    int i;
    int id;
    double pos;
    int ret;

    double ppos, px, py, npos, nx, ny;
    int player, ptab, nlayer, ntab;

    Progname = argv[0];

    ret = sc_lu_read_table("../Maps/lut.cam2.msu1195");
    if (ret != 0) {
	fprintf(stderr, "%s: sc_lu_read_table (A) ret was %d, wanted 0\n",
	    Progname, ret);
	return -1;
    }
    id = ret;

    for (i=0; i<NVALS; i++) {
	ret = sc_lu_pos_extra(id, &pos, vals[i].layer, vals[i].tab, vals[i].x,
	    vals[i].y, &ppos, &player, &ptab, &px, &py, &npos, &nlayer,
	    &ntab, &nx, &ny);
	if (ret) {
	    fprintf(stderr,"%s: sc_lu_pos bad return = %d (i=%d)\n", 
	    	Progname, ret, i);
	    return -1;
	}
	d = ceil(pos);
#ifndef NOTDEF
	if (vals[i].pos != d) {
	    fprintf(stderr,"%s: (i=%d) wanted pos=%f but got %f\n",
	    	Progname, i, vals[i].pos, d);
	    return -1;
	}
	if (vals[i].ppos != ceil(ppos) || vals[i].player != player ||
		vals[i].ptab != ptab || vals[i].px != ceil(px) || 
		vals[i].py != ceil(py) ||
		vals[i].npos != ceil(npos) || vals[i].nlayer != nlayer ||
		vals[i].ntab != ntab || vals[i].nx != ceil(nx) || 
		vals[i].ny != ceil(ny)) {
	    fprintf(stderr, "%s: (i=%d) one of these values is wrong:\n", 
	    	Progname, i);
	    fprintf(stderr, "\tppos: want = %.2f, got = %.2f\n",
		vals[i].ppos, ceil(ppos));
	    fprintf(stderr, "\tplayer: want = %d, got = %d\n",
		vals[i].player, player);
	    fprintf(stderr, "\tptab: want = %d, got = %d\n",
		vals[i].ptab, ptab);
	    fprintf(stderr, "\tpx: want = %.2f, got = %.2f\n",
		vals[i].px, ceil(px));
	    fprintf(stderr, "\tpy: want = %.2f, got = %.2f\n",
		vals[i].py, ceil(py));
	    fprintf(stderr, "\tnpos: want = %.2f, got = %.2f\n",
		vals[i].npos, ceil(npos));
	    fprintf(stderr, "\tnlayer: want = %d, got = %d\n",
		vals[i].nlayer, nlayer);
	    fprintf(stderr, "\tntab: want = %d, got = %d\n",
		vals[i].ntab, ntab);
	    fprintf(stderr, "\tnx: want = %.2f, got = %.2f\n",
		vals[i].nx, ceil(nx));
	    fprintf(stderr, "\tny: want = %.2f, got = %.2f\n",
		vals[i].ny, ceil(ny));
	    return -1;
	}
#else
	printf("%.2f, %.2f, %d, %d, %.2f, ", vals[i].x, vals[i].y,
	    vals[i].tab, vals[i].layer, d);
	printf("%.2f, %d, %d, %.2f, %.2f, ", ceil(ppos), player, ptab,
	    ceil(px), ceil(py));
	printf("%.2f, %d, %d, %.2f, %.2f,\n", ceil(npos), nlayer, ntab, 
	    ceil(nx), ceil(ny));
#endif
    }

    return 0;
}
