/* t_msu_blobify.c - sc_msu_blobify(). */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <softcris.h>

char *Progname;

int correct_x_cen[] = {
    91, 80, 996, 510, 1005, 172, 507, 982, 
};
 
int correct_y_cen[] = {
    63, 72, 201, 140, 212, 286, 439, 344, 
};
 
long correct_blob_int[] = {
    890, 3407, 7542, 5066, 2933, 3388, 4202, 8642, 
};
 
double correct_Fx_cen[] = {
    91.339325842696624, 80.413266803639559, 995.715460090161741, 
    509.838136596920663, 1005.286737129219205, 171.554309327036606, 
    506.645406949071855, 981.626706780837821, 
};
 
double correct_Fy_cen[] = {
    63.287640449438200, 72.273554446727331, 200.603288252452927, 
    139.900118436636404, 211.746334810773959, 286.109208972845352, 
    439.287006187529755, 344.277482064336937, 
};
 
int correct_pix_per_blob[] = {
    7, 23, 38, 24, 23, 21, 25, 59, 
};

int
main(int argc, char *argv[])
{
    struct sc_msuEvt e;
    double Fx_cen[SC_MSU_MAXBLOBS];
    double Fy_cen[SC_MSU_MAXBLOBS];
    int gotevt = 0;
    int i;
    FILE *ifp;
    int ret;
    long blob_int[SC_MSU_MAXBLOBS];
    int blob_npix;
    char errormesg[1000];
    int pix_per_blob[SC_MSU_MAXBLOBS];
    int x_cen[SC_MSU_MAXBLOBS], y_cen[SC_MSU_MAXBLOBS];
    int totpix;

    Progname = argv[0];

    ifp = fopen("Input/msu.200000", "rb");
    if (ifp == (FILE *)NULL) {
	fprintf(stderr, "%s: can't fopen 'Input/msu.200000' (%s)\n",
	    Progname, strerror(errno));
	return -1;
    }

    ret = sc_msu_init(ifp, NULL, 0, Progname);
    if (ret) {
	fprintf(stderr, "%s says:\n", Progname);
	fprintf(stderr, 
	 "sc_msu_init(ifp, NULL, 0, %s) returned %d (wanted 0)\n",
	    Progname, ret);
	return -1;
    }

    while (sc_msu_next_evt(&e) == 0) {
	if (e.abs == 200075) {
	    gotevt = 1;
	    ret = sc_msu_blobify(&e, x_cen, y_cen, errormesg, NULL,
	    	&blob_npix, blob_int, pix_per_blob, 5, Fx_cen, Fy_cen, 2);
	    totpix = sc_blob_totpix();
#ifndef NOTDEF
	    if (ret != 8) {
		fprintf(stderr,
		    "%s: incorrect no. of blobs (want %d, got %d)\n", 
		    Progname, 8, ret);
		return -1;
	    }
	    if (totpix != 229) {
		fprintf(stderr,"%s: bad totpix; was %d, should be %d\n",
		    Progname, totpix, 229);
		return -1;
	    }
	    for (i=0; i<ret; i++) {
		if (x_cen[i] != correct_x_cen[i]) {
		    fprintf(stderr,
		    	"%s: bad x centroid x[%d] (want %d, got %d)\n",
			Progname, i, correct_x_cen[i], x_cen[i]);
		    return -1;
		}
		if (y_cen[i] != correct_y_cen[i]) {
		    fprintf(stderr,
		    	"%s: bad y centroid y[%d] (want %d, got %d)\n",
			Progname, i, correct_y_cen[i], y_cen[i]);
		    return -1;
		}
		if (blob_int[i] != correct_blob_int[i]) {
		    fprintf(stderr,
		    	"%s: bad intensity blob_int[%d] (want %ld, got %ld)\n",
			Progname, i, correct_blob_int[i], blob_int[i]);
		    return -1;
		}
		if (Fx_cen[i] != correct_Fx_cen[i]) {
		    fprintf(stderr,
			"%s: bad x centroid Fx[%d] (want %f, got %f)\n",
			Progname, i, correct_Fx_cen[i], Fx_cen[i]);
		    return -1;
		}
		if (Fy_cen[i] != correct_Fy_cen[i]) {
		    fprintf(stderr,
			"%s: bad y centroid Fy[%d] (want %f, got %f)\n",
			Progname, i, correct_Fy_cen[i], Fy_cen[i]);
		    return -1;
		}
		if (pix_per_blob[i] != correct_pix_per_blob[i]) {
		    fprintf(stderr,
			"%s: bad pix_per_blob[%d] (want %d, got %d)\n",
			Progname, i, correct_pix_per_blob[i], pix_per_blob[i]);
		    return -1;
		}
	    }
#else
	    printf("int correct_x_cen[] = {\n    ");
	    for (i=0; i<ret; i++) {
		printf("%d, ", x_cen[i]);
	    }
	    printf("\n};\n\n");

	    printf("int correct_y_cen[] = {\n    ");
	    for (i=0; i<ret; i++) {
		printf("%d, ", y_cen[i]);
	    }
	    printf("\n};\n\n");

	    printf("long correct_blob_int[] = {\n    ");
	    for (i=0; i<ret; i++) {
		printf("%ld, ", blob_int[i]);
	    }
	    printf("\n};\n\n");

	    printf("double correct_Fx_cen[] = {\n    ");
	    for (i=0; i<ret; i++) {
		printf("%.15f, ", Fx_cen[i]);
	    }
	    printf("\n};\n\n");

	    printf("double correct_Fy_cen[] = {\n    ");
	    for (i=0; i<ret; i++) {
		printf("%.15f, ", Fy_cen[i]);
	    }
	    printf("\n};\n\n");

	    printf("int correct_pix_per_blob[] = {\n    ");
	    for (i=0; i<ret; i++) {
		printf("%d, ", pix_per_blob[i]);
	    }
	    printf("\n};\n\n");

#endif	/* NOTDEF */
	}
    }

    if (!gotevt) {
	fprintf(stderr,"%s: never got event 200075!\n");
	return -1;
    }

    return 0;
}
