/* t_msu_blob_sort.c - sc_msu_blob_sort() for blob select method 1. */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <softcris.h>

char *Progname;

float val[] = {
    239.00, 149.00, 200.00, 149.00, 168.00, 149.00,
};

int
main(int argc, char *argv[])
{
    struct sc_msuEvt e;
    struct sc_softCoords c1;
    struct sc_softCoords c2;
    double Fx_cen[SC_MSU_MAXBLOBS];
    double Fy_cen[SC_MSU_MAXBLOBS];
    int gotevt = 0;
    int i;
    FILE *ifp;
    int ret;
    long blob_int[SC_MSU_MAXBLOBS];
    int blob_npix;
    char errormesg[1000];
    int pix_per_blob[SC_MSU_MAXBLOBS];
    int x_cen[SC_MSU_MAXBLOBS], y_cen[SC_MSU_MAXBLOBS];

    Progname = argv[0];

    ifp = fopen("Input/msu.200000", "rb");
    if (ifp == (FILE *)NULL) {
	fprintf(stderr, "%s: can't fopen 'Input/msu.200000' (%s)\n",
	    Progname, strerror(errno));
	return -1;
    }

    (void)sc_blob_select_method(1);

    ret = sc_msu_init(ifp, NULL, 0, Progname);
    if (ret) {
	fprintf(stderr, "%s says:\n", Progname);
	fprintf(stderr, 
	 "sc_msu_init(ifp, NULL, 0, %s) returned %d (wanted 0)\n",
	    Progname, ret);
	return -1;
    }

    while (sc_msu_next_evt(&e) == 0) {
	if (e.abs == 200072) {
	    gotevt = 1;

	    ret = sc_msu_blobify(&e, x_cen, y_cen, errormesg, NULL,
	    	&blob_npix, blob_int, pix_per_blob, 2, Fx_cen, Fy_cen,
		SC_CAM1);

	    for (i=0; i<ret; i++) {
		/* replace the bad pre-calculated centroids with good ones */
		e.blobx[i] = x_cen[i];
		e.bloby[i] = y_cen[i];
		e.blobI[i] = blob_int[i];
	    }
	    e.nblobs = ret;

	    ret = sc_msu_blobify(&e, x_cen, y_cen, errormesg, NULL,
	    	&blob_npix, blob_int, pix_per_blob, 2, Fx_cen, Fy_cen,
		SC_CAM2);

	    for (i=0; i<ret; i++) {
		/* replace the bad pre-calculated centroids with good ones */
		e.blobxb[i] = x_cen[i];
		e.blobyb[i] = y_cen[i];
		e.blobIb[i] = blob_int[i];
	    }
	    e.nblobsb = ret;

	    ret = sc_msu_coords(&e, &c1, &c2);
#ifndef NOTDEF
	    if (ceil(c1.x1) != val[0] || ceil(c1.y1) != val[1] || 
		    ceil(c1.x2) != val[2] || ceil(c1.y2) != val[3] || 
		    ceil(c1.x3) != val[4] || ceil(c1.y3) != val[5]) {
		fprintf(stderr, "%s: something wrong:\n", Progname);
		fprintf(stderr, "c1 x1: wanted %.2f, got %.2f\n", 
		    val[0], ceil(c1.x1));
		fprintf(stderr, "c1 y1: wanted %.2f, got %.2f\n", 
		    val[1], ceil(c1.y1));
		fprintf(stderr, "c1 x2: wanted %.2f, got %.2f\n", 
		    val[2], ceil(c1.x2));
		fprintf(stderr, "c1 y2: wanted %.2f, got %.2f\n", 
		    val[3], ceil(c1.y2));
		fprintf(stderr, "c1 x3: wanted %.2f, got %.2f\n", 
		    val[4], ceil(c1.x3));
		fprintf(stderr, "c1 y3: wanted %.2f, got %.2f\n", 
		    val[5], ceil(c1.y3));
		return -1;
	    }
	    if (ceil(c2.x1) != val[0] || ceil(c2.y1) != val[1] || 
		    ceil(c2.x2) != val[2] || ceil(c2.y2) != val[3] || 
		    ceil(c2.x3) != val[4] || ceil(c2.y3) != val[5]) {
		fprintf(stderr, "%s: something wrong:\n", Progname);
		fprintf(stderr, "c2 x1: wanted %.2f, got %.2f\n", 
		    val[0], ceil(c2.x1));
		fprintf(stderr, "c2 y1: wanted %.2f, got %.2f\n", 
		    val[1], ceil(c2.y1));
		fprintf(stderr, "c2 x2: wanted %.2f, got %.2f\n", 
		    val[2], ceil(c2.x2));
		fprintf(stderr, "c2 y2: wanted %.2f, got %.2f\n", 
		    val[3], ceil(c2.y2));
		fprintf(stderr, "c2 x3: wanted %.2f, got %.2f\n", 
		    val[4], ceil(c2.x3));
		fprintf(stderr, "c2 y3: wanted %.2f, got %.2f\n", 
		    val[5], ceil(c2.y3));
		return -1;
	    }
#else
	    printf("\nHere are the real positions. Both cameras should be the same.\n");
	    printf("\nC1\n\t%.2f,\t%.2f,\t%.2f,\t%.2f,\t%.2f,\t%.2f,\n",
		ceil(c1.x1), ceil(c1.y1), ceil(c1.x2), 
		ceil(c1.y2), ceil(c1.x3), ceil(c1.y3));
	    printf("\nC2\n\t%.2f,\t%.2f,\t%.2f,\t%.2f,\t%.2f,\t%.2f,\n",
		ceil(c2.x1), ceil(c2.y1), ceil(c2.x2), 
		ceil(c2.y2), ceil(c2.x3), ceil(c2.y3));

	    printf("Just for fun, here are the fiber numbers. The 2 cameras should be different.\n");
	    printf("\nC1\n\t%.2f,\t%.2f,\t%.2f,\t%.2f,\t%.2f,\t%.2f,\n",
	    	c1.fiber[0], c1.fiber[1], c1.fiber[2], c1.fiber[3], c1.fiber[4],
		c1.fiber[5]);
	    printf("\nC2\n\t%.2f,\t%.2f,\t%.2f,\t%.2f,\t%.2f,\t%.2f,\n",
	    	c2.fiber[0], c2.fiber[1], c2.fiber[2], c2.fiber[3], c2.fiber[4],
		c2.fiber[5]);
#endif
	}
    }

    if (!gotevt) {
	fprintf(stderr,"%s: never got event 200055!\n");
	return -1;
    }

    return 0;
}
