/* t_msu_blob_sort.c - sc_msu_blob_sort(). */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <softcris.h>

char *Progname;

struct sc_softTraj Tr1 = {
        238.485764,     148.237839,
        199.434952,     148.305984,
        167.304764,     148.520325,
        166.811612,     148.490248,
        44.359212,      -0.223577,
        11.545328,      0.282149,
};
 
 
struct sc_softTraj Tr2 = {
        238.184204,     148.431778,
        199.473740,     148.196487,
        167.023361,     148.244324,
        166.648923,     148.195874,
        44.343763,      0.156780,
        1.771599,       0.705540,
};

int
main(int argc, char *argv[])
{
    struct sc_msuEvt e;
    struct sc_softCoords c1;
    struct sc_softCoords c2;
    int dbg = 0;
    double Fx_cen[SC_MSU_MAXBLOBS];
    double Fy_cen[SC_MSU_MAXBLOBS];
    int gotevt = 0;
    int i;
    FILE *ifp;
    int ret;
    struct sc_softTraj t1, t2;
    long blob_int[SC_MSU_MAXBLOBS];
    int blob_npix;
    char errormesg[1000];
    int pix_per_blob[SC_MSU_MAXBLOBS];
    int x_cen[SC_MSU_MAXBLOBS], y_cen[SC_MSU_MAXBLOBS];

    if (argc > 1) {
	if (strcmp(argv[1], "-d") == 0) {
	    dbg = 1;
	}
    }

    Progname = argv[0];

    ifp = fopen("Input/msu.200000", "rb");
    if (ifp == (FILE *)NULL) {
	fprintf(stderr, "%s: can't fopen 'Input/msu.200000' (%s)\n",
	    Progname, strerror(errno));
	return -1;
    }

    sc_traj_select_method(4);

    ret = sc_msu_init(ifp, NULL, 0, Progname);
    if (ret) {
	fprintf(stderr, "%s says:\n", Progname);
	fprintf(stderr, 
	 "sc_msu_init(ifp, NULL, 0, %s) returned %d (wanted 0)\n",
	    Progname, ret);
	return -1;
    }

    while (sc_msu_next_evt(&e) == 0) {
	if (e.abs == 200072) {
	    gotevt = 1;
	    ret = sc_msu_blobify(&e, x_cen, y_cen, errormesg, NULL,
	    	&blob_npix, blob_int, pix_per_blob, 2, Fx_cen, Fy_cen,
		SC_CAM1);

	    for (i=0; i<ret; i++) {
		/* replace the bad pre-calculated centroids with good ones */
		e.blobx[i] = x_cen[i];
		e.bloby[i] = y_cen[i];
		e.blobI[i] = blob_int[i];
	    }
	    e.nblobs = ret;

	    ret = sc_msu_blobify(&e, x_cen, y_cen, errormesg, NULL,
	    	&blob_npix, blob_int, pix_per_blob, 2, Fx_cen, Fy_cen,
		SC_CAM2);

	    for (i=0; i<ret; i++) {
		/* replace the bad pre-calculated centroids with good ones */
		e.blobxb[i] = x_cen[i];
		e.blobyb[i] = y_cen[i];
		e.blobIb[i] = blob_int[i];
	    }
	    e.nblobsb = ret;

	    ret = sc_msu_coords(&e, &c1, &c2);
	    if (ret) {
		fprintf(stderr,"bad call to sc_msu_coords = %d\n",
		    ret);
		return -1;
	    }
	    ret = sc_trajectory(&c1, &t1);
	    if (ret) {
		fprintf(stderr,"bad ret on 1st call to sc_traj = %d\n",
		    ret);
		return -1;
	    }
	    ret = sc_trajectory(&c2, &t2);
	    if (ret) {
		fprintf(stderr,"bad ret on 2nd call to sc_traj = %d\n",
		    ret);
		return -1;
	    }
	    if (!dbg) {
		ret = checkit(Tr1.layer1x, t1.layer1x, "layer1x");
		ret += checkit(Tr1.layer1y, t1.layer1y, "layer1y");
		ret += checkit(Tr1.layer2x, t1.layer2x, "layer2x");
		ret += checkit(Tr1.layer2y, t1.layer2y, "layer2y");
		ret += checkit(Tr1.layer3x, t1.layer3x, "layer3x");
		ret += checkit(Tr1.layer3y, t1.layer3y, "layer3y");
		ret += checkit(Tr1.stackx, t1.stackx, "stackx");
		ret += checkit(Tr1.stacky, t1.stacky, "stacky");
		ret += checkit(Tr1.theta, t1.theta, "theta");
		ret += checkit(Tr1.phi, t1.phi, "phi");
		ret += checkit(Tr1.chisqx, t1.chisqx, "chisqx");
		ret += checkit(Tr1.chisqy, t1.chisqy, "chisqy");
		ret += checkit(Tr2.layer1x, t2.layer1x, "layer1x");
		ret += checkit(Tr2.layer1y, t2.layer1y, "layer1y");
		ret += checkit(Tr2.layer2x, t2.layer2x, "layer2x");
		ret += checkit(Tr2.layer2y, t2.layer2y, "layer2y");
		ret += checkit(Tr2.layer3x, t2.layer3x, "layer3x");
		ret += checkit(Tr2.layer3y, t2.layer3y, "layer3y");
		ret += checkit(Tr2.stackx, t2.stackx, "stackx");
		ret += checkit(Tr2.stacky, t2.stacky, "stacky");
		ret += checkit(Tr2.theta, t2.theta, "theta");
		ret += checkit(Tr2.phi, t2.phi, "phi");
		ret += checkit(Tr2.chisqx, t2.chisqx, "chisqx");
		ret += checkit(Tr2.chisqy, t2.chisqy, "chisqy");
	    } else {
		printf("\nstruct sc_softTraj Tr1 = {\n");
		printf("\t%f,\t%f,\n", t1.layer1x, t1.layer1y);
		printf("\t%f,\t%f,\n", t1.layer2x, t1.layer2y);
		printf("\t%f,\t%f,\n", t1.layer3x, t1.layer3y);
		printf("\t%f,\t%f,\n", t1.stackx, t1.stacky);
		printf("\t%f,\t%f,\n", t1.theta, t1.phi);
		printf("\t%f,\t%f,\n", t1.chisqx, t1.chisqy);
		printf("};\n\n");

		printf("\nstruct sc_softTraj Tr2 = {\n");
		printf("\t%f,\t%f,\n", t2.layer1x, t2.layer1y);
		printf("\t%f,\t%f,\n", t2.layer2x, t2.layer2y);
		printf("\t%f,\t%f,\n", t2.layer3x, t2.layer3y);
		printf("\t%f,\t%f,\n", t2.stackx, t2.stacky);
		printf("\t%f,\t%f,\n", t2.theta, t2.phi);
		printf("\t%f,\t%f,\n", t2.chisqx, t2.chisqy);
		printf("};\n\n");
	    }
	}
    }

    if (!gotevt) {
	fprintf(stderr,"%s: never got event 200072!\n");
	return -1;
    }

    return 0;
}

int
checkit(got, want, phrase)
    float got, want;
    char *phrase;
{
    static float epsilon = 0.0003;
    if (want - epsilon <= got && got <= want + epsilon) {
	return 0;
    } else {
	fprintf(stderr," %s: got %f, wanted %f\n", phrase, got, want);
	return 1;
    }
}
