/* t_msu_reorder.c - sc_msu_reorder routines */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <softcris.h>
#include <msu.h>

char *Progname;

int
main(int argc, char *argv[])
{
    u_char b[60000];
    u_char b2[60000];
    struct sc_msuEvt e;
    char filename[100];
    int gotevt = 0;
    int i;
    FILE *ifp;
    FILE *ofp;
    u_char *p;
    int ret;
    long size, vidsize1, vidsize2;
    u_short exp_id;

    Progname = argv[0];

    ifp = fopen("Input/msu.au000", "rb");
    if (ifp == (FILE *)NULL) {
	fprintf(stderr, "%s: can't fopen 'Input/msu.au000' (%s)\n",
	    Progname, strerror(errno));
	return -1;
    }

    ofp = fopen("Input/msu.au000.out", "wb");
    if (ofp == (FILE *)NULL) {
	fprintf(stderr, "%s: can't fopen 'Input/msu.au000.out' (%s)\n",
	    Progname, strerror(errno));
	return -1;
    }

    ret = sc_msu_init(ifp, NULL, 0, Progname);
    if (ret) {
	fprintf(stderr, "%s says:\n", Progname);
	fprintf(stderr, 
	 "sc_msu_init(ifp, NULL, 0, %s) returned %d (wanted 0)\n",
	    Progname, ret);
	return -1;
    }

    while (sc_msu_next_evt(&e) == 0) {
	p = b;
	ret = sc_msu_reorder_std(e.ptr, p);
	if (ret) {
	    fprintf(stderr, "%s: reorder_std returned %d (not 0)\n",
		Progname, ret);
	    return -1;
	}
	
	p += STDSIZ;
	if (e.type == T_REAL) {
	    ret = sc_msu_reorder_real(e.ptr, p, &size, &vidsize1, 
		&vidsize2, &exp_id);
	    if (ret) {
		fprintf(stderr,"%s: reorder_real returned %d\n", Progname, ret);
		return -1;
	    }
	} else if (e.type == T_REPLY) {
	    size = e.size;
	    ret = sc_msu_reorder_reply(e.ptr, p);
	    if (ret) {
		fprintf(stderr,"%s: reorder_reply returned %d\n", Progname, ret);
		return -1;
	    }
	} else {
	    fprintf(stderr, "%s: sorry can't do event type '%d' yet\n",
	    	Progname, e.type);
	    return -1;
	}
	ret = fwrite(b, 1, size, ofp);
	if (ret != size) {
	    fprintf(stderr, "%s: bad write to output file (%s)\n",
	    	Progname, strerror(errno));
	    exit(1);
	}
    }
    fclose(ofp);
    fclose(ifp);

#define RDSIZE	16348

    strcpy(filename, "Input/msu.au000.other");
    ifp = fopen(filename, "rb");
    if (ifp == (FILE *)NULL) {
	fprintf(stderr,"%s: can't open '%s' (%s)\n",
	    Progname, filename, strerror(errno));
	return -1;
    }
    ret = fread(b, 1, RDSIZE, ifp);
    if (ret != RDSIZE) {
	fprintf(stderr,"%s: bad read on '%s' (%s)\n",
	    Progname, filename, strerror(errno));
	return -1;
    }
    fclose(ifp);

    strcpy(filename, "Input/msu.au000.out");
    ifp = fopen(filename, "rb");
    if (ifp == (FILE *)NULL) {
	fprintf(stderr,"%s: can't open '%s' (%s)\n",
	    Progname, filename, strerror(errno));
	return -1;
    }
    ret = fread(b2, 1, RDSIZE, ifp);
    if (ret != RDSIZE) {
	fprintf(stderr,"%s: bad read on '%s' (%s)\n",
	    Progname, filename, strerror(errno));
	return -1;
    }
    fclose(ifp);

    for (i=0; i<RDSIZE; i++) {
	if (*(b+i) != *(b2 + i)) {
	    fprintf(stderr, "%s: byte %d wrong, wanted 0x%02x, got 0x%02x\n",
	    	Progname, i, *(b+i), *(b2+i));
	    return -1;
	}
    }

    return 0;
}
