/* raw_traj_test.c - sc_raw_traj() */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <sys/stat.h>
#include <softcris.h>

char *Progname;

#define NCHECK	1
int Ntraj = 1;

struct sc_Coords_Traj Tr[NCHECK] = {
	/* trajectory 0 */
		/* coords */
	95.2358627, 27.5224609, 91.0648880,
	85.5469437, 87.7743454, 134.3697357,
	634.0062866, 1679.2814941, 3021.4333496,
	4308.2368164, 5211.0390625, 6692.9853516,
	-0.3671875, -0.6250000, 0.0234375,
	-0.4843750, -0.1875000, -0.4062500,
	2,
	511, 694, 443,
	598, 943, 449,
	68, 123, 190,
	259, 311, 402,
	4096, 8192, 8192,
	8192, 8192, 6144,
		/* trajectories */
	95.2358627, 27.5224609, 91.0648880,
	85.5469437, 87.7743454, 134.3697357,
	86.0527463, 85.6275085, 84.9637228,
	84.6007149, 84.9637228, 84.1028754,
	83.7502395, 84.1028754, 82.9619936,
	82.5989857, 82.9619936, 82.1011465,
	81.7485101, 82.1011465, 81.0224944,
	80.6594865, 81.0224944, 80.1616477,
	79.8090113, 80.1616477, 79.3734014,
	158.0831484, 164.1665591, 173.6626135,
	178.8557685, 173.6626135, 185.9778114,
	191.0225872, 185.9778114, 202.2991558,
	207.4923109, 202.2991558, 214.6143510,
	219.6591324, 214.6143510, 230.0454425,
	235.2385975, 230.0454425, 242.3606319,
	247.4054134, 242.3606319, 253.6372062,
	56.0861310, -86.0014601,
	0.5026017, 0.2652522,
		/* quality */
	0.1396948, 0,
	0.0000000, 0.0000000, 0.0000000,
	0.0000000, 0.0000000, 0.0000000,
	0.0000000,    0,    0,
	      0, 0.0000000,
	  5,   3,   2,
	  1,   0,   4,
	      0, 0.0000000,
};

#define NTRAJ	20

int
main(int argc, char *argv[])
{
    struct sc_crisEvt c;
    struct sc_softCoords coords;
    u_char *cp;
    char filename[100];
    FILE *fp;
    int i;
    int j;
    u_char nbytes;
    int ntraj = NTRAJ;
    int ret;
    struct stat statbuf;
    struct sc_Coords_Traj t[NTRAJ];
    char tmpstr[20];

    int x[128];
    int y[128];
    long I[128];

    int dbg = 0;

    if (argc > 1) {
	if (strcmp(argv[1], "-d") == 0) {
	    dbg = 1;
	}
    }

    Progname = argv[0];

    /* crisevt.357.100 is the sc_crisEvt struct for event 100 from day 357
     * of year 1997 */
    strcpy(filename, "Input/crisevt.357.100");
    ret = stat(filename, &statbuf);
    if (ret != 0) {
	fprintf(stderr, "%s: can't stat '%s'\n", Progname, filename);
	return -1;
    }
    fp = fopen(filename, "rb");
    if (fp == NULL) {
	fprintf(stderr, "%s: can't open '%s' (%s)\n", Progname, filename,
	    strerror(errno));
	return -1;
    }

    ret = fread(&c, 1, statbuf.st_size, fp);
    if (ret != statbuf.st_size) {
	fprintf(stderr, "%s: bad read\n", Progname);
	return -1;
    }

    sc_cris_set_cam(SC_CAM2);
    sc_traj_select_method(3);

   for (i=0; i<c.nblobs; i++) {
	x[i] = (int)c.hodo[i].x;
	y[i] = (int)c.hodo[i].y;
	I[i] = (long)c.hodo[i].I;
    }
    ret = sc_raw_traj(x, y, I, c.nblobs, t, &ntraj);

    if (ret < 0) {
	fprintf(stderr, "%s: bad return from sc_raw_traj() = %d\n",
	    Progname, ret);
	fprintf(stderr, "    \"%s\"\n", sc_get_err_mesg());
	return -1;
    }

    if (!dbg) {
	ret = dcheckit(Ntraj, ntraj, "ntraj");
	for (j=0; j<NCHECK; j++) {
	    fprintf(stderr,"event %d\n", j);
	    ret += checkit(Tr[j].C.x1, t[j].C.x1, "coordx1");
	    ret += checkit(Tr[j].C.y1, t[j].C.y1, "coordy1");
	    ret += checkit(Tr[j].C.x2, t[j].C.x2, "coordx2");
	    ret += checkit(Tr[j].C.y2, t[j].C.y2, "coordy2");
	    ret += checkit(Tr[j].C.x3, t[j].C.x3, "coordx3");
	    ret += checkit(Tr[j].C.y3, t[j].C.y3, "coordy3");
	    ret += checkit(Tr[j].C.fiber[SC_X1], t[j].C.fiber[SC_X1], "fiberX1");
	    ret += checkit(Tr[j].C.fiber[SC_Y1], t[j].C.fiber[SC_Y1], "fiberY1");
	    ret += checkit(Tr[j].C.fiber[SC_X2], t[j].C.fiber[SC_X2], "fiberX2");
	    ret += checkit(Tr[j].C.fiber[SC_Y2], t[j].C.fiber[SC_Y2], "fiberY2");
	    ret += checkit(Tr[j].C.fiber[SC_X3], t[j].C.fiber[SC_X3], "fiberX3");
	    ret += checkit(Tr[j].C.fiber[SC_Y3], t[j].C.fiber[SC_Y3], "fiberY3");
	    ret += checkit(Tr[j].C.ypos[SC_X1], t[j].C.ypos[SC_X1], "yposX1");
	    ret += checkit(Tr[j].C.ypos[SC_Y1], t[j].C.ypos[SC_Y1], "yposY1");
	    ret += checkit(Tr[j].C.ypos[SC_X2], t[j].C.ypos[SC_X2], "yposX2");
	    ret += checkit(Tr[j].C.ypos[SC_Y2], t[j].C.ypos[SC_Y2], "yposY2");
	    ret += checkit(Tr[j].C.ypos[SC_X3], t[j].C.ypos[SC_X3], "yposX3");
	    ret += checkit(Tr[j].C.ypos[SC_Y3], t[j].C.ypos[SC_Y3], "yposY3");
	    ret += dcheckit(Tr[j].C.cam, t[j].C.cam, "coordscam");
	    ret += dcheckit(Tr[j].C.xpix[SC_X1], t[j].C.xpix[SC_X1], "xpixX1");
	    ret += dcheckit(Tr[j].C.xpix[SC_Y1], t[j].C.xpix[SC_Y1], "xpixY1");
	    ret += dcheckit(Tr[j].C.xpix[SC_X2], t[j].C.xpix[SC_X2], "xpixX2");
	    ret += dcheckit(Tr[j].C.xpix[SC_Y2], t[j].C.xpix[SC_Y2], "xpixY2");
	    ret += dcheckit(Tr[j].C.xpix[SC_X3], t[j].C.xpix[SC_X3], "xpixX3");
	    ret += dcheckit(Tr[j].C.xpix[SC_Y3], t[j].C.xpix[SC_Y3], "xpixY3");
	    ret += dcheckit(Tr[j].C.ypix[SC_X1], t[j].C.ypix[SC_X1], "ypixX1");
	    ret += dcheckit(Tr[j].C.ypix[SC_Y1], t[j].C.ypix[SC_Y1], "ypixY1");
	    ret += dcheckit(Tr[j].C.ypix[SC_X2], t[j].C.ypix[SC_X2], "ypixX2");
	    ret += dcheckit(Tr[j].C.ypix[SC_Y2], t[j].C.ypix[SC_Y2], "ypixY2");
	    ret += dcheckit(Tr[j].C.ypix[SC_X3], t[j].C.ypix[SC_X3], "ypixX3");
	    ret += dcheckit(Tr[j].C.ypix[SC_Y3], t[j].C.ypix[SC_Y3], "ypixY3");
	    ret += dcheckit(Tr[j].C.I[SC_X1], t[j].C.I[SC_X1], "IX1");
	    ret += dcheckit(Tr[j].C.I[SC_Y1], t[j].C.I[SC_Y1], "IY1");
	    ret += dcheckit(Tr[j].C.I[SC_X2], t[j].C.I[SC_X2], "IX2");
	    ret += dcheckit(Tr[j].C.I[SC_Y2], t[j].C.I[SC_Y2], "IY2");
	    ret += dcheckit(Tr[j].C.I[SC_X3], t[j].C.I[SC_X3], "IX3");
	    ret += dcheckit(Tr[j].C.I[SC_Y3], t[j].C.I[SC_Y3], "IY3");
	    ret += checkit(Tr[j].T.layer1x, t[j].T.layer1x, "layer1x");
	    ret += checkit(Tr[j].T.layer1y, t[j].T.layer1y, "layer1y");
	    ret += checkit(Tr[j].T.layer2x, t[j].T.layer2x, "layer2x");
	    ret += checkit(Tr[j].T.layer2y, t[j].T.layer2y, "layer2y");
	    ret += checkit(Tr[j].T.layer3x, t[j].T.layer3x, "layer3x");
	    ret += checkit(Tr[j].T.layer3y, t[j].T.layer3y, "layer3y");
	    for(i=0; i<SC_NSTACK; i++) {
		sprintf(tmpstr, "stackx[%d]", i);
		ret += checkit(Tr[j].T.stackx[i], t[j].T.stackx[i], tmpstr);
		sprintf(tmpstr, "stacky[%d]", i);
		ret += checkit(Tr[j].T.stacky[i], t[j].T.stacky[i], tmpstr);
	    }
	    ret += checkit(Tr[j].T.theta, t[j].T.theta, "theta");
	    ret += checkit(Tr[j].T.phi, t[j].T.phi, "phi");
	    ret += checkit(Tr[j].T.chisqx, t[j].T.chisqx, "chisqx");
	    ret += checkit(Tr[j].T.chisqy, t[j].T.chisqy, "chisqy");
	    ret += checkit(Tr[j].Q.fit_param, t[j].Q.fit_param, "fit_param");
	    ret += dcheckit(Tr[j].Q.hop_param, t[j].Q.hop_param, "hop_param");
	    for(i=0; i<SC_NLAYERS; i++) {
		sprintf(tmpstr, "Iratio[%d]", i);
		ret += checkit(Tr[j].Q.Iratio[i], t[j].Q.Iratio[i], tmpstr);
	    }
	    ret += checkit(Tr[j].Q.qual_factor, t[j].Q.qual_factor, "qualfactor");
	    ret += dcheckit(Tr[j].Q.raw_dx_1000, t[j].Q.raw_dx_1000,
	    "Q.raw_dx_1000");
	    ret += dcheckit(Tr[j].Q.raw_dy_1000, t[j].Q.raw_dy_1000,
	    "Q.raw_dy_1000");
	    ret += dcheckit(Tr[j].Q.lreserved, t[j].Q.lreserved, "Q.lreserved");
	    ret += checkit(Tr[j].Q.reserved, t[j].Q.reserved, "Q.reserved");
	    for(i=0; i<SC_NLAYERS; i++) {
		sprintf(tmpstr, "blobid[%d]", i);
		ret += dcheckit((int)Tr[j].blobid[i],(int)t[j].blobid[i], tmpstr);
	    }
	    ret += dcheckit(Tr[j].sreserved, t[j].sreserved, "sreserved");
	    ret += checkit(Tr[j].reserved, t[j].reserved, "reserved");
	}
	
	if (ret) {
	    return -1;
	}
	
    } else {
	printf("int Ntraj = %d;\n\n", ntraj);
	printf("struct sc_Coords_Traj Tr[NCHECK] = {\n");
	for (j=0; j<NCHECK; j++) {
	    printf("\t/* trajectory %d */\n", j);
	    printf("\t\t/* coords */\n");
	    printf("\t%.7f, %.7f, %.7f,\n", t[j].C.x1, t[j].C.y1, t[j].C.x2);
	    printf("\t%.7f, %.7f, %.7f,\n", t[j].C.y2, t[j].C.x3, t[j].C.y3);
	    for (i=0; i<SC_NLAYERS; i+=3) {
		printf("\t%.7f, %.7f, %.7f,\n", 
		    t[j].C.fiber[i], t[j].C.fiber[i+1], t[j].C.fiber[i+2]);
	    }
	    for (i=0; i<SC_NLAYERS; i+=3) {
		printf("\t%.7f, %.7f, %.7f,\n", 
		    t[j].C.ypos[i], t[j].C.ypos[i+1], t[j].C.ypos[i+2]);
	    }
	    printf("\t%d,\n", t[j].C.cam);
	    for (i=0; i<SC_NLAYERS; i+=3) {
		printf("\t%d, %d, %d,\n", 
		    t[j].C.xpix[i], t[j].C.xpix[i+1], t[j].C.xpix[i+2]);
	    }
	    for (i=0; i<SC_NLAYERS; i+=3) {
		printf("\t%d, %d, %d,\n", 
		    t[j].C.ypix[i], t[j].C.ypix[i+1], t[j].C.ypix[i+2]);
	    }
	    for (i=0; i<SC_NLAYERS; i+=3) {
		printf("\t%ld, %ld, %ld,\n", 
		    t[j].C.I[i], t[j].C.I[i+1], t[j].C.I[i+2]);
	    }
	    printf("\t\t/* trajectories */\n");
	    printf("\t%.7f, %.7f, %.7f,\n\t%.7f, %.7f, %.7f,\n",
		t[j].T.layer1x, t[j].T.layer1y, t[j].T.layer2x, t[j].T.layer2y,
		t[j].T.layer3x, t[j].T.layer3y);
	    for (i=0; i<SC_NSTACK; i+=3) {
		printf("\t%.7f, %.7f, %.7f,\n", 
		    t[j].T.stackx[i], t[j].T.stackx[i+1], t[j].T.stackx[i+2]);
	    }
	    for (i=0; i<SC_NSTACK; i+=3) {
		printf("\t%.7f, %.7f, %.7f,\n", 
		    t[j].T.stacky[i], t[j].T.stacky[i+1], t[j].T.stacky[i+2]);
	    }
	    printf("\t%.7f, %.7f,\n\t%.7f, %.7f,\n",
		t[j].T.theta, t[j].T.phi, t[j].T.chisqx, t[j].T.chisqy);
	    printf("\t\t/* quality */\n");
	    printf("\t%.7f, %d,\n", t[j].Q.fit_param, t[j].Q.hop_param);
	    for (i=0; i<SC_NLAYERS; i+=3) {
		printf("\t%.7f, %.7f, %.7f,\n",
		    t[j].Q.Iratio[i], t[j].Q.Iratio[i+1], t[j].Q.Iratio[i+2]);
	    }
	    printf("\t%.7f, %4d, %4d,\n", t[j].Q.qual_factor,
		t[j].Q.raw_dx_1000, t[j].Q.raw_dy_1000);
	    printf("\t%7d, %.7f,\n", t[j].Q.lreserved, t[j].Q.reserved);
	    for (i=0; i<SC_NLAYERS; i+=3) {
		printf("\t%3d, %3d, %3d,\n",
		    t[j].blobid[i], t[j].blobid[i+1], t[j].blobid[i+2]);
	    }
	    printf("\t%7d, %.7f,\n", t[j].sreserved, t[j].reserved);
	    printf("\n");
	}
	printf("};\n");
    }

    return 0;
}

int
dcheckit(want, got, phrase)
    int want, got;
    char *phrase;
{
    if (got == want) {
	return 0;
    } else {
	fprintf(stderr," %s: got %d, wanted %d\n", phrase, got, want);
	return 1;
    }
}

int
checkit(want, got, phrase)
    float want, got;
    char *phrase;
{
    static float epsilon = 0.0003;
    if (want - epsilon <= got && got <= want + epsilon) {
	return 0;
    } else {
	fprintf(stderr," %s: got %f, wanted %f\n", phrase, got, want);
	return 1;
    }
}

