/* t_read_fmap_by_name.c - read_fmap_by_name() */

/* MAPDIR should be #define'd in Makefile.in */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <softcris.h>

char *Progname;

struct {
    int x;
    int y;
    float fiber;
    char ypos;
    short layer;
} vals[] = {
    0,		100,    1.0,    0x00,   -1,
    50,		45,     1.0,    0x00,   -1,
    250,        40,     138.0,  0x3f,   0,
    150,        65,     590.0,  0x1b,   0,
    400,        60,     504.0,  0x14,   0,
    57,		122,    1.0,    0x00,   -1,
    433,        122,    1713.0, 0x1d,   1,
    40,		240,    1.0,    0x00,   -1,
    240,        200,    3233.0, 0x75,   2,
    300,        189,    3003.0, 0x45,   2,
    400,        60,     504.0,  0x14,   0,
    400,        120,    1717.0, 0x70,   1,
    400,        200,    3253.0, 0x62,   2,
    400,        268,    4559.0, 0x5e,   3,
    400,        348,    5768.0, 0x70,   4,
    400,        425,    7150.0, 0x28,   5,
#ifdef NOTDEF
    0,        100,  0.0,    0x00,   -1,
    50,       45,   0.0,    0x00,   -1,
    250,      40,   137.0,  0x3f,   0,
    150,      65,   588.0,  0x1b,   0,
    400,      60,   502.0,  0x14,   0,
    57,       122,  0.0,    0x00,   -1,
    433,      122,  1711.0, 0x1d,   1,
    40,       240,  0.0,    0x00,   -1,
    240,      200,  3230.0, 0x75,   2,
    300,      189,  3000.0, 0x45,   2,
    400,      60,   502.0,  0x14,   0,
    400,      120,  1715.0, 0x70,   1,
    400,      200,  3250.0, 0x62,   2,
    400,      268,  4556.0, 0x5e,   3,
    400,      348,  5765.0, 0x70,   4,
    400,      425,  7144.0, 0x28,   5,
#endif
};

#define NVALS (sizeof(vals)/sizeof(vals[0]))

int
main(int argc, char *argv[])
{
    int i;
    char mapname[512];
    int ret;
    struct sc_fmapElem *f;
    struct sc_fmapElem *fmap[SC_FMAPX];

    Progname = argv[0];

    for (i=0; i<SC_FMAPX; i++) {
	fmap[i] = (struct sc_fmapElem *)
	    malloc(SC_FMAPY * sizeof(struct sc_fmapElem));
	if (fmap[i] == (struct sc_fmapElem *)NULL) {
	    fprintf(stderr, "%s: can't malloc map (tab %d)\n", 
	    	Progname, i);
	    exit(1);
	}
    }

    sprintf(mapname, "%s/fibermap.cam2", MAPDIR);
    ret = sc_read_fmap_by_name(mapname, fmap);
    if (ret) {
	fprintf(stderr,"%s: bad ret = %d from sc_read_fmap_by_name()\n",
	    Progname, ret);
	return (-1);
    }

    for (i=0; i<NVALS; i++) {
	f = &fmap[vals[i].x][vals[i].y];
#ifndef NOTDEF
	if (ceil(f->fiber) != vals[i].fiber || f->ypos != vals[i].ypos ||
		f->layer != vals[i].layer) {
	    fprintf(stderr,"%s: (i=%d) some of these things are wrong\n", 
	    	Progname, i);
	    fprintf(stderr,"\tfiber: want %.1f, got %.1f\n",
	    	vals[i].fiber, ceil(f->fiber));
	    fprintf(stderr,"\typos: want %02x, got %02x\n",
	    	vals[i].ypos, f->ypos);
	    fprintf(stderr,"\tlayer: want %d, got %d\n",
	    	vals[i].layer, f->layer);
	    return -1;
	}
#else
	printf("    %d,\t%d,\t%.1f,\t0x%02x,\t%d,\n", 
	    vals[i].x, vals[i].y, ceil(f->fiber), f->ypos, f->layer);
#endif
    }

    return 0;
}
