/* z_test.c - sc_z() test */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <math.h>
#include <softcris.h>

char *Progname;

struct sc_crisZ Zz = {
    925.1044332100000247, 969.66459503999999469670, 1917.49711597999998957675,
    1824.2800007099999675, 2060.75775359999988722848, 2091.22061598000027515809,
    2171.6904280799999469, 2188.22028110000019296422, -6.58986282348633523043, 
    16.7211020870209594, 
    8,
};

int
main(int argc, char *argv[])
{
    u_char buf[1000];
    struct sc_crisEvt c;
    struct sc_softCoords coords;
    u_char *cp;
    int dbg = 0;
    char filename[100];
    FILE *fp;
    int i;
    u_char nbytes;
    int ret;
    struct sc_softTraj traj;
    struct sc_crisZ z;
    char tmpstr[20];

    if (argc > 1) {
	if (strcmp(argv[1], "-d") == 0) {
	    dbg = 1;
	}
    }

    Progname = argv[0];

    strcpy(filename, "Input/cris.normal");
    fp = fopen(filename, "rb");
    if (fp == NULL) {
	fprintf(stderr, "%s: can't open '%s' (%s)\n", Progname, filename,
	    strerror(errno));
	return -1;
    }

    cp = buf;
    ret = fread(cp, 1, 1, fp);
    if (ret != 1) {
	fprintf(stderr, "%s: bad first read\n", Progname);
	return -1;
    }

    nbytes = *cp;
    cp++;
    ret = fread(cp, 1, (int)nbytes-1, fp);
    if (ret != nbytes-1) {
	fprintf(stderr, "%s: bad read got %d wanted %d\n", Progname, ret,
	    nbytes-1);
	return -1;
    }
    fclose(fp);

    sc_traj_select_method(4);

    ret = sc_cris_unpack(buf, &c, SC_CAM2);
    if (ret) {
	fprintf(stderr, "%s: bad return = %d\n", ret);
	return -1;
    }

    ret = sc_cris_coords(&c, &coords, SC_CAM2);
    if (ret) {
	fprintf(stderr, "%s: bad return from sc_cris_coords() = %d\n",
	    Progname, ret);
	return -1;
    }

    ret = sc_trajectory(&coords, &traj);
    if (ret) {
	fprintf(stderr, "%s: bad return from sc_trajectory() = %d\n",
	    Progname, ret);
	return -1;
    }

    ret = sc_z(&c, &traj, &z);
    if (ret) {
	fprintf(stderr, "%s: bad return from sc_z() = %d\n",
	    Progname, ret);
	return -1;
    }


    if (!dbg) {
	ret = checkit(Zz.MeV[0], z.MeV[0], "z.MeV[0]");
	for (i=1; i < SC_CRIS_NCTRPH - 1; i++) {
	    sprintf(tmpstr, "z.MeV[%d]", i);
	    ret += checkit(Zz.MeV[i], z.MeV[i], tmpstr);
	}
	ret += checkit(Zz.zbest, z.zbest, "z.zbest");
	ret += checkit((float)Zz.range, (float)z.range, "z.range");
	
	if (ret) {
	    return -1;
	}
    } else {
	printf("\t");
	for (i=0; i < SC_CRIS_NCTRPH - 1; i++) {
	    printf("%.20f, ", z.MeV[i]);
	}
	printf("\n");
	printf("\t%.20f, %d\n", z.zbest, z.range);
    }

    return 0;
}

int
checkit(want, got, phrase)
    float want, got;
    char *phrase;
{
    static float epsilon = 0.0003;
    if (want - epsilon <= got && got <= want + epsilon) {
	return 0;
    } else {
	fprintf(stderr," %s: got %f, wanted %f\n", phrase, got, want);
	return 1;
    }
}

