void BuildCrisSubset(block, subset)
AceBlock *block;
CrisSubset *subset;
{
   UnpackedDddHeader         DddHeader;
   UnpackedHeaderAggregation HeaderAggregation;
   UnpackedVctfHeader        VctfHeader;
   UnpackedPacketHeader      PacketHeader;
   UnpackedMinorFrameHeader  MinorFrameHeader;

   int i;
   int ibeg;
   int iend;
   int ii;

   UnpackDddHeader(block->DddHeader, &DddHeader);
   subset->TimeOfDay = DddHeader.TimeOfDay;
   subset->DayOfYear = DddHeader.DayOfYear;
   subset->year = DddHeader.year;
   subset->facility = DddHeader.source.facility;
   subset->subfacility = DddHeader.source.subfacility;
   subset->assembly = DddHeader.source.assembly;
   subset->DataNature = DddHeader.stream.DataNature;

   UnpackHeaderAggregation(block->HeaderAggregation,
                                                   &HeaderAggregation);
   subset->ErrorFlags = HeaderAggregation.secondary.ErrorFlags;
   
   UnpackVctfHeader(block->telemetry.VctfHeader, &VctfHeader); 
   subset->SpacecraftId = (u_char) VctfHeader.SpacecraftId;
   subset->VirtualChannelId = VctfHeader.VirtualChannelId;

   UnpackPacketHeader(block->telemetry.PacketHeader, &PacketHeader);
   subset->PacketSecondaryHeader = PacketHeader.PacketSecondaryHeader;

   UnpackMinorFrameHeader(block->telemetry.MinorFrameHeader,
                                                    &MinorFrameHeader);
   subset->MajorFrameCount = MinorFrameHeader.MajorFrameCount;
   subset->MinorFrameCount = MinorFrameHeader.MinorFrameCount;
   subset->FormatId = MinorFrameHeader.FormatId;
  
   subset->PhaseAngle = 0; 
   if ((0 < subset->FormatId) && (subset->FormatId < 10))
      subset->PhaseAngle = ((u_short) block->telemetry.data[1])*256 
                                  + (u_short) block->telemetry.data[2];
   
   switch (subset->FormatId)
   {
       case DOWNLINK_FORMAT_SCIENCE:
          if (subset->MinorFrameCount == 3)
          {
             subset->MainBusCurrent = block->telemetry.data[13];
             subset->MainBusVoltage = block->telemetry.data[14];
             subset->CrisInternalTemperature1 = 0;
             subset->CrisInternalTemperature2 = 0;
             subset->CrisInterfaceTemperature = 0;
             subset->CrisCurrent = 0;
          }
          else if (subset->MinorFrameCount == 9)
          {
             subset->MainBusCurrent = 0;
             subset->MainBusVoltage = 0;
             subset->CrisInternalTemperature1 =
                                        block->telemetry.data[17];
             subset->CrisInternalTemperature2 =
                                        block->telemetry.data[18];
             subset->CrisInterfaceTemperature = 0;
             subset->CrisCurrent = 0;
          }
          else if (subset->MinorFrameCount == 10)
          {
             subset->MainBusCurrent = 0;
             subset->MainBusVoltage = 0;
             subset->CrisCurrent = block->telemetry.data[6];
             subset->CrisInternalTemperature1 = 0;
             subset->CrisInternalTemperature2 = 0;
             subset->CrisInterfaceTemperature = 0;
          }
          else
          {
             subset->MainBusCurrent = 0;
             subset->MainBusVoltage = 0;
             subset->CrisInternalTemperature1 = 0;
             subset->CrisInternalTemperature2 = 0;
             subset->CrisInterfaceTemperature = 0;
             subset->CrisCurrent = 0;
          }
          break;
       case DOWNLINK_FORMAT_ADC:
          subset->MainBusCurrent           = block->telemetry.data[507];
          subset->MainBusVoltage           = block->telemetry.data[508];
          subset->CrisInternalTemperature1 = block->telemetry.data[628];
          subset->CrisInternalTemperature2 = block->telemetry.data[629];
          subset->CrisInterfaceTemperature = block->telemetry.data[772];
          subset->CrisCurrent = block->telemetry.data[635];
          break;
       case DOWNLINK_FORMAT_LOW_RATE_HSKP:
       case DOWNLINK_FORMAT_LOW_RATE_CDH_MEMORY:
       case DOWNLINK_FORMAT_LOW_RATE_CDH_BIN:
          subset->MainBusCurrent = block->telemetry.data[70];
          subset->MainBusVoltage = block->telemetry.data[71];
          subset->CrisInternalTemperature1 = block->telemetry.data[191];
          subset->CrisInternalTemperature2 = block->telemetry.data[192];
          subset->CrisInterfaceTemperature = block->telemetry.data[335];
          subset->CrisCurrent = block->telemetry.data[198];
          break;
       case DOWNLINK_FORMAT_LOW_RATE_ADC:
          subset->MainBusCurrent = block->telemetry.data[67];
          subset->MainBusVoltage = block->telemetry.data[68];
          subset->CrisInternalTemperature1 =
                                       block->telemetry.data[188];
          subset->CrisInternalTemperature2 =
                                       block->telemetry.data[189];
          subset->CrisCurrent = block->telemetry.data[195];
          subset->CrisInterfaceTemperature = 0;
          break;
      case DOWNLINK_FORMAT_RTSW:
          subset->MainBusCurrent = block->telemetry.data[809];
          subset->MainBusVoltage = block->telemetry.data[810];
          subset->CrisInternalTemperature1 = 0;
          subset->CrisInternalTemperature2 = 0;
          subset->CrisInterfaceTemperature = 0;
          subset->CrisCurrent = 0;
          break;
      default:
          subset->MainBusCurrent = 0;
          subset->MainBusVoltage = 0;
          subset->CrisInternalTemperature1 = 0;
          subset->CrisInternalTemperature2 = 0;
          subset->CrisInterfaceTemperature = 0;
          subset->CrisCurrent = 0;
          break;
   }

   switch (subset->FormatId)
   {
       case DOWNLINK_FORMAT_SCIENCE:
       case DOWNLINK_FORMAT_ADC:
       case DOWNLINK_FORMAT_CDH_MEMORY:
       case DOWNLINK_FORMAT_CDH_BIN:
          ibeg = TELEMETRY_OFFSET_CRIS;
          iend = ibeg + NUM_CRIS_TELEMETRY_BYTES;
          ii = 0;
          for (i=ibeg; i<iend; i++) 
             subset->CrisData[ii++] = block->telemetry.data[i];
          break;
      default:
	  /* NOTE: bugfix? In the line below, it used to say
	   * subset->CrisData[ii++] causing a core dump under certain
	   * conditions. */
          for (i=0; i<NUM_CRIS_TELEMETRY_BYTES; i++)
             subset->CrisData[i] = 0;
          break;
   }
}
