/* chknorm - check the size of the cris normal evt
 *
 *	USAGE: chknorm b1 b2 b3 b4 b5 b6 [b7]
 *
 *		b1 - b7 are the hex representation of the first 7 bytes
 *		of the normal frame. b7 may or may not be present.
 *
 *	Marty Olevitch, Jan 1997
 */

#include <stdio.h>

char *Progname;

int
main(argc, argv)
    int argc;
    char *argv[];
{
    int b[7];
    int g1;
    int g2;
    int got_2nd_tel;
    int i;
    int len;
    int nblob;
    int r1;
    int r2;
    int stim;
    int sum;
    int xmit_guards;

    Progname = argv[0];
    if (argc < 7 || argc > 8) {
	fprintf(stderr,"USAGE: %s b1 b2 b3 b4 b5 b6 [b7]\n", Progname);
	exit(1);
    }

    for (i=0; i<argc-1; i++) {
	b[i] = (int)strtol(argv[i+1], (char **)NULL,16);
    }

    len = b[0];

    if (b[1] & 0x80) {
	stim = 1;
    } else {
	stim = 0;
    }

    g1 = (b[2] & 0x38) >> 3;
    r1 = (b[2] & 0x07) + 2;

    if (b[3] & 0x80) {
	got_2nd_tel = 1;
    } else {
	got_2nd_tel = 0;
    }

    if (b[3] & 0x40) {
	xmit_guards = 1;
    } else {
	xmit_guards = 0;
    }

    nblob = b[3] & 0x1f;

    if (got_2nd_tel) {
	if (argc != 8) {
	    fprintf(stderr,"I need 7 bytes, not 6!\n");
	    exit(1);
	}
	g2 = (b[6] & 0x38) >> 3;
	r2 = (b[6] & 0x07) + 2;
	sum = 7;	/* 7 header bytes */
    } else {
	sum = 6;	/* 6 header bytes */
    }

    if (stim) {
	sum += 4;
    }

    sum += 2;	/* primary tag bits */
    if (xmit_guards) {
	sum += (g1 * 2);
    }
    if (r1 % 2 == 0) {
	/* r1 even */
	sum += (((r1 + 1) * 12) + 4) / 8;
    } else {
	/* r1 odd */
	sum += ((r1 + 1) * 12) / 8;
    }

    sum += (nblob * 3);	/* SOFT blobs */

    if (got_2nd_tel) {
	sum += 2;	/* 2nd tag bits */
	if (xmit_guards) {
	    sum += (g2 * 2);
	}
	if (r2 % 2 == 0) {
	    /* r2 even */
	    sum += (((r2 + 1) * 12) + 4) / 8;
	} else {
	    sum += ((r2 + 1) * 12) / 8;
	}
    }

    if (sum == len) {
	printf("ok (len = %d)\n", len);
	exit(0);
    } else {
	printf("bad (len should be %d, not %d)\n", sum, len);
	exit(1);
    }
}
