/* cris_test_extr - extract the "Cris Subset" from the raw test data
 *
 *	USAGE: cris_test_extr input_file > outputfile
 *
 *	Uses Robert Radocynski's downlink routines.
 *
 * Marty Olevitch, Jan 1997
 *
 *	Format of output data:
 *
 *	Bytes 0 - 7				the string "next frm"		
 *	bytes 8 - (8+sizeof(struct CrisSubset))	CrisSubset for this minor frm
 *	(above repeats ad infinitum)
 */

#include "include/wustl.h"
#include <errno.h>	/* strerror */

#include "DownlinkLib/BuildCrisSubset.c"
#include "DownlinkLib/UnpackClcw.c"
#include "DownlinkLib/UnpackDddHeader.c"
#include "DownlinkLib/UnpackHeaderAggregation.c"
#include "DownlinkLib/UnpackMinorFrameHeader.c"
#include "DownlinkLib/UnpackPacketHeader.c"
#include "DownlinkLib/UnpackSfduLabel.c"
#include "DownlinkLib/UnpackVctfHeader.c"
#include "general/ConvertFromSeconds1970.c"
#include "general/ConvertToSeconds1970.c"
#include "general/byteorder.c"

#define FRMSTART "next frm"

main(argc, argv)
int argc;
char *argv[];
{
    AceBlock block;  
    FILE *fd;
    char *filename;
    char frmstart[8];
    CrisSubset subset;

    if (argc == 2) {
	filename = argv[1];
    } else {
	fprintf(stderr,"USAGE: %s inputfile > outputfile\n", argv[0]);
	exit(1);
    }

    fd = fopen(filename, "r");
    if (!fd) {
       fprintf(stderr, "%s: can't open '%s' (%s)\n", 
       	argv[0], argv[1], strerror(errno));
       exit(0);
    }

    while (fread(&block, ACE_BLOCK_LENGTH, 1, fd)) {
       BuildCrisSubset(&block, &subset);
       fwrite(FRMSTART, 1, strlen(FRMSTART), stdout);
       fwrite((char *)&subset, 1, sizeof(CrisSubset), stdout);
    }
}
    
