/* cris_test_extr - extract the "Cris Subset" from the raw test data
 *
 *	USAGE: cris_test_extr input_file > outputfile
 *	       cris_test_extr -v	(print version)
 *	       cris_test_extr < infile > outfile
 *
 *	Uses Robert Radocynski's downlink routines.
 *
 * Marty Olevitch, Jan 1997
 *
 *	New format of output data:
 *	
 *	Bytes 0 - 3				0x2ace	data ID
 *	Bytes 4 - 7				0x0000	filler
 *	bytes 8 - (8+sizeof(struct CrisSubset))	CrisSubset for this minor frm
 *	(above repeats ad infinitum)
 *
 *	Old format of output data:
 *
 *	Bytes 0 - 7				the string "next frm"		
 *	bytes 8 - (8+sizeof(struct CrisSubset))	CrisSubset for this minor frm
 *	(above repeats ad infinitum)
 */

static char cvsid[] = "$Id: cris_test_extr.c,v 1.3 1997/05/09 14:21:38 marty Exp $";

#define CTE_VERSION	"1.3"
/******
	revision log

	1.3	 9 May 1997	bug fix in included file
				    DownlinkLib/BuildCrisSubset.c
	1.2	14 Mar 1997	read stdin if no file given on cmd line
	1.1	25 Feb 1997	used 0x2ace data format
******/

#include "include/wustl.h"
#include <errno.h>	/* strerror */

#include "DownlinkLib/BuildCrisSubset.c"
#include "DownlinkLib/UnpackClcw.c"
#include "DownlinkLib/UnpackDddHeader.c"
#include "DownlinkLib/UnpackHeaderAggregation.c"
#include "DownlinkLib/UnpackMinorFrameHeader.c"
#include "DownlinkLib/UnpackPacketHeader.c"
#include "DownlinkLib/UnpackSfduLabel.c"
#include "DownlinkLib/UnpackVctfHeader.c"
#include "general/ConvertFromSeconds1970.c"
#include "general/ConvertToSeconds1970.c"
#include "general/byteorder.c"

#define FRMSTART 0x2ace
#define FILLER 0x0000

main(argc, argv)
int argc;
char *argv[];
{
    AceBlock block;  
    FILE *fd;
    char *filename;
    unsigned int fill;
    unsigned int id;
    CrisSubset subset;

    if (argc == 2) {
	if (strcmp(argv[1], "-v") == 0) {
	    printf("%s version %s\n", argv[0], CTE_VERSION);
	    exit(0);
	}
	filename = argv[1];
	fd = fopen(filename, "r");
	if (!fd) {
	   fprintf(stderr, "%s: can't open '%s' (%s)\n", 
	    argv[0], argv[1], strerror(errno));
	   exit(0);
	}
    } else {
	fd = stdin;
    }

    id = FRMSTART;
    fill = FILLER;

    while (fread(&block, ACE_BLOCK_LENGTH, 1, fd)) {
       BuildCrisSubset(&block, &subset);
       fwrite((char *)&id, 1, sizeof(id), stdout);
       fwrite((char *)&fill, 1, sizeof(fill), stdout);
       fwrite((char *)&subset, 1, sizeof(CrisSubset), stdout);
    }
}
    
