/* diagextr - write a diagnostic evt to stdout
 *
 *	USAGE: diagextr n f > out
 *		n   specify nth event to extract
 *		f   input file name
 *
 *	Raw telemetry data processed by cris_test_extr is the input 
 *	to this program. 
 *
 * Marty Olevitch, Feb 1997
 *
 *	Compile: gcc -o diagextr diagextr.c -L/usr/local/lib -lsoftcris -lm
 */

#include <stdio.h>
#include <string.h>	/* memcpy */
#include <errno.h>
#include <softcris.h>

u_char D[10000];
u_char *dp;

int
main(argc, argv)
    int argc;
    char *argv[];
{
    int cmdlong;
    struct sc_diagEvt d;
    int evtlong;
    FILE *fp;
    int i = 0;
    int indiag = 0;
    int j;
    int len;
    struct sc_minorFrame m;
    int majorcnt = 0;
    int n;
    int ret;
    int unpackerr;

    if (argc < 2 || argc > 3) {
	fprintf(stderr, "USAGE: %s nevt infile > outfile\n", argv[0]);
	exit(1);
    }

    n = atoi(argv[1]);

    if (argc == 3) {
	fp = fopen(argv[2], "r");
	if (fp == NULL) {
	    fprintf(stderr,"%s: can't open '%s' (%s)\n", argv[0], argv[2],
		strerror(errno));
	    exit(1);
	}
    } else {
	fp = stdin;
    }

    ret = sc_tlm_sync_major(fp);
    if (ret == 0) {
	exit(0);
    } else if (ret < 0) {
	fprintf(stderr,"%s: error in sync_major", argv[0]);
	if (ferror(fp)) {
	    fprintf(stderr, " (%s)", strerror(errno));
	}
	fprintf(stderr, "\n");
	exit(1);
    }
    fprintf(stderr,"sync after %d minor frames\n", ret);
    majorcnt++;
    /* printf("Major %d\n", majorcnt); */
    while (1) {
	cmdlong = evtlong = unpackerr = 0;
	ret = sc_tlm_next_minor(&m, fp);
	if (ret == 0) {
	    exit(0);
	} else if (ret == -1) {
	    fprintf(stderr,"%s: error after %d.%d frames", argv[0],
		majorcnt, m.seq);
	    if (ferror(fp)) {
		fprintf(stderr, " (%s)", strerror(errno));
	    }
	    fprintf(stderr, "\n");
	    exit(1);
	} else if (ret == -5) {
	    if (feof(fp)) {
		fprintf(stderr, "EOF after %d.%d frames\n", majorcnt, m.seq);
	    } else if (ferror(fp)) {
		fprintf(stderr, "error (%s) after %d.%d frames\n", 
		    strerror(errno), majorcnt, m.seq);
	    } else {
		fprintf(stderr,"%s: malformed intro around frame %d.%d\n",
		    argv[0], majorcnt, m.seq);
	    }
	    exit(1);
	} else if (ret == -2) {
	    cmdlong = 1;
	} else if (ret == -3) {
	    evtlong = 1;
	} else if (ret == -4) {
	    unpackerr = 1;
	} else {
	    if (m.major_frame_start) {
		majorcnt++;
	    }
	    if (m.diag) {
		if (indiag == 0) {
		    i++;
		    indiag = 1;
		    /* save this portion at begin of buf */
		    memcpy(D, m.diagdata, m.diag);
		    dp = D + m.diag;
		    len = m.diag;
		} else {
		    /* save this portion at cur ptr loc */
		    memcpy(dp, m.diagdata, m.diag);
		    dp += m.diag;
		    len += m.diag;
		}
	    } else {
		if (indiag) {
		    if (i == n) {
			/* output evt */
			fprintf(stderr,"Diag event %d (%d.%d), %d bytes\n", 
			    i, majorcnt, m.seq, len);
			ret = fwrite(D, 1 , len, stdout);
	    		if (ret != len) {
			    if (feof(fp)) {
				fprintf(stderr, "EOF after %d.%d frames\n", majorcnt, m.seq);
			    } else if (ferror(fp)) {
				fprintf(stderr, "error (%s) after %d.%d frames\n", 
				    strerror(errno), majorcnt, m.seq);
			    } else {
				fprintf(stderr,"%s: malformed intro around frame %d.%d\n",
				    argv[0], majorcnt, m.seq);
			    }
			}
			exit(0);
		    } else {
			indiag = 0;
		    }
		}
	    }
	}
    }
}
