/* extrminor - extract a set of minor frames from the 0x2ace data
 *
 *	USAGE: extrminor startmajor startminor count [file]
 *		(extract count minor frames starting at	startmajor.startminor)
 *
 *	Raw telemetry data processed by cris_test_extr is the input 
 *	to this program. A similar data set is the output.
 *
 * Marty Olevitch, Feb 1997
 */

#include <stdio.h>
#include <string.h>	/* memcpy */
#include <errno.h>
#include <softcris.h>

#define FRMSTART_ID	0x2ace
#define FILLER		0x0000

int
main(argc, argv)
    int argc;
    char *argv[];
{
    int cmdlong;
    int count;
    int evtlong;
    int extract = 0;
    unsigned int fill;
    FILE *fp;
    int i;
    unsigned int id;
    int j;
    struct sc_minorFrame m;
    int majorcnt = 0;
    int ret;
    int startmaj;
    int startmin;
    int unpackerr;

    if (argc == 5) {
	fp = fopen(argv[4], "r");
	if (fp == NULL) {
	    fprintf(stderr,"%s: can't open '%s' (%s)\n", argv[0], argv[4],
		strerror(errno));
	    exit(1);
	}
    } else if (argc == 4) {
	fp = stdin;
    } else {
	fprintf(stderr,"USAGE:extrminor startmajor startminor count [file]\n");
	exit(1);
    }

    startmaj = atoi(argv[1]);
    startmin = atoi(argv[2]);
    count = atoi(argv[3]);

    if (count <= 0) {
	exit(0);
    }

    id = FRMSTART_ID;
    fill = FILLER;

    fprintf(stderr,"startmaj: %d  startmin: %d  count: %d\n",
    	startmaj, startmin, count);

    fprintf(stderr,"trying to sync\n");
    ret = sc_tlm_sync_major(fp);
    fprintf(stderr,"done with sync\n");
    if (ret == 0) {
	exit(0);
    } else if (ret < 0) {
	fprintf(stderr,"%s: error in sync_major", argv[0]);
	if (ferror(fp)) {
	    fprintf(stderr, " (%s)", strerror(errno));
	}
	fprintf(stderr, "\n");
	exit(1);
    }
    fprintf(stderr,"sync after %d minor frames\n", ret);
    majorcnt++;
    while (1) {
	cmdlong = evtlong = unpackerr = 0;
	ret = sc_tlm_next_minor(&m, fp);
	if (ret == 0) {
	    exit(0);
	} else if (ret == -1) {
	    fprintf(stderr,"%s: error after %d.%d frames", argv[0],
		majorcnt, m.seq);
	    if (ferror(fp)) {
		fprintf(stderr, " (%s)", strerror(errno));
	    }
	    fprintf(stderr, "\n");
	    exit(1);
	} else if (ret == -5) {
	    if (feof(fp)) {
		fprintf(stderr, "EOF after %d.%d frames\n", majorcnt, m.seq);
	    } else if (ferror(fp)) {
		fprintf(stderr, "error (%s) after %d.%d frames\n", 
		    strerror(errno), majorcnt, m.seq);
	    } else {
		fprintf(stderr,"%s: malformed intro around frame %d.%d\n",
		    argv[0], majorcnt, m.seq);
	    }
	    exit(1);
	} else if (ret == -2) {
	    cmdlong = 1;
	} else if (ret == -3) {
	    evtlong = 1;
	} else if (ret == -4) {
	    unpackerr = 1;
	} else {
	    if (m.major_frame_start) {
		majorcnt++;
	    }
	    if (!extract) {
		if (majorcnt >= startmaj && m.seq >= startmin) {
		    extract = count;
		} else {
		    continue;
		}
	    }
	    fwrite((char *)&id, 1, sizeof(id), stdout);
	    fwrite((char *)&fill, 1, sizeof(fill), stdout);
	    fwrite((char *)&m.s, 1, sizeof(sc_crisSubset), stdout);
	    extract--;
	    if (extract <= 0) {
		exit(0);
	    }
	}
    }
}
