/* rdminor - process the cris minor frames
 *
 *	USAGE: rdminor [-s] [file]
 *		-s	no attempt to sync at beginning of data
 *		file	input file (default stdin)
 *
 *	Raw telemetry data processed by cris_test_extr is the input 
 *	to this program. 
 *
 * Marty Olevitch, Jan 1997
 */

#include <stdio.h>
#include <stdlib.h>	/* getopt */
#include <string.h>	/* memcpy */
#include <errno.h>
#include <softcris.h>
#include <time.h>

int
main(argc, argv)
    int argc;
    char *argv[];
{
    int cmdlong;
    int evtlong;
    static int Firsttime = 1;
    FILE *fp;
    int i;
    int j;
    struct sc_minorFrame m;
    int majorcnt = 0;
    int dosync = 1;
    int ret;
    int unpackerr;

    struct tm *tm;
    int sec1970;
    int second;

    int lastminor = -1;
    int lastmajor = 0;
    int lastsecond = -1;

    extern char *optarg;
    extern int optind;
    int c;

    while ((c = getopt(argc, argv, "s")) != EOF) {
	switch (c) {
	    case 's':
	    	dosync = 0;
		break;
	    case '?':
	    default:
	    	fprintf(stderr,"USAGE: %s [-s] [file]\n", argv[0]);
		exit(1);
	}
    }

    if (optind < argc) {
	fp = fopen(argv[optind], "r");
	if (fp == NULL) {
	    fprintf(stderr,"%s: can't open '%s' (%s)\n", 
	    	argv[0], argv[optind], strerror(errno));
	    exit(1);
	}
    } else {
	fp = stdin;
    }

    if (dosync) {
	fprintf(stderr,"trying to sync\n");
	ret = sc_tlm_sync_major(fp);
	fprintf(stderr,"done with sync\n");
	if (ret == 0) {
	    exit(0);
	} else if (ret < 0) {
	    fprintf(stderr,"%s: error in sync_major", argv[0]);
	    if (ferror(fp)) {
		fprintf(stderr, " (%s)", strerror(errno));
	    }
	    fprintf(stderr, "\n");
	    exit(1);
	}
	fprintf(stderr,"sync after %d minor frames\n", ret);
    } else {
	fprintf(stderr,"Not attempting to sync\n");
    }

    while (1) {
	cmdlong = evtlong = unpackerr = 0;
	ret = sc_tlm_next_minor(&m, fp);
	if (ret == 0) {
	    exit(0);
	} else if (ret == -1) {
	    fprintf(stderr,"%s: error after %d.%d frames", argv[0],
		majorcnt, m.seq);
	    if (ferror(fp)) {
		fprintf(stderr, " (%s)", strerror(errno));
	    }
	    fprintf(stderr, "\n");
	    exit(1);
	} else if (ret == -5) {
	    if (feof(fp)) {
		fprintf(stderr, "EOF after %d.%d frames\n", majorcnt, m.seq);
	    } else if (ferror(fp)) {
		fprintf(stderr, "error (%s) after %d.%d frames\n", 
		    strerror(errno), majorcnt, m.seq);
	    } else {
		fprintf(stderr,"%s: malformed intro around frame %d.%d\n",
		    argv[0], majorcnt, m.seq);
	    }
	    exit(1);
	} else {
	    if (ret == -2) {
		cmdlong = 1;
	    } else if (ret == -3) {
		evtlong = 1;
	    } else if (ret == -4) {
		unpackerr = 1;
	    } else if (ret == -7) {
		printf("******* ZEROED DATA *******\n");
		continue;
	    }
	    if (m.major_frame_start) {
		majorcnt++;
		if (majorcnt != lastmajor + 1) {
		    printf("=============== break in major number! ======\n");
		}
		printf("Major %d\n", majorcnt);
		lastmajor = majorcnt;
	    }
	    if (m.seq != lastminor + 1) {
		printf("++++++++++++ break in our minor number! ++++++\n");
	    }
	    lastminor = m.seq;
	    if (lastminor == 255) {
		lastminor = -1;
	    }
	    sec1970 = sc_tlm_sec1970(&(m.s));

	    tm = gmtime((time_t *)&sec1970);
	    second = tm->tm_sec;

	    if (!Firsttime) {
		if (second - lastsecond != 1) {
		    printf("----------------- break in time! ----------\n");
		}
	    } else {
		Firsttime = 0;
	    }
	    lastsecond = second;
	    if (lastsecond == 59) {
		lastsecond = -1;
	    }
	    if (m.hsk[0] & SC_TLM_SYNC) {
		printf("\tminor %03d+", m.seq);
	    } else {
		printf("\tminor %03d", m.seq);
	    }
	    printf(" %u", m.s.PacketSecondaryHeader);
	    printf(" %d/%02d", m.s.MajorFrameCount, m.s.MinorFrameCount);

	    printf("\t%02d/%02d/%02d %02d:%02d:%02d",
	    	tm->tm_mon+1, tm->tm_mday, tm->tm_year, tm->tm_hour,
		tm->tm_min, tm->tm_sec);

	    if (m.diag) {
		printf("\tDIAG %d", m.diag);
	    } else {
		if (evtlong) {
		    printf("\tnorm len too long");
		} else if (unpackerr) {
		    printf("\tnorm unpack err");
		} else {
		    printf("\tnorm %d", m.nevt);
		    for (i=0; i<m.nevt; i++) {
			printf(" %d", m.e[i].nbytes);
			if (m.e[i].nbytes > 150) {
			    printf(" [");
			    for (j=0; j<7; j++) {
				printf(" %02x", m.e[i].p[j]);
			    }
			    printf(" ]");
			}
		    }
		}
	    }
	    if (cmdlong) {
		printf("\tCMD too long");
	    } else if (m.hsk[0] & SC_TLM_CMD) {
		printf("\tCMD");
		printf(" %s", m.cmd);
	    }
	    printf("\n");
	}
    }
}
