/* selminor - process events in a major, minor frame range
 *
 *	USAGE: selminor startmajor startminor endmajor endminor [infile]
 *
 * Marty Olevitch, Feb 1997
 */

#include <stdio.h>
#include <string.h>	/* memcpy */
#include <errno.h>
#include <softcris.h>

int
main(argc, argv)
    int argc;
    char *argv[];
{
    struct sc_crisEvt c;
    int cmdlong;
    struct sc_diagEvt d;
    int emaj, emin;	/* ending major, minor frame no. */
    int endfrm;		/* ending frame cnt */
    int evtlong;
    FILE *fp;
    int frmcnt;		/* current frame cnt */
    int i;
    int j;
    struct sc_minorFrame m;
    int majorcnt = 0;
    int ret;
    int smaj, smin;	/* starting major, minor frame no. */
    int startfrm;	/* starting frame cnt */
    int unpackerr;

    if (argc < 5 || argc > 6) {
	fprintf(stderr,"USAGE:selminor startmajor startminor endmajor endminor [infile]\n", argv[0]);
	exit(1);
    }

    if (argc == 6) {
	fp = fopen(argv[5], "r");
	if (fp == NULL) {
	    fprintf(stderr,"%s: can't open '%s' (%s)\n", argv[0], argv[5],
		strerror(errno));
	    exit(1);
	}
    } else {
	fp = stdin;
    }

    smaj = atoi(argv[1]);
    smin = atoi(argv[2]);
    emaj = atoi(argv[3]);
    emin = atoi(argv[4]);

    startfrm = (smaj * 256) + smin;
    endfrm = (emaj * 256) + emin;

    fprintf(stderr,"trying to sync\n");
    ret = sc_tlm_sync_major(fp);
    fprintf(stderr,"done with sync\n");
    if (ret == 0) {
	exit(0);
    } else if (ret < 0) {
	fprintf(stderr,"%s: error in sync_major", argv[0]);
	if (ferror(fp)) {
	    fprintf(stderr, " (%s)", strerror(errno));
	}
	fprintf(stderr, "\n");
	exit(1);
    }
    fprintf(stderr,"sync after %d minor frames\n", ret);
    majorcnt++;
    while (1) {
	cmdlong = evtlong = unpackerr = 0;
	ret = sc_tlm_next_minor(&m, fp);
	if (ret == 0) {
	    exit(0);
	} else if (ret == -1) {
	    fprintf(stderr,"%s: error after %d.%d frames", argv[0],
		majorcnt, m.seq);
	    if (ferror(fp)) {
		fprintf(stderr, " (%s)", strerror(errno));
	    }
	    fprintf(stderr, "\n");
	    exit(1);
	} else if (ret == -5) {
	    if (feof(fp)) {
		fprintf(stderr, "EOF after %d.%d frames\n", majorcnt, m.seq);
	    } else if (ferror(fp)) {
		fprintf(stderr, "error (%s) after %d.%d frames\n", 
		    strerror(errno), majorcnt, m.seq);
	    } else {
		fprintf(stderr,"%s: malformed intro around frame %d.%d\n",
		    argv[0], majorcnt, m.seq);
	    }
	    exit(1);
	} else if (ret == -2) {
	    cmdlong = 1;
	} else if (ret == -3) {
	    evtlong = 1;
	} else if (ret == -4) {
	    unpackerr = 1;
	} else {
	    if (m.major_frame_start) {
		majorcnt++;
	    }
	    frmcnt = (majorcnt * 256) + m.seq;
	    if (frmcnt >= startfrm && frmcnt <= endfrm) {
		/* process data */
		if (m.diag) {
		    ret = sc_tlm_next_diag(&d, fp);
		    printf("major: %d  minor: %d", majorcnt, m.seq);
		    printf("\tDIAG\tnblobs: %d\n", d.e.nblobs);
		} else {
		    ret = sc_tlm_next_cris_evt(&c, fp);
		    if (ret >= 0) {
			printf("major: %d  minor: %d", majorcnt, m.seq);
			printf("\tNORM\tnblobs: %d\n", c.nblobs);
		    }
		}
	    }
	}
    }
}
