/* rdminor - process the cris minor frames
 *
 *	Raw telemetry data processed by cris_test_extr is the input 
 *	to this program. See end of file for a description of the 
 *	CrisData format.
 *
 * Marty Olevitch, Jan 1997
 *
 *	Compile: gcc -o testprog testprog.c -L/usr/local/lib -lsoftcris -lm
 */

#include <stdio.h>
#include <string.h>	/* memcpy */
#include <errno.h>
#include <softcris.h>

int
main(argc, argv)
    int argc;
    char *argv[];
{
    int cmdlong;
    int evtlong;
    FILE *fp;
    int i;
    int j;
    struct sc_minorFrame m;
    int majorcnt = 0;
    int ret;
    int unpackerr;

    if (argc == 2) {
	fp = fopen(argv[1], "r");
	if (fp == NULL) {
	    fprintf(stderr,"%s: can't open '%s' (%s)\n", argv[0], argv[1],
		strerror(errno));
	    exit(1);
	}
    } else {
	fp = stdin;
    }

    ret = sc_tlm_sync_major(fp);
    if (ret == 0) {
	exit(0);
    } else if (ret < 0) {
	fprintf(stderr,"%s: error in sync_major", argv[0]);
	if (ferror(fp)) {
	    fprintf(stderr, " (%s)", strerror(errno));
	}
	fprintf(stderr, "\n");
	exit(1);
    }
    fprintf(stderr,"sync after %d minor frames\n", ret);
    majorcnt++;
    /* printf("Major %d\n", majorcnt); */
    while (1) {
	cmdlong = evtlong = unpackerr = 0;
	ret = sc_tlm_next_minor(&m, fp);
	if (ret == 0) {
	    exit(0);
	} else if (ret == -1) {
	    fprintf(stderr,"%s: error after %d.%d frames", argv[0],
		majorcnt, m.seq);
	    if (ferror(fp)) {
		fprintf(stderr, " (%s)", strerror(errno));
	    }
	    fprintf(stderr, "\n");
	    exit(1);
	} else if (ret == -5) {
	    if (feof(fp)) {
		fprintf(stderr, "EOF after %d.%d frames\n", majorcnt, m.seq);
	    } else if (ferror(fp)) {
		fprintf(stderr, "error (%s) after %d.%d frames\n", 
		    strerror(errno), majorcnt, m.seq);
	    } else {
		fprintf(stderr,"%s: malformed intro around frame %d.%d\n",
		    argv[0], majorcnt, m.seq);
	    }
	    exit(1);
	} else if (ret == -2) {
	    cmdlong = 1;
	} else if (ret == -3) {
	    evtlong = 1;
	} else if (ret == -4) {
	    unpackerr = 1;
	} else {
	    if (m.major_frame_start) {
		majorcnt++;
	    }
	    if (m.diag) {
		/* printf("\tDIAG"); */
	    } else {
		if (evtlong) {
		    /* printf("\tnorm len too long"); */
		} else if (unpackerr) {
		    /* printf("\tnorm unpack err"); */
		} else {
		    /* printf("\tnorm %d", m.nevt); */
		    for (i=0; i<m.nevt; i++) {
			printf("%d", m.e[i].nbytes);
			for (j=0; j < m.e[i].nblobs; j++) {
			    printf(" %d %d %u", m.e[i].hodo[j].x,
			    	m.e[i].hodo[j].y, m.e[i].hodo[j].I);
			}
			printf("\n");
		    }
		}
	    }
	}
    }
}
