/* rdminor - process the cris minor frames
 *
 *	Raw telemetry data processed by cris_test_extr is the input 
 *	to this program. See end of file for a description of the 
 *	CrisData format.
 *
 * Marty Olevitch, Jan 1997
 *
 *	Compile: gcc -o testprog testprog.c -L/usr/local/lib -lsoftcris -lm
 */

#include <stdio.h>
#include <string.h>	/* memcpy */
#include <errno.h>
#include <softcris.h>

int
main(argc, argv)
    int argc;
    char *argv[];
{
    struct sc_crisEvt c;
    FILE *fp;
    int i;
    int j;
    int ret;

    if (argc == 2) {
	fp = fopen(argv[1], "r");
	if (fp == NULL) {
	    fprintf(stderr,"%s: can't open '%s' (%s)\n", argv[0], argv[1],
		strerror(errno));
	    exit(1);
	}
    } else {
	fp = stdin;
    }

    ret = sc_tlm_sync_major(fp);
    if (ret == 0) {
	exit(0);
    } else if (ret < 0) {
	fprintf(stderr,"%s: error in sync_major", argv[0]);
	if (ferror(fp)) {
	    fprintf(stderr, " (%s)", strerror(errno));
	}
	fprintf(stderr, "\n");
	exit(1);
    }
    fprintf(stderr,"sync after %d minor frames\n", ret);
    while (1) {
	ret = sc_tlm_next_cris_evt(&c, fp);
	if (ret == 0) {
	    exit(0);
	} else if (ret == -1) {
	    fprintf(stderr,"%s: error", argv[0]);
	    if (ferror(fp)) {
		fprintf(stderr, " (%s)", strerror(errno));
	    }
	    fprintf(stderr, "\n");
	    exit(1);
	} else if (ret == -5) {
	    if (feof(fp)) {
		fprintf(stderr, "EOF\n");
	    } else if (ferror(fp)) {
		fprintf(stderr, "error (%s)\n", strerror(errno));
	    } else {
		fprintf(stderr,"%s: malformed intro\n", argv[0]);
	    }
	    exit(1);
	} else if (ret == -2) {
	    ;
	} else if (ret == -3) {
	    ;
	} else if (ret == -4) {
	    ;
	} else {
	    printf("%d", c.nbytes);
	    printf(" cam %d", sc_tlm_cam_guess());
	    printf(" mcp1 %d mcp2 %d", sc_tlm_hsk_mcp1(), sc_tlm_hsk_mcp2());
	    printf("\n");
	}
    }
}
