/* conv_test_data - convert the test data files to little-endian 
 *
 * USAGE
 *	conv_test_data infile > outfile
 *
 * COMPILE
 *	cc -o conv_test_data conv_test_data.c -L/usr/local/lib -lsoftcris
 *
 * Marty Olevitch, Nov 1996
 */

#include <stdio.h>
#include <string.h>	/* strerror */
#include <errno.h>	/* errno decl */
#include <softcris.h>
#include <msu.h>

char *Progname;

int
main(int argc, char *argv[])
{
    u_char b[60000];
    struct sc_msuEvt e;
    char *filename;
    int gotevt = 0;
    int i;
    FILE *ifp;
    u_char *p;
    int ret;
    long size, vidsize1, vidsize2;
    u_short exp_id;

    Progname = argv[0];
    if (argc != 2) {
	fprintf(stderr, "USAGE: %s inputfile > outfile\n", Progname);
	exit(1);
    }

    filename = argv[1];

    ifp = fopen(filename, "r");
    if (ifp == (FILE *)NULL) {
	fprintf(stderr, "%s: can't fopen '%s' (%s)\n",
	    Progname, filename, strerror(errno));
	return -1;
    }

    ret = sc_msu_init(ifp, NULL, 0, Progname);
    if (ret) {
	fprintf(stderr, "%s: bad sc_msu_init()", Progname);
	return -1;
    }

    while (sc_msu_next_evt(&e) == 0) {
	p = b;
	fprintf(stderr, "Evt no. %ld\n", e.abs);
	ret = sc_msu_reorder_std(e.ptr, p);
	if (ret) {
	    fprintf(stderr, "%s: reorder_std returned %d (not 0)\n",
		Progname, ret);
	    return -1;
	}
	
	p += STDSIZ;
	if (e.type == T_REAL) {
	    fprintf(stderr, "\treal\n");
	    ret = sc_msu_reorder_real(e.ptr, p, &size, &vidsize1, 
		&vidsize2, &exp_id);
	    if (ret) {
		fprintf(stderr,"%s: reorder_real returned %d\n", Progname, ret);
		return -1;
	    }
	} else if (e.type == T_REPLY) {
	    fprintf(stderr, "\treply\n");
	    size = e.size;
	    ret = sc_msu_reorder_reply(e.ptr, p);
	    if (ret) {
		fprintf(stderr,"%s: reorder_reply returned %d\n", Progname, ret);
		return -1;
	    }
	} else {
	    fprintf(stderr, "%s: sorry can't do event type '%d' yet\n",
	    	Progname, e.type);
	    return -1;
	}
	ret = fwrite(b, 1, size, stdout);
	if (ret != size) {
	    fprintf(stderr, "%s: bad write to output file (%s)\n",
	    	Progname, strerror(errno));
	    exit(1);
	}
    }

    return 0;
}
