/* cris_raw_extr - extract sc_crisEvt struct to stdout
 *
 * usage: cris_raw_extr -n n [-s] [infile ...]
 *	-n n	event no. in input (sequence starts with 0)  
 *	-s	don't sync at start of file
 *
 * Marty Olevitch, Feb 1998 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <local.h>
#include "softcris.h"

int Debug = 0;
char *Progname;

char **Argvp;				/* scan pointer for nextfile() */
char *Nullargv[] = { "-", NULL };	/* dummy argv for case of no args */
char *Inname;				/* filename for messages etc. */
u_long Bytecnt;				/* line number for messages etc. */
FILE *In = NULL;			/* current input file */

#ifdef UTZOOERR
extern char *mkprogname();
#else
#define	mkprogname(a)	(a)
#endif

char *nextfile(void);

#define NTRAJ 100
int Ntraj = NTRAJ;

int Nevt = -1;
int Sync = 1;

/* main - parse arguments and handle options */
main(argc, argv)
    int argc;
    char *argv[];
{
    int c;
    int errflg = 0;
    extern int optind;
    extern char *optarg;
    void process(void);
    int ret;

    Progname = mkprogname(argv[0]);

    while ((c = getopt(argc, argv, "dn:s")) != EOF) {
	switch (c) {
	case 'd':	/* Debugging. */
	    Debug++;
	    break;
	case 'n':
	    Nevt = atoi(optarg);
	    break;
	case 's':
	    Sync = 0;
	    break;
	case '?':
	default:
	    errflg++;
	    break;
	}
    }
    if (errflg) {
	fprintf(stderr, "usage: %s ", Progname);
	fprintf(stderr, "[-d] [-n n] [file ...]\n");
	exit(2);
    }

    if (optind >= argc) {
	Argvp = Nullargv;
    } else {
	Argvp = &argv[optind];
    }

    if (Nevt == -1) {
	fprintf(stderr,"%s: no -n option specified!\n");
	fprintf(stderr, "usage: %s ", Progname);
	fprintf(stderr, "[-d] [-n n] [file ...]\n");
	exit(2);
    }

    Inname = nextfile();
    if (Inname != NULL) {
	process();
    }
    exit(3);
}

/* nextfile - switch files */
char *				/* filename */
nextfile(void)
{
    char *namep;
    struct stat statbuf;

    if (In != NULL) {
	(void) fclose(In);
    }

    namep = *Argvp;
    if (namep == NULL) {	/* no more files */
	return(NULL);
    }
    Argvp++;

    if (STREQ(namep, "-")) {
	In = stdin;
	namep = "stdin";
    } else {
	In = efopen(namep, "r");
	if (fstat(fileno(In), &statbuf) < 0) {
	    errmesg("can't fstat `%s'", namep);
	}
	if ((statbuf.st_mode & S_IFMT) == S_IFDIR) {
	    errmesg("`%s' is directory!", namep);
	}
    }

    return(namep);
}


/* process - process input data */
void
process(void)
{
    struct sc_crisEvt c;
    static int evtcnt = 0;
    int ret;

    if (Sync) {
	ret = sc_tlm_sync_major(In);
	if (ret < 0) {
	    fprintf(stderr, "sync error (ret = %d)\n", ret);
	    return;
	}
	fprintf(stderr, "sync after %d minor frames\n", ret);
    }

    while (1) {
    	ret = sc_tlm_next_cris_evt(&c, In);
	if (ret == 0) {
	    evtcnt--;
	    Inname = nextfile();
	    if (Inname == NULL) {
		break;
	    }
	    continue;
	} else if (ret < -1) {
	    continue;
	}
	if (evtcnt == Nevt) {
	    ret = fwrite(c.p, 1, c.nbytes, stdout);
	    if (ret != c.nbytes) {
		exit(1);
	    } else {
		exit(0);
	    }
	}
	evtcnt++;
    }
}
