/* msu_trcnt - print count of number of trajectories per event
 *
 * usage: msu_trcnt [-v] [-m method] [file ...]
 *	-v = verbose mode
 *
 * Marty Olevitch, Feb 1998 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <local.h>
#include "softcris.h"

int Debug = 0;
char *Progname;

char **Argvp;				/* scan pointer for nextfile() */
char *Nullargv[] = { "-", NULL };	/* dummy argv for case of no args */
char *Inname;				/* filename for messages etc. */
u_long Bytecnt;				/* line number for messages etc. */
FILE *In = NULL;			/* current input file */

#ifdef UTZOOERR
extern char *mkprogname();
#else
#define	mkprogname(a)	(a)
#endif

char *nextfile(void);

#define NTRAJ 100
int Ntraj = NTRAJ;

static int Tot = 0;
static int Ret0 = 0;
static int Ret1 = 0;

int Method = 8;
int Verbose = 0;

/* main - parse arguments and handle options */
main(argc, argv)
    int argc;
    char *argv[];
{
    int c;
    int errflg = 0;
    extern int optind;
    extern char *optarg;
    void process(void);
    int ret;

    Progname = mkprogname(argv[0]);

    while ((c = getopt(argc, argv, "dm:v")) != EOF) {
	switch (c) {
	case 'd':	/* Debugging. */
	    Debug++;
	    break;
	case 'm':
	    Method = atoi(optarg);
	    break;
	case 'v':	/* verbose mode */
	    Verbose++;
	    break;
	case '?':
	default:
	    errflg++;
	    break;
	}
    }
    if (errflg) {
	fprintf(stderr, "usage: %s ", Progname);
	fprintf(stderr, "[-d] [-v] [file ...]\n");
	exit(2);
    }

    if (optind >= argc) {
	Argvp = Nullargv;
    } else {
	Argvp = &argv[optind];
    }

    ret = sc_msu_init(NULL, NULL, 0, "msu_trcnt");
    if (ret != 0) {
	fprintf(stderr,"%s: bad sc_msu_init\n", Progname);
	exit(1);
    }

    ret = sc_traj_select_method(Method);

    Inname = nextfile();
    if (Inname != NULL) {
	process();
    }
    exit(0);
}

/* nextfile - switch files */
char *				/* filename */
nextfile(void)
{
    char *namep;
    struct stat statbuf;

    if (In != NULL) {
	(void) fclose(In);
    }

    namep = *Argvp;
    if (namep == NULL) {	/* no more files */
	return(NULL);
    }
    Argvp++;

    if (STREQ(namep, "-")) {
	In = stdin;
	namep = "stdin";
    } else {
	In = efopen(namep, "r");
	if (fstat(fileno(In), &statbuf) < 0) {
	    errmesg("can't fstat `%s'", namep);
	}
	if ((statbuf.st_mode & S_IFMT) == S_IFDIR) {
	    errmesg("`%s' is directory!", namep);
	}
    }

    return(namep);
}


/* process - process input data */
void
process(void)
{
    struct sc_msuEvt c;
    int evtcnt;
    int errcode;
    char *errstr;
    int i;
    struct sc_softCoords k1, k2;
    int ntraj1;
    int ntraj2;
    int ret;
    int ret1;
    int ret2;
    struct sc_softTraj t1;
    struct sc_softTraj t2;

    evtcnt = 0;

    while (1) {
	evtcnt++;
    	ret = sc_msu_next_evt(&c);
	if (ret < 0) {
	    evtcnt--;
	    Inname = nextfile();
	    if (Inname == NULL) {
		break;
	    }
	    continue;
	}
	Tot++;

	if (!c.valid) {
	    continue;
	}
	ret = sc_msu_fix_blobs(&c);
	if (ret != 0) {
	    continue;
	}

	ret = sc_msu_coords(&c, &k1, &k2);
	if ((ret & 0x10000) == 0) {
	    ret1 = sc_trajectory(&k1, &t1);
	}
	if ((ret & 0x20000) == 0) {
	    ret2 = sc_trajectory(&k2, &t2);
	}

	if (ret == 0 && ret1 >= 0 && ret2 >= 0) {
	    Ret0++;
	    printf("%d\n", Tot);
	} else {
	    Ret1++;
	    if (Verbose) {
		errcode = sc_get_err_code();
		errstr = sc_get_err_mesg();
		printf("%d %d %s\n", Tot, errcode, errstr);
	    }
	}
    }
}
