/* msuevt_extr - print count of number of trajectories per event
 *
 * usage: msuevt_extr -n n < infile > outfile
 *	-n n	event no. in input (sequence starts with 0)
 *
 * WARNING: the sc_msuEvt struct has some pointers in it. If it is saved to
 * disk, the pointers will become useless.
 *
 * Marty Olevitch, Feb 1998 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <local.h>
#include "softcris.h"

int Debug = 0;
char *Progname;

char **Argvp;				/* scan pointer for nextfile() */
char *Nullargv[] = { "-", NULL };	/* dummy argv for case of no args */
char *Inname;				/* filename for messages etc. */
u_long Bytecnt;				/* line number for messages etc. */
FILE *In = NULL;			/* current input file */

#ifdef UTZOOERR
extern char *mkprogname();
#else
#define	mkprogname(a)	(a)
#endif

char *nextfile(void);

int Nevt = -1;

/* main - parse arguments and handle options */
main(argc, argv)
    int argc;
    char *argv[];
{
    int c;
    int errflg = 0;
    extern int optind;
    extern char *optarg;
    void process(void);
    int ret;

    Progname = mkprogname(argv[0]);

    while ((c = getopt(argc, argv, "dn:")) != EOF) {
	switch (c) {
	case 'd':	/* Debugging. */
	    Debug++;
	    break;
	case 'n':
	    Nevt = atoi(optarg);
	    break;
	case '?':
	default:
	    errflg++;
	    break;
	}
    }
    if (errflg) {
	fprintf(stderr, "usage: %s ", Progname);
	fprintf(stderr, "[-d] [-v] [file ...]\n");
	exit(2);
    }

    if (optind >= argc) {
	Argvp = Nullargv;
    } else {
	Argvp = &argv[optind];
    }

    if (Nevt == -1) {
	fprintf(stderr,"%s: no -n option specified!\n");
	fprintf(stderr, "usage: %s ", Progname);
	fprintf(stderr, "[-d] [-n n] [file ...]\n");
	exit(2);
    }

    ret = sc_msu_init(NULL, NULL, 0, "msuevt_extr");
    if (ret != 0) {
	fprintf(stderr,"%s: bad sc_msu_init\n", Progname);
	exit(1);
    }

    Inname = nextfile();
    if (Inname != NULL) {
	process();
    }
    exit(0);
}

/* nextfile - switch files */
char *				/* filename */
nextfile(void)
{
    char *namep;
    struct stat statbuf;

    if (In != NULL) {
	(void) fclose(In);
    }

    namep = *Argvp;
    if (namep == NULL) {	/* no more files */
	return(NULL);
    }
    Argvp++;

    if (STREQ(namep, "-")) {
	In = stdin;
	namep = "stdin";
    } else {
	In = efopen(namep, "r");
	if (fstat(fileno(In), &statbuf) < 0) {
	    errmesg("can't fstat `%s'", namep);
	}
	if ((statbuf.st_mode & S_IFMT) == S_IFDIR) {
	    errmesg("`%s' is directory!", namep);
	}
    }

    return(namep);
}


/* process - process input data */
void
process(void)
{
    struct sc_msuEvt c;
    static int evtcnt = 0;
    int ret;

    while (1) {
    	ret = sc_msu_next_evt(&c);
	if (ret < 0) {
	    evtcnt--;
	    Inname = nextfile();
	    if (Inname == NULL) {
		break;
	    }
	    continue;
	}
	if (evtcnt == Nevt) {
	    ret = fwrite(&c, sizeof(struct sc_msuEvt), 1, stdout);
	    if (ret != 1) {
		exit(1);
	    } else {
		exit(0);
	    }
	}
	evtcnt++;
    }
}
