/* trcnt - print count of number of trajectories per event
 *
 * usage: trcnt [-v] [file ...]
 *	-v = verbose mode
 *
 * Marty Olevitch, Feb 1998 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <local.h>
#include "softcris.h"

int Debug = 0;
char *Progname;

char **Argvp;				/* scan pointer for nextfile() */
char *Nullargv[] = { "-", NULL };	/* dummy argv for case of no args */
char *Inname;				/* filename for messages etc. */
u_long Bytecnt;				/* line number for messages etc. */
FILE *In = NULL;			/* current input file */

#ifdef UTZOOERR
extern char *mkprogname();
#else
#define	mkprogname(a)	(a)
#endif

char *nextfile(void);

#define NTRAJ 100
int Ntraj = NTRAJ;

static int Tot = 0;
static int Ret0 = 0;
static int Ret1 = 0;

int Verbose = 0;

/* main - parse arguments and handle options */
main(argc, argv)
    int argc;
    char *argv[];
{
    int c;
    int errflg = 0;
    extern int optind;
    extern char *optarg;
    void process(void);
    int ret;

    Progname = mkprogname(argv[0]);

    while ((c = getopt(argc, argv, "dv")) != EOF) {
	switch (c) {
	case 'd':	/* Debugging. */
	    Debug++;
	    break;
	case 'v':	/* verbose mode */
	    Verbose++;
	    break;
	case '?':
	default:
	    errflg++;
	    break;
	}
    }
    if (errflg) {
	fprintf(stderr, "usage: %s ", Progname);
	fprintf(stderr, "[-d] [-v] [file ...]\n");
	exit(2);
    }

    if (optind >= argc) {
	Argvp = Nullargv;
    } else {
	Argvp = &argv[optind];
    }

    sc_cris_set_cam(SC_CAM2);
    ret = sc_traj_select_method(8);

    Inname = nextfile();
    if (Inname != NULL) {
	process();
    }
    exit(0);
}

/* nextfile - switch files */
char *				/* filename */
nextfile(void)
{
    char *namep;
    struct stat statbuf;

    if (In != NULL) {
	(void) fclose(In);
    }

    namep = *Argvp;
    if (namep == NULL) {	/* no more files */
	return(NULL);
    }
    Argvp++;

    if (STREQ(namep, "-")) {
	In = stdin;
	namep = "stdin";
    } else {
	In = efopen(namep, "r");
	if (fstat(fileno(In), &statbuf) < 0) {
	    errmesg("can't fstat `%s'", namep);
	}
	if ((statbuf.st_mode & S_IFMT) == S_IFDIR) {
	    errmesg("`%s' is directory!", namep);
	}
    }

    return(namep);
}


/* process - process input data */
void
process(void)
{
    struct sc_crisEvt c;
    int evtcnt;
    int errcode;
    char *errstr;
    int i;
    int ntraj;
    int ret;
    struct sc_Coords_Traj t[NTRAJ];

    ret = sc_tlm_sync_major(In);
    if (ret < 0) {
	fprintf(stderr, "sync error (ret = %d)\n", ret);
	return;
    }
    fprintf(stderr, "sync after %d minor frames\n", ret);
    evtcnt = 0;

    while (1) {
	evtcnt++;
    	ret = sc_tlm_next_cris_evt(&c, In);
	if (ret == 0) {
	    evtcnt--;
	    Inname = nextfile();
	    if (Inname == NULL) {
		break;
	    }
	    continue;
	} else if (ret < -1) {
	    continue;
	}

	Tot++;
	ntraj = NTRAJ;
	ret = sc_cris_traj(&c, t, &ntraj);
	if (ret >= 0) {
	    Ret0++;
	    printf("%d %d\n", Tot, ntraj);
	} else {
	    Ret1++;
	    if (Verbose) {
		errcode = sc_get_err_code();
		errstr = sc_get_err_mesg();
		printf("%d %d %s\n", Tot, errcode, errstr);
	    }
	}
    }
}
