/* charge.c - implementation of Mike Thayer's Z algorithm
 *
 * NOTE: I don't know how or if most of this code works. I just copied a
 * lot of it from Mike's original program.
 *
 * reworked by Marty Olevitch, Mar 1997
 */

static char cvsid[] = "$Id: charge.c,v 1.2 1997/10/08 15:31:59 marty Exp $";

#include <math.h>
#include "softcris.h"

/* originally from nucleus.h: 12/10/96, MRT
 * 
 * used ... to pick out which set of coefficients ... to use. */
int nucleus[40][2] = {
    {1,1},  {2,4},  {3,7},  {4,9},  {5,11}, {6,12}, {7,14}, {8,16},
    {9,19}, {10,20},{11,23},{12,24},{13,27},{14,28},{15,31},{16,32},
    {17,35},{18,40},{19,39},{20,40},{21,45},{22,48},{23,51},{24,52},
    {25,55},{26,56},{27,59},{28,58},{29,63},{30,64},{31,69},{32,74},
    {33,76},{34,80},{35,79},{36,84},{37,85},{38,88},{39,89},{40,90},
};

static double diffgen(int z, int a, double totalE);
static double best_z(double *diff3, int *z3);
static double pathlength(int detector, double x0, double y0, double theta,
    int deadflag);
static double deaden(int z, int a, int detector, double theta, double Eprime);
static void phamev(int primtel, short *pha, double *MeV);

/* sc_z - fill in the sc_crisZ charge struct. Returns:
 *	0 if all went well
 *	-2 = bad range
 *	-3 = stim evt 
 *	-4 = no min or bad min
 */
int
sc_z(c, t, z)
    struct sc_crisEvt *c;
    struct sc_softTraj *t;
    struct sc_crisZ *z;
{
    double deadE;
    double deltaE;
    int detector;
    double diff[40];
    double diff3[3];
    double Eprime;
    int i;
    double MeV_hybrid[SC_CRIS_NCTRPH - 1];
    int range;
    double RHS;
    double smallest;
    int smallZ;
    double T_sectheta;
    int tel;
    double x[SC_CRIS_NCTRPH - 1];
    double x0;
    double y[SC_CRIS_NCTRPH - 1];
    double y0;
    int z3[3];

    if (c->R1 <= 1)  {
	return -2;
    }

    if (c->stim) {
	return -3;
    }

    /* Need to determine which telescope the event occurred in.  Look in
     * E1A(C) and E1B(D); whichever pulse ht is larger is primary stack. */
    if (c->prim_tel_pair == SC_CRIS_AB) {
	if (c->prim_ctr_ph[0] > c->prim_ctr_ph[1]) {
	    tel = 0;	/* tel A */
	} else {
	    tel = 1;	/* tel B */
	}
    } else {
	if (c->prim_ctr_ph[0] > c->prim_ctr_ph[1]) {
	    tel = 2;	/* tel C */
	} else {
	    tel = 3;	/* tel D */
	}
    }

    /* pha -> MeV conversion */
    phamev(c->prim_tel_pair, c->prim_ctr_ph, MeV_hybrid);
    if (tel == 0 || tel == 2) {
	/* tel A or C */
	z->MeV[0] = MeV_hybrid[0];
    } else {
	/* tel B or D */
	z->MeV[0] = MeV_hybrid[1];
    }
    for (i=1; i < SC_CRIS_NCTRPH - 1; i++) {
	z->MeV[i] = MeV_hybrid[i+1];
    }

    x[0] = t->stackx[SC_E1];
    x[1] = t->stackx[SC_E2];
    x[2] = t->stackx[SC_E3];
    x[3] = t->stackx[SC_E4];
    x[4] = t->stackx[SC_E5];
    x[5] = t->stackx[SC_E6];
    x[6] = t->stackx[SC_E7];
    x[7] = t->stackx[SC_E8];
    x[8] = t->stackx[SC_E9];

    y[0] = t->stacky[SC_E1];
    y[1] = t->stacky[SC_E2];
    y[2] = t->stacky[SC_E3];
    y[3] = t->stacky[SC_E4];
    y[4] = t->stacky[SC_E5];
    y[5] = t->stacky[SC_E6];
    y[6] = t->stacky[SC_E7];
    y[7] = t->stacky[SC_E8];
    y[8] = t->stacky[SC_E9];

    z->range = range = (int)c->R1;

    /* find T_sectheta (thickness above stopping detector) 
     * Detector ID: 1 = stack1A, 2 = stack2A, ..... 38 = stack8D, 
     *     39 = stack9D */

    x0 = x[range-1];
    y0 = y[range-1];

    /* range should go from 2 through 9, so this is proper formula to find
     * "detector above stopping". */
    detector = (tel*10) + (range-1); 

    /* correct for deadlayer in T_sectheta value */
    T_sectheta = pathlength(detector, x0, y0, t->theta, 1);

    Eprime = z->MeV[range-1];
    deltaE = z->MeV[range-2];

    /* find DIFF quantities; i = Z - 1 */

    for (i = 0; i < 40; i++) {
	/* need to calculate deadE for appropriate Z */
	deadE =
	  deaden(nucleus[i][0], nucleus[i][1], detector, t->theta, Eprime);

	RHS = 
	  diffgen(nucleus[i][0], nucleus[i][1], Eprime+deltaE+deadE) -
	  diffgen(nucleus[i][0], nucleus[i][1], Eprime);

	diff[i] = fabs(T_sectheta - RHS);

	/* we do this because we want to find smallest absolute diff */
	if (i == 0) { 
	    smallest = diff[i]; 
	    smallZ = i+1; 
	} else if (diff[i] < smallest) { 
	    smallest = diff[i]; 
	    smallZ = i+1; 
	}
    }

    /* pick out DIFFs for parabola fit: smallest DIFF + one on each
     * side */

    if ((smallZ != 1) && (smallZ != 40)) {
	diff3[0] = diff[smallZ-2];
	diff3[1] = diff[smallZ-1];
	diff3[2] = diff[smallZ];
	z3[0] = smallZ-1;
	z3[1] = smallZ;
	z3[2] = smallZ+1; 
	z->zbest = best_z(diff3,z3);
    } else { 
	/* Z is just determined by smallest DIFF */
	z->zbest = (double) smallZ; 
    }
    if (z->zbest < 0) { 
	/* will occur if no min found or bad min found */
	return -4;
    }
    return 0;
}

/* pha -> MeV conversion - Originally from phamev.h: 12/6/96, MRT 
 * 	
 * Currently using INST values.  Can change this any time as appropriate. 
 * gain[0][] = PH1A,PH1B,PH2AB,....PH9AB; similar for gain[1][].
 * These values were updated 18 Aug 97 by Mike Thayer at Caltech and
 * finally installed the softcris library 8 Oct 97 by MAO. */

static double gain[2][10] = {
    {1.9793, 2.0090, 1.9854, 3.0619, 3.0337, 
             3.0590, 3.0598, 2.9992, 3.0142, 0.1334}, 
    {1.9834, 1.9542, 1.9534, 3.0093, 2.9805,
             3.0231, 2.9761, 3.0813, 2.9647, 0.1355}
};

static double offset[2][10] = {
    {47.6103, 48.0460, 47.6024, 47.7558, 49.6617, 
              48.3296, 48.5499, 47.9101, 49.0295, 48.3993}, 
    {48.2297, 48.4013, 48.2537, 47.3497, 48.3325, 
              47.8696, 48.6618, 47.7963, 48.2018, 46.0991}
};

/* originally from phamev.c: 12/10/96, MRT (under RCS) 
 * 
 * Will convert read-in PHAs to MeV (returned).  Currently using INST
 * values to do the conversion in phamev.h. */
static void phamev(int primtel, short *pha, double *MeV)
{
    int i;

    for (i=0; i<10; i++) {
	*(MeV+i) = gain[primtel][i] * (pha[i] - offset[primtel][i]);
    }
}

/* originally from detector.c: 12/30/96, MRT 
 * 
 * This subroutine generates the ranges for a given thickness, Z,
 * energy, etc.  Also generates "best Z".  Uses range tables from coefs.h.
 * CHANGED 12/30/96, MRT.  Now uses direct linear interpolation of
 * range-energy tables (in log-log space) to get proper range.  It's
 * probably slower (as it has to go through all 23 energies), but it
 * removes the "inverse" problem.
 * 
 * Also have included path_deadE.c here, as both files require use of
 * range_en.h */

/* originally from range.h:  MRT 12/30/96
 * Takes range-table values from Janni (H) and Hubert et al. (He-Zr),
 * puts them into array format.  range[i][Z-1] is array fmt, where i
 * indexes energy values as well (see energy.h) */

static double range[23][40] = {
    {16.1080,
    16.6100,13.6600,10.7500,9.14000,7.49000,6.85000,
    6.45000,6.57000,6.09000,6.28000,5.91000,6.06000,
    5.78000,5.93000,5.71000,5.86000,6.29000,5.82000,
    5.67000,6.07000,6.19000,6.30000,6.17000,6.28000,
    6.17000,6.28000,5.97000,6.28000,6.19000,6.49000,
    6.76000,6.67000,6.92000,6.67000,6.92000,6.84000,
     6.92000,6.84000,6.76000},
    {50.3700,
    50.8600,40.4600,30.3400,24.7200,19.5400,17.4400,
    15.9700,15.7700,14.1800,14.2100,13.0800,13.1500,
    12.3180,12.4290,11.7840,11.9180,12.6430,11.5350,
    11.1040,11.7610,11.8630,11.9530,11.5920,11.6890,
    11.3790,11.4820,10.8290,11.3040,11.0570,11.5040,
    11.9060,11.6680,12.0380,11.5270,11.8900,11.6870,
       11.7610,11.5680,11.3830},
    {89.2600,
    89.6200,70.6500,52.2700,42.0000,32.7700,28.9300,
    26.1900,25.5200,22.6500,22.4200,20.4300,20.3500,
    18.8900,18.9040,17.7840,17.8550,18.8220,17.0540,
    16.3170,17.1880,17.2420,17.2820,16.6770,16.7380,
    16.2180,16.2950,15.3050,15.9140,15.5080,16.0720,
    16.5780,16.1930,16.6560,15.8970,16.3500,16.0230,
       16.0800,15.7760,15.4860},
    {165.060,
    165.370,129.610,94.9800,75.5300,58.3300,51.0100,
    45.6900,44.0300,38.6300,37.8100,34.1200,33.6700,
    30.9900,30.7600,28.7100,28.6100,29.9700,26.9600,
    25.6400,26.8500,26.7800,26.7000,25.6300,25.6000,
    24.6800,24.6900,23.0900,23.9100,23.2000,23.9500,
    24.6200,23.9600,24.5700,23.3700,23.9500,23.4000,
       23.4200,22.9100,22.4300},
    {335.060,
    335.600,262.100,190.800,150.600,115.370,100.100,
    88.8700,84.8000,73.6500,71.3800,63.8200,62.4200,
    56.9600,56.0700,51.9300,51.3700,53.4300,47.7400,
    45.0900,46.9300,46.5300,46.1300,44.0400,43.7500,
    41.9700,41.7700,38.8900,40.0900,38.7400,39.8100,
    40.7600,39.5100,40.3600,38.2500,39.0800,38.0400,
       37.9400,37.0000,36.1200},
    {556.200,
    557.500,434.600,315.700,248.300,189.480,163.750,
    144.720,137.390,118.680,114.400,101.760,99.0100,
    89.9100,88.0800,81.1900,79.9400,82.8000,73.6500,
    69.2900,71.8400,70.9500,70.0700,66.6600,66.0000,
    63.1100,62.6100,58.1100,59.7300,57.5500,58.9700,
    60.2200,58.2200,59.3400,56.0900,57.1600,55.5200,
       55.2400,53.7600,52.3700},
    {826.575,
    828.000,645.000,467.800,367.400,279.750,241.200,
    212.600,201.220,173.260,166.440,147.570,143.120,
    129.540,126.500,116.250,114.100,117.840,104.510,
    98.0400,101.370,99.8500,98.3700,93.3500,92.2000,
    89.9500,87.0600,80.6200,82.6900,79.5100,81.3100,
    82.8700,79.9700,81.3500,76.7600,78.0900,75.7200,
       75.2200,73.0900,71.0900},
    {1141.30,
    1144.80,891.500,646.100,506.800,385.420,331.840,
    291.990,275.820,236.990,227.150,200.950,194.450,
    175.600,171.100,156.880,153.650,158.340,140.130,
    131.180,135.380,133.080,130.860,123.950,122.210,
    116.370,114.990,106.310,108.860,104.510,106.720,
    108.600,104.650,106.310,100.160,101.770,98.5500,
       97.7700,94.8800,92.1800},
    {1903.90,
    1910.00,1487.00,1076.60,843.600,640.600,550.600,
    483.500,455.700,390.500,373.300,329.300,317.800,
    286.100,278.000,254.100,248.200,255.000,225.100,
    210.100,216.200,212.000,207.900,196.400,193.200,
    183.500,180.900,166.800,170.400,163.300,166.300,
    168.900,162.400,164.700,154.800,157.000,151.700,
       150.300,145.600,141.200},
    {2832.40,
    2841.00,2211.00,1600.10,1253.10,950.900,816.600,
    716.400,674.200,577.000,550.700,485.000,467.200,
    419.900,407.200,371.600,362.300,371.600,327.300,
    305.000,313.400,306.700,300.200,283.200,278.000,
    263.600,259.500,238.900,243.700,233.100,237.100,
    240.400,230.900,233.700,219.500,222.200,214.500,
       212.100,205.200,198.800},
    {3917.40,
    3927.00,3055.00,2211.10,1731.10,1313.00,1127.00,
    988.000,929.200,794.400,757.400,666.400,641.200,
    575.900,557.700,508.300,494.900,507.000,446.000,
    415.100,425.900,416.300,407.100,383.500,376.100,
    356.200,350.100,322.000,328.100,313.500,318.500,
    322.700,309.500,313.000,293.600,297.000,286.300,
       282.900,273.400,264.600},
    {5150.90,
    5161.00,4015.00,2905.00,2274.00,1724.00,1479.00,
    1296.00,1218.60,1041.30,992.100,872.300,838.700,
    752.400,728.200,663.200,645.100,660.300,580.400,
    539.700,553.200,540.300,527.800,496.700,486.700,
    460.500,452.300,415.700,423.200,403.900,410.100,
    415.000,397.800,401.900,376.700,380.800,366.800,
       362.100,349.700,338.100},
    {6525.80,
    6534.00,5083.00,3678.00,2878.00,2182.00,1872.00,
    1640.00,1540.90,1316.10,1253.40,1101.40,1058.40,
    949.000,917.900,835.400,812.100,830.800,729.700,
    678.000,694.600,677.800,661.700,622.300,609.300,
    576.200,565.500,519.300,528.400,504.000,511.300,
    517.200,495.300,500.200,468.500,473.200,455.600,
       449.500,433.800,419.200},
    {8036.90,
    8042.00,6256.00,4526.00,3542.00,2685.00,2303.00,
    2017.00,1894.60,1617.80,1540.10,1352.90,1299.50,
    1164.60,1126.00,1024.30,995.200,1017.60,893.300,
    829.600,849.400,828.500,808.300,759.800,743.500,
    702.700,689.300,632.600,643.300,613.300,621.900,
    628.600,601.800,607.400,568.600,574.000,552.400,
       544.700,525.500,507.500},
    {9675.70,
    9678.00,7529.00,5446.00,4262.00,3230.00,2770.00,
    2426.00,2278.40,1945.00,1851.20,1625.70,1561.10,
    1398.60,1351.70,1229.20,1193.80,1220.10,1070.70,
    993.900,1017.20,991.700,967.200,908.700,888.800,
    839.600,823.200,755.300,767.600,731.500,741.400,
    749.100,716.800,723.100,676.600,682.800,656.900,
       647.400,624.300,602.700},
    {19645.0,
    19629.0,15268.0,11043.0,8640.00,6547.00,5613.00,
    4914.00,4613.00,3935.00,3743.00,3284.00,3151.00,
    2821.00,2724.00,2474.00,2400.00,2450.00,2148.00,
    1991.00,2035.00,1982.00,1930.00,1811.00,1769.00,
    1669.00,1634.00,1497.00,1519.00,1446.00,1463.00,
    1477.00,1411.00,1421.00,1328.00,1338.00,1286.00,
       1265.00,1218.00,1175.00},
    {32194.0,
    32202.0,25047.0,18116.0,14172.0,10738.0,9206.00,
    8057.00,7562.00,6450.00,6133.00,5380.00,5160.00,
    4617.00,4456.00,4046.00,3924.00,4003.00,3507.00,
    3250.00,3320.00,3230.00,3144.00,2948.00,2878.00,
    2713.00,2655.00,2430.00,2465.00,2344.00,2371.00,
    2390.00,2282.00,2298.00,2145.00,2160.00,2073.00,
       2039.00,1962.00,1890.00},
    {46914.0,
    47001.0,36557.0,26440.0,20684.0,15671.0,13434.0,
    11757.0,11033.0,9410.00,8946.00,7847.00,7525.00,
    6732.00,6495.00,5896.00,5716.00,5831.00,5106.00,
    4730.00,4831.00,4699.00,4572.00,4286.00,4182.00,
    3941.00,3854.00,3527.00,3576.00,3399.00,3436.00,
    3463.00,3306.00,3327.00,3104.00,3124.00,2997.00,
       2946.00,2834.00,2728.00},
    {63480.0,
    63621.0,49484.0,35789.0,27997.0,21211.0,18183.0,
    15912.0,14932.0,12734.0,12103.0,10617.0,10180.0,
    9106.00,8785.00,7974.00,7729.00,7883.00,6902.00,
    6392.00,6527.00,6348.00,6175.00,5787.00,5645.00,
    5319.00,5201.00,4758.00,4823.00,4583.00,4632.00,
    4667.00,4452.00,4480.00,4179.00,4205.00,4033.00,
       3963.00,3810.00,3668.00},
    {81632.0,
    81818.0,63638.0,46025.0,36003.0,27275.0,23382.0,
    20461.0,19200.0,16373.0,15564.0,13650.0,13087.0,
    11705.0,11293.0,10249.0,9933.00,10130.0,8868.00,
    8213.00,8384.00,8153.00,7930.00,7430.00,7247.00,
    6827.00,6674.00,6105.00,6187.00,5878.00,5940.00,
    5983.00,5707.00,5740.00,5354.00,5386.00,5165.00,
       5075.00,4878.00,4694.00},
    {101150.,
    101394.,78863.0,57036.0,44617.0,33802.0,28975.0,
    25355.0,23792.0,20288.0,19285.0,16912.0,16215.0,
    14502.0,13990.0,12696.0,12304.0,12547.0,10983.0,
    10170.0,10382.0,10094.0,9818.00,9198.00,8971.00,
    8450.00,8260.00,7555.00,7654.00,7271.00,7346.00,
    7399.00,7057.00,7096.00,6618.00,6657.00,6383.00,
       6270.00,6026.00,5798.00},
    {121860.,
    122183.,95032.0,68730.0,53764.0,40732.0,34915.0,
    30552.0,28669.0,24446.0,23237.0,20377.0,19536.0,
    17472.0,16854.0,15295.0,14822.0,15114.0,13229.0,
    12249.0,12503.0,12156.0,11822.0,11075.0,10800.0,
    10172.0,9942.00,9093.00,9212.00,8750.00,8840.00,
    8902.00,8489.00,8536.00,7960.00,8006.00,7675.00,
       7540.00,7245.00,6970.00},
    {143600.,
    144045.,112036.,81028.0,63383.0,48019.0,41161.0,
    36018.0,33797.0,28819.0,27392.0,24021.0,23029.0,
    20595.0,19866.0,18027.0,17469.0,17812.0,15591.0,
    14435.0,14734.0,14325.0,13931.0,13050.0,12725.0,
    11983.0,11712.0,10710.0,10850.0,10305.0,10410.0,
    10483.0,9996.00,10050.0,9371.00,9424.00,9033.00,
       8872.00,8524.00,8200.00}
};

/* Takes energy/nucleon values from Hubert et al., multiplies them by
 * A (mass), to get total kinetic energies appropriate to range.h
 * arrays.  nrg[i][Z-1] is array fmt, where i indexes range values
 * as well (see range.h) */

static double nrg[23][40] = {
    {2.50000,10.0000,17.5000,22.5000,27.5000,30.0000,
    35.0000,40.0000,47.5000,50.0000,57.5000,60.0000,
    67.5000,70.0000,77.5000,80.0000,87.5000,100.000,
    97.5000,100.000,112.500,120.000,127.500,130.000,
    137.500,140.000,147.500,145.000,157.500,160.000,
    172.500,185.000,187.500,200.000,197.500,210.000,
     212.500,220.000,222.500,225.000},
    {5.00000,20.0000,35.0000,45.0000,55.0000,60.0000,
    70.0000,80.0000,95.0000,100.000,115.000,120.000,
    135.000,140.000,155.000,160.000,175.000,200.000,
    195.000,200.000,225.000,240.000,255.000,260.000,
    275.000,280.000,295.000,290.000,315.000,320.000,
    345.000,370.000,375.000,400.000,395.000,420.000,
       425.000,440.000,445.000,450.000},
    {7.00000,28.0000,49.0000,63.0000,77.0000,84.0000,
    98.0000,112.000,133.000,140.000,161.000,168.000,
    189.000,196.000,217.000,224.000,245.000,280.000,
    273.000,280.000,315.000,336.000,357.000,364.000,
    385.000,392.000,413.000,406.000,441.000,448.000,
    483.000,518.000,525.000,560.000,553.000,588.000,
       595.000,616.000,623.000,630.000},
    {10.0000,40.0000,70.0000,90.0000,110.000,120.000,
    140.000,160.000,190.000,200.000,230.000,240.000,
    270.000,280.000,310.000,320.000,350.000,400.000,
    390.000,400.000,450.000,480.000,510.000,520.000,
    550.000,560.000,590.000,580.000,630.000,640.000,
    690.000,740.000,750.000,800.000,790.000,840.000,
       850.000,880.000,890.000,900.000},
    {15.0000,60.0000,105.000,135.000,165.000,180.000,
    210.000,240.000,285.000,300.000,345.000,360.000,
    405.000,420.000,465.000,480.000,525.000,600.000,
    585.000,600.000,675.000,720.000,765.000,780.000,
    825.000,840.000,885.000,870.000,945.000,960.000,
    1035.00,1110.00,1125.00,1200.00,1185.00,1260.00,
       1275.00,1320.00,1335.00,1350.00},
    {20.0000,80.0000,140.000,180.000,220.000,240.000,
    280.000,320.000,380.000,400.000,460.000,480.000,
    540.000,560.000,620.000,640.000,700.000,800.000,
    780.000,800.000,900.000,960.000,1020.00,1040.00,
    1100.00,1120.00,1180.00,1160.00,1260.00,1280.00,
    1380.00,1480.00,1500.00,1600.00,1580.00,1680.00,
       1700.00,1760.00,1780.00,1800.00},
    {25.0000,100.000,175.000,225.000,275.000,300.000,
    350.000,400.000,475.000,500.000,575.000,600.000,
    675.000,700.000,775.000,800.000,875.000,1000.00,
    975.000,1000.00,1125.00,1200.00,1275.00,1300.00,
    1375.00,1400.00,1475.00,1450.00,1575.00,1600.00,
    1725.00,1850.00,1875.00,2000.00,1975.00,2100.00,
       2125.00,2200.00,2225.00,2250.00},
    {30.0000,120.000,210.000,270.000,330.000,360.000,
    420.000,480.000,570.000,600.000,690.000,720.000,
    810.000,840.000,930.000,960.000,1050.00,1200.00,
    1170.00,1200.00,1350.00,1440.00,1530.00,1560.00,
    1650.00,1680.00,1770.00,1740.00,1890.00,1920.00,
    2070.00,2220.00,2250.00,2400.00,2370.00,2520.00,
       2550.00,2640.00,2670.00,2700.00},
    {40.0000,160.000,280.000,360.000,440.000,480.000,
    560.000,640.000,760.000,800.000,920.000,960.000,
    1080.00,1120.00,1240.00,1280.00,1400.00,1600.00,
    1560.00,1600.00,1800.00,1920.00,2040.00,2080.00,
    2200.00,2240.00,2360.00,2320.00,2520.00,2560.00,
    2760.00,2960.00,3000.00,3200.00,3160.00,3360.00,
       3400.00,3520.00,3560.00,3600.00},
    {50.0000,200.000,350.000,450.000,550.000,600.000,
    700.000,800.000,950.000,1000.00,1150.00,1200.00,
    1350.00,1400.00,1550.00,1600.00,1750.00,2000.00,
    1950.00,2000.00,2250.00,2400.00,2550.00,2600.00,
    2750.00,2800.00,2950.00,2900.00,3150.00,3200.00,
    3450.00,3700.00,3750.00,4000.00,3950.00,4200.00,
       4250.00,4400.00,4450.00,4500.00},
    {60.0000,240.000,420.000,540.000,660.000,720.000,
    840.000,960.000,1140.00,1200.00,1380.00,1440.00,
    1620.00,1680.00,1860.00,1920.00,2100.00,2400.00,
    2340.00,2400.00,2700.00,2880.00,3060.00,3120.00,
    3300.00,3360.00,3540.00,3480.00,3780.00,3840.00,
    4140.00,4440.00,4500.00,4800.00,4740.00,5040.00,
       5100.00,5280.00,5340.00,5400.00},
    {70.0000,280.000,490.000,630.000,770.000,840.000,
    980.000,1120.00,1330.00,1400.00,1610.00,1680.00,
    1890.00,1960.00,2170.00,2240.00,2450.00,2800.00,
    2730.00,2800.00,3150.00,3360.00,3570.00,3640.00,
    3850.00,3920.00,4130.00,4060.00,4410.00,4480.00,
    4830.00,5180.00,5250.00,5600.00,5530.00,5880.00,
       5950.00,6160.00,6230.00,6300.00},
    {80.0000,320.000,560.000,720.000,880.000,960.000,
    1120.00,1280.00,1520.00,1600.00,1840.00,1920.00,
    2160.00,2240.00,2480.00,2560.00,2800.00,3200.00,
    3120.00,3200.00,3600.00,3840.00,4080.00,4160.00,
    4400.00,4480.00,4720.00,4640.00,5040.00,5120.00,
    5520.00,5920.00,6000.00,6400.00,6320.00,6720.00,
       6800.00,7040.00,7120.00,7200.00},
    {90.0000,360.000,630.000,810.000,990.000,1080.00,
    1260.00,1440.00,1710.00,1800.00,2070.00,2160.00,
    2430.00,2520.00,2790.00,2880.00,3150.00,3600.00,
    3510.00,3600.00,4050.00,4320.00,4590.00,4680.00,
    4950.00,5040.00,5310.00,5220.00,5670.00,5760.00,
    6210.00,6660.00,6750.00,7200.00,7110.00,7560.00,
       7650.00,7920.00,8010.00,8100.00},
    {100.000,400.000,700.000,900.000,1100.00,1200.00,
    1400.00,1600.00,1900.00,2000.00,2300.00,2400.00,
    2700.00,2800.00,3100.00,3200.00,3500.00,4000.00,
    3900.00,4000.00,4500.00,4800.00,5100.00,5200.00,
    5500.00,5600.00,5900.00,5800.00,6300.00,6400.00,
    6900.00,7400.00,7500.00,8000.00,7900.00,8400.00,
       8500.00,8800.00,8900.00,9000.00},
    {150.000,600.000,1050.00,1350.00,1650.00,1800.00,
    2100.00,2400.00,2850.00,3000.00,3450.00,3600.00,
    4050.00,4200.00,4650.00,4800.00,5250.00,6000.00,
    5850.00,6000.00,6750.00,7200.00,7650.00,7800.00,
    8250.00,8400.00,8850.00,8700.00,9450.00,9600.00,
    10350.0,11100.0,11250.0,12000.0,11850.0,12600.0,
       12750.0,13200.0,13350.0,13500.0},
    {200.000,800.000,1400.00,1800.00,2200.00,2400.00,
    2800.00,3200.00,3800.00,4000.00,4600.00,4800.00,
    5400.00,5600.00,6200.00,6400.00,7000.00,8000.00,
    7800.00,8000.00,9000.00,9600.00,10200.0,10400.0,
    11000.0,11200.0,11800.0,11600.0,12600.0,12800.0,
    13800.0,14800.0,15000.0,16000.0,15800.0,16800.0,
       17000.0,17600.0,17800.0,18000.0},
    {250.000,1000.00,1750.00,2250.00,2750.00,3000.00,
    3500.00,4000.00,4750.00,5000.00,5750.00,6000.00,
    6750.00,7000.00,7750.00,8000.00,8750.00,10000.0,
    9750.00,10000.0,11250.0,12000.0,12750.0,13000.0,
    13750.0,14000.0,14750.0,14500.0,15750.0,16000.0,
    17250.0,18500.0,18750.0,20000.0,19750.0,21000.0,
       21250.0,22000.0,22250.0,22500.0},
    {300.000,1200.00,2100.00,2700.00,3300.00,3600.00,
    4200.00,4800.00,5700.00,6000.00,6900.00,7200.00,
    8100.00,8400.00,9300.00,9600.00,10500.0,12000.0,
    11700.0,12000.0,13500.0,14400.0,15300.0,15600.0,
    16500.0,16800.0,17700.0,17400.0,18900.0,19200.0,
    20700.0,22200.0,22500.0,24000.0,23700.0,25200.0,
       25500.0,26400.0,26700.0,27000.0},
    {350.000,1400.00,2450.00,3150.00,3850.00,4200.00,
    4900.00,5600.00,6650.00,7000.00,8050.00,8400.00,
    9450.00,9800.00,10850.0,11200.0,12250.0,14000.0,
    13650.0,14000.0,15750.0,16800.0,17850.0,18200.0,
    19250.0,19600.0,20650.0,20300.0,22050.0,22400.0,
    24150.0,25900.0,26250.0,28000.0,27650.0,29400.0,
       29750.0,30800.0,31150.0,31500.0},
    {400.000,1600.00,2800.00,3600.00,4400.00,4800.00,
    5600.00,6400.00,7600.00,8000.00,9200.00,9600.00,
    10800.0,11200.0,12400.0,12800.0,14000.0,16000.0,
    15600.0,16000.0,18000.0,19200.0,20400.0,20800.0,
    22000.0,22400.0,23600.0,23200.0,25200.0,25600.0,
    27600.0,29600.0,30000.0,32000.0,31600.0,33600.0,
       34000.0,35200.0,35600.0,36000.0},
    {450.000,1800.00,3150.00,4050.00,4950.00,5400.00,
    6300.00,7200.00,8550.00,9000.00,10350.0,10800.0,
    12150.0,12600.0,13950.0,14400.0,15750.0,18000.0,
    17550.0,18000.0,20250.0,21600.0,22950.0,23400.0,
    24750.0,25200.0,26550.0,26100.0,28350.0,28800.0,
    31050.0,33300.0,33750.0,36000.0,35550.0,37800.0,
       38250.0,39600.0,40050.0,40500.0},
    {500.000,2000.00,3500.00,4500.00,5500.00,6000.00,
    7000.00,8000.00,9500.00,10000.0,11500.0,12000.0,
    13500.0,14000.0,15500.0,16000.0,17500.0,20000.0,
    19500.0,20000.0,22500.0,24000.0,25500.0,26000.0,
    27500.0,28000.0,29500.0,29000.0,31500.0,32000.0,
    34500.0,37000.0,37500.0,40000.0,39500.0,42000.0,
       42500.0,44000.0,44500.0,45000.0}
};

static double 
diffgen(int z, int a, double totalE)
{
    double value;   /* temporary */
    double e_lo,e_hi,r_lo,r_hi;
    int i,lo,hi;

    /* first, find "bracketing" energies. */
    e_lo = 0.0;
    e_hi = 0.0;

    for(i=0; i<23; i++) {
	if (nrg[i][z-1] > totalE) { 
	    /* found upper bracket */
	    if (i == 0) { 
		/* upper bracket below energies, so we extrapolate */
		e_lo = nrg[i][z-1];
		e_hi = nrg[i+1][z-1];
		lo = i;
		hi = i+1;
		break; /* no need to continue search */
	    } else {
		e_lo = nrg[i-1][z-1];
		e_hi = nrg[i][z-1];
		lo = i-1;
		hi = i;
		break; /* no need to continue search */
	    }
	}
    }

    if (i == 23) { 
	/* none found, must mean totalE > all energies for this charge */
	e_lo = nrg[21][z-1];
	e_hi = nrg[22][z-1]; /* lower bracket above energies, so
			    we extrapolate */
	lo = 21;
	hi = 22;
    }

    /* now we do the interp-/extrapolation in log space */
    e_lo = log10(e_lo);
    e_hi = log10(e_hi);
    r_lo = log10(range[lo][z-1]);
    r_hi = log10(range[hi][z-1]);
    totalE = log10(totalE);

    value = ((totalE - e_lo)/(e_hi - e_lo)) * r_hi + 
	    ((e_hi - totalE)/(e_hi - e_lo))*r_lo;
    value = pow(10.0,value);
    if (value < 0.0) {
	return(0.0);
    } else {
	/* this converts range in units of mg/cm^2 
	 * (which data is reported in) to microns Si. */ 
	return(value*4.292);
    }
}

static double 
best_z(double diff3[3], int z3[3])
{
    double c,b;

    /* The idea is to find the parabola which goes through the three
     * data points given.  The formula for this is (from Press et al.,
     * _Numerical Recipes_, among others):
     * 
     * P(x) = \frac{(x-x2)(x-x3)y1}{(x1-x2)(x1-x3)} +
     * \frac{(x-x1)(x-x3)y2}{(x2-x1)(x2-x3)} +
     * \frac{(x-x1)(x-x2)y3}{(x3-x1)(x3-x2)}.
     * 
     * However, what we _really_ want is to verify that this DIFF really
     * is a minimum (coeff. of Z^2 > 0), and then if that is the case,
     * find the Z value of the minimum (P(Z) = a + bZ + cZ^2, min =
     * -b/2c).  We're helped by the fact that the Z values are spaced by
     * one.  With a little algebra, one gets:
     * 
     * c = (DIFF3[0] -2*DIFF3[1] + DIFF3[2]), b = -(DIFF3[0]*(2*z3[1] +
     * 1) -2*DIFF3[1]*(2*z3[1]) + DIFF3[2]*(2*z3[1] - 1)).
     */

    /* check that this DIFF3 really is a minimum */
    
    c = diff3[0] - 2.0*diff3[1] + diff3[2];
    if (c <= 0.0) {
	/* flag that no minimum found */
	return(-7.0);
    }

    b = -(diff3[0] * (2.0 * (double)z3[1] + 1.0) - 
    	2.0 * diff3[1] * (2.0 * (double)z3[1]) + 
	diff3[2] * (2.0 * (double)z3[1] - 1.0));

    return (-b / (2.0 * c));
}

/* pathlength() - Returns pathlength through detector with local deadlayer. 
 * 	
 * Deadlayer no longer returned: change made 12/19/96.  The deadlayer
 *   corrections are handled separately.  "deadflag" tells routine
 *   whether or not to subtract off top deadlayer. 
 * 
 * CHANGE MADE 12/20/96 (to id_particle.c, relevant to pathlength()
 *   usage: Z algorithm will now always request deadlayer
 *   correction to be made.  A algorithm may not, but probably will
 *   also. 
 * 
 * CHANGE MADE 12/30/96: changes deaden() to do linear interpolation of r-e
 *   tables in log-log space rather than using "functional form" which
 *   may not use inverses properly. */

/* originally from thick_dead.h: 12/10/96, MRT (under RCS) 
 * 
 * Will eventually contain all deadlayer maps, thickness maps, etc.,
 * and will be plugged into pathlength.c.  For now will just contain a
 * table with average values (taken from CRIS pre-ship review
 * books).  NOTE! The detector values include the value for detector
 * E7a, stack B, which was swapped in at GSI.  All values in microns
 * of Si.
 * 
 * Detector index runs from 1-39, with none at 10,20,30.  See
 * id_particle.c for details.
 * 
 * Format modified: deadlayer[det][loc] is top deadlayer/bottom
 * deadlayer (for single detectors, one of these always = 0).
 * 
 * Thickness also modified: thick[det][loc] is top/bottom detector
 * thickness.  Define single detectors to have top thickness = 0.
 * 
 * Change made 12/19/96, MRT, to allow deadlayer corrections and to
 * make this file more flexible. */

static double deadlayer[40][2] = {
    {0.0,0.0},{46.5,0.0},{0.0,33.0},{67.0,66.0},{68.0,65.0},{75.0,56.0},
    {52.0,53.0},{41.0,150.0},{82.5,77.0},{0.0,39.0},
    {0.0,0.0},{35.0,0.0},{0.0,67.0},{58.0,77.0},{49.0,61.0},{64.0,51.0},
    {67.0,54.0},{94.0,50.0},{46.0,31.0},{0.0,43.0},
    {0.0,0.0},{61.0,0.0},{0.0,53.0},{64.0,65.0},{49.0,66.0},{70.0,82.0},
    {74.0,66.5},{68.0,0.0},{32.0,47.0},{0.0,55.0},
    {0.0,0.0},{43.0,0.0},{0.0,76.0},{73.0,58.0},{44.0,79.0},{58.0,75.0},
    {69.0,70.0},{51.0,0.0},{71.0,64.0},{0.0,43.0}
};

static double thick[40][2] = {
    {0.0,0.0},{0.0,2940.0},{0.0,2980.0},{3006.0,3002.0},{3020.0,3021.0},
    {3016.0,2991.0},{2947.0,3007.0},{2966.0,2963.0},{3064.5,3010.0},
    {0.0,2944.0},{0.0,0.0},{0.0,2989.0},{0.0,3015.0},{2998.0,3027.0},
    {2945.0,2971.0},{2991.0,3012.0},{3011.0,2961.0},{2938.5,3012.0},
    {2992.0,2968.0},{0.0,2977.0},{0.0,0.0},{0.0,2996.0},{0.0,3025.0},
    {3018.0,2980.0},{2996.0,2996.0},{2965.0,2999.0},{2967.0,3022.5},
    {3006.0,2963.0},{2938.0,3006.0},{0.0,2980.0},{0.0,0.0},{0.0,3011.0},
    {0.0,2963.0},{2966.0,3007.0},{3011.0,3038.0},{2969.0,2978.0},
    {3022.0,2987.0},{3022.0,2918.0},{3003.0,3020.0},{0.0,2939.0}
};

static double 
pathlength(int detector, double x0, double y0, double theta, int deadflag) 
{
    double sectheta;
    double pi = 3.141592;
    double deadthick;

    theta = pi * theta / 180.0;
    sectheta = 1.0 / cos(theta);

    if (!deadflag) {
	return((thick[detector][0]+thick[detector][1])*sectheta);
    } else {
	deadthick = -deadlayer[detector][0]+deadlayer[detector+1][0];

	/* subtract off deadlayer thicknesses in DE det; actually
	   subtracts off DE top deadlayer thickness, adds in E' top
	   deadlayer thickness; see notebook #1, p.35 (bottom) for
	   details.  */
					
	return((thick[detector][0]+thick[detector][1]+deadthick)*sectheta);
    }
}

/* deaden() - Returns energy lost in relevant dead layers, by 
 *	using "log E vs log R" tables.  Basically works in 
 *	reverse of diffgen(). 
 * 
 * CHANGE MADE MRT 12/30/96: See comments at top. */
static double 
deaden(int z, int a, int detector, double theta, double Eprime)
{
    double sectheta;
    double pi = 3.141592;
    double r_eprime,rng;
    double r_lo,r_hi,e_lo,e_hi,value;
    int i,lo,hi;

    theta = pi*theta/180.0;
    sectheta = 1.0/cos(theta);

    r_eprime = diffgen(z,a,Eprime);
    rng = (deadlayer[detector][1] + deadlayer[detector+1][0]) * sectheta +
	r_eprime;
    rng = rng/4.292; /* convert from microns Si to mg/cm^2 */

    /* now we do what we did before (in diffgen(), as rewritten).
       We find the appropriate energies which bracket a given range,
       then interp/extrapolate. 
    */


    /* first, find "bracketing" ranges. */
    r_lo = 0.0;
    r_hi = 0.0;

    for(i=0; i<23; i++) {
	if (range[i][z-1] > rng) {
	    /* found upper bracket */
	    if (i == 0) {
		/* upper bracket below ranges, so we extrapolate */
		r_lo = range[i][z-1];
		r_hi = range[i+1][z-1];
		lo = i;
		hi = i+1;
		break; /* no need to continue search */
	    } else {
		r_lo = range[i-1][z-1];
		r_hi = range[i][z-1];
		lo = i-1;
		hi = i;
		break; /* no need to continue search */
	    }
	}
    }

    if (i == 23) {
	/* none found, must mean rng > all ranges for this charge */
	r_lo = range[21][z-1];
	r_hi = range[22][z-1]; /* lower bracket above energies, so
				  we extrapolate */
	lo = 21;
	hi = 22;
    }

    /* now we do the interp-/extrapolation in log space */
    r_lo = log10(r_lo);
    r_hi = log10(r_hi);
    e_lo = log10(nrg[lo][z-1]);
    e_hi = log10(nrg[hi][z-1]);
    rng = log10(rng);

    value = ((rng - r_lo)/(r_hi - r_lo)) * e_hi + 
    	    ((r_hi - rng)/(r_hi - r_lo)) * e_lo;
    value = pow(10.0,value) - Eprime;
    if (value < 0.0) {
	return(0.0);
    } else {
	return(value);
    }
}
