/* crisevt.h - declarations for cris normal event structures
 * and associated macros. This file is part of the common environment
 * between the softlib and the level1 library. It is #included in
 * softcris.h */

#ifndef CRISEVT_H
#define CRISEVT_H

static char crisevt_h_cvsid[] = "$Id: crisevt.h,v 1.2 1998/07/20 21:18:50 marty Exp $";

/* CE_CRIS_MAXBLOBS - Never more blobs per frame (telemetered in the cris
 * normal data stream) than this in a CRIS event */
#define CE_CRIS_MAXBLOBS	31

#define CE_CRIS_NCTRPH	10	/* no. of stack center pulse heights */
#define CE_CRIS_NGUARD	6	/* no. of stack guards */

/* cris "normal" event stuff */
struct ce_crisHodoData {
    short x;
    short y;
    unsigned long I;
};

struct ce_crisStimData {
    u_char ctr_hi_gain;     /* REFA (center) high gain flag */
    u_char ctr_pulsers;     /* Flags for active REFA (center) pulsers */
    u_char ctr_dac_level;   /* REFA (center) DAC level */
    u_char guard_hi_gain;   /* REFB (guard) high gain flag */
    u_char guard_pulsers;   /* Flags for active REFB (guard) pulsers */
    u_char guard_dac_level; /* REFB (guard) DAC level */
    u_char soft_led_pulse;  /* SOFT LED pulse length */
};

struct ce_crisEvt {
    unsigned long evtno;	/* event number */
    unsigned long secs1996;
    unsigned long usec;
    unsigned long rfu;
    long npix;		/* no. of soft pixels */
    u_char *p;		/* ptr to cris normal data for this event */
    u_char evt_offset;
    char cam;		/* which camera (guess) */
    u_char nbytes;	/* byte 0 - no. of bytes in this event */
    u_char stim;	/* stimulated event flag */
    u_char prim_tel_pair;	/* primary telescope pair (CRIS_AB or _CD) */
    u_char buffer_id;	/* event buffer id */
    u_char guard_hi_or;	/* guard high level "OR" */
    u_char hazard;	/* hazard flag */
    u_char G1;		/* # low level guards hit in prim. tel. pair */
    u_char R1;		/* range in primary tel. pair */
    u_char got_2nd_tel;	/* is there 2nd telescope data in this evt? */
    u_char got_guard;	/* transmit guard pulse heights in this evt? */
    u_char centroid_err;/* centroid calculation error */
    u_char nblobs;	/* number of soft centroids transmitted */
    u_char age;         /* age - units 256 sec. */
    u_char sec_tel_act; /* activity in the second telescope */
    u_char G2;		/* # low level guards hit in 2nd tel. pair */
    u_char R2;		/* range in 2nd telescope pair */
    struct ce_crisHodoData hodo[CE_CRIS_MAXBLOBS];	/* SOFT hodo data */
    u_short prim_tag;	/* primary telescope tag bits */
    short prim_ctr_ph[CE_CRIS_NCTRPH];		/* primary centr pulse hts */
    short prim_guard_ph[CE_CRIS_NGUARD];	/* primary guard pulse hts */
    u_short sec_tag;		/* secondary telescope tag bits */
    short sec_ctr_ph[CE_CRIS_NCTRPH];		/* 2ndary ctr pulse hts */
    short sec_guard_ph[CE_CRIS_NGUARD];		/* 2ndary guard pulse hts */
    struct ce_crisStimData  stimdata;		/* stimulated event parameters */
};
#endif /* CRISEVT_H */

/* These next two don't seem to be in the ACEware level1 headers */
#define CE_CRIS_AB	0	/* primary telescope pair is AB */
#define CE_CRIS_CD 	1	/* primary telescope pair is CD */

