/* reorder.c - routines for reordering the SOFT (not just msu) data stream
 *	from big- to little-endian and vice-versa
 *
 *  Marty Olevitch, Jul 1994
 */

static char cvsid[] = "$Id: evt_reorder.c,v 1.2 1997/09/23 21:15:07 marty Exp $";

#include "msu.h"
#include "softcris.h"

/* sc_msu_reorder_std() - swap the bytes as needed in the std stuff in the
 * event pointed to by the argument. If the std stuff changes, this routine
 * will probably need alteration as well. The reordered data is found in
 * the data pointed to by swapbuf and the original data pointed to by ptr
 * is left alone. Returns 0 if all goes well, -1 for problems. */
int
sc_msu_reorder_std(ptr, swapbuf)
    u_char *ptr;	/* points to start of event */
    u_char *swapbuf;	/* points to reordered data */
{   
    u_char *dp, *p;  /* work pointers */
    u_short eid;
    u_short *sp;
    u_char *srcp;   /* location of stuff of interest relative to start of
		     * evt pointed to by ptr */
    
    p = srcp = GET_HDR_HDR(ptr);
    dp = swapbuf;
    swab(p, dp, 4*sizeof(u_short));	/* HDR_HDR,EVT_HDR,type,exp_id */
    sp = (u_short *)GET_EVT_HDR(ptr);
    if (*sp == b_EVT_HDR) {
	/* non-native endian - use swapped value */
	eid = *(u_short *)GET_exp_id(dp);	/* get experiment id */
    } else if (*sp == EVT_HDR) {
	/* native - use unswapped value */
	eid = *(u_short *)GET_exp_id(p);
    } else {
	/* bad data */
	return -1;
    }
    p += 4*sizeof(u_short);
    dp += 4*sizeof(u_short);
    sc_little_endian_long(p, (u_long *)dp);	/* size */
    p += sizeof(u_long);
    dp += sizeof(u_long);
    sc_little_endian_long(p, (u_long *)dp);	/* evtno */
    p += sizeof(u_long);
    dp += sizeof(u_long);
    if (eid < 6) {
	swab(p, dp, 2*sizeof(u_short));		/* HDR_HDR, CLK_HDR */
	p += 2*sizeof(u_short);
	dp += 2*sizeof(u_short);
    } else {
	sc_little_endian_long(p, (u_long *)dp);	/* correct abs evt no */
	p += sizeof(u_long);
	dp += sizeof(u_long);
    }
    sc_little_endian_long(p, (u_long *)dp);	/* clock */
    p += sizeof(u_long);
    dp += sizeof(u_long);
    if (eid < 5) {
	swab(p, dp, 2*sizeof(u_short));		/* HDR_HDR, ABS_HDR */
	p += 2*sizeof(u_short);
	dp += 2*sizeof(u_short);
    } else {
	sc_little_endian_long(p, (u_long *)dp);	/* vid size 2 */
	p += sizeof(u_long);
	dp += sizeof(u_long);
    }
    sc_little_endian_long(p, (u_long *)dp);	/* abs. evtno */
    p += sizeof(u_long);
    dp += sizeof(u_long);
    sc_little_endian_long(p, (u_long *)dp);	/* vidsize */
    return 0;
}

/* sc_msu_reorder_reply() - swap the bytes as needed in the reply stuff. See
 * the sc_msu_reorder_std() routine above. */
int
sc_msu_reorder_reply(ptr, swapbuf)
    u_char *ptr;	/* points to start of event */
    u_char *swapbuf;	/* points to reordered data */
{
    int amt;
    u_short eid;    /* experiment id */
    int replysiz;

    u_char *dp, *p; /* work pointers */
    u_short *sp;
    u_char *srcp;   /* location of stuff of interest relative to start of
		     * evt pointed to by ptr */
    

    sp = (u_short *)GET_EVT_HDR(ptr);
    p = GET_exp_id(ptr);    /* get experiment id */
    if (*sp == b_EVT_HDR) {
	/* non-native endian - use swapped value */
	swab(p, (char *)&eid, sizeof(u_short));
    } else if (*sp == EVT_HDR) {
	/* native - use unswapped value */
	eid = *(u_short *)p;
    } else {
	/* bad data */
	return -1;
    }

    if (eid < 5) {
	replysiz = REPLYSIZ;
    } else {
	replysiz = REPLYSIZ_5;
    }

    p = srcp = GET_REPLY_HDR(ptr);
    dp = swapbuf;
    amt = 3 * sizeof(u_short);
    swab(p, dp, amt);		/* HDR_HDR, REPLY_HDR, code */
    p += amt;
    dp += amt;
    /* don't swab the reply info as it is in ascii bytes already */
    memcpy(dp, p, replysiz);
    p += replysiz;
    dp += replysiz;
    /* don't forget the EVT_END */
    swab(p, dp, 2*sizeof(u_short));
    return 0;
}

/* sc_msu_reorder_real() - swap the bytes as needed in the pha, disc, and
 * video portions of the pointed-to event. See the sc_msu_reorder_std() routine
 * above.  As a help to the caller, this routine places the size of the evt
 * into the long pointed to by sizep. Also the vidsize into vidsizep and the
 * experiment id into exp_id. Returns 0 (okay), -1 if size too big, -2 if
 * vidsizeb too big. */
int
sc_msu_reorder_real(ptr, swapbuf, sizep, vidsizep, vidsizep2, exp_id)
    u_char *ptr;
    u_char *swapbuf;
    long *sizep;
    long *vidsizep;
    long *vidsizep2;
    u_short *exp_id;
{
    u_short blob_hdr;
    u_char *dp, *p; /*work pointers */
    u_char *srcp;   /* location of stuff of interest relative to start of
		     * evt pointed to by ptr */
    int discsiz;    /* no. of bytes in discrim stuff that needn't be swapped */
    int discswap;   /* no. of bytes in discrim stuff that must be swapped */
    u_short eid;    /* experiment id */
    int i;
    int j;
    int maxdata;
    int nalignbytes;	/* no. of alignment bytes at end of evt */
    int native;
    u_short nblobs;
    int ntimes;
    int phasiz;
    u_short *sp;
    long sz;
    long vidsz;	    /* vid sys 1 size in bytes (excluding HDR_HDR,VID_HDR) */
    long vidsz2;    /* vid sys 2 size in bytes (excluding HDR_HDR,VID_HDR) */

    p = GET_exp_id(ptr);    /* get experiment id */
    sp = (u_short *)GET_EVT_HDR(ptr);
    if (*sp == b_EVT_HDR) {
	/* non-native endian - use swapped value */
	native = 0;
	swab(p, (char *)&eid, sizeof(u_short));
    } else if (*sp == EVT_HDR) {
	/* native - use unswapped value */
	native = 1;
	eid = *(u_short *)p;
    } else {
	/* bad data */
	return -1;
    }

    *exp_id = eid;

    if (eid < 5) {
	maxdata = MAXDATA;
	phasiz = PHASIZ;
	discsiz = DISCSIZ;
	discswap = 0;
    } else {
	maxdata = MAXDATA2;
	phasiz = PHASIZ_5;
	discswap = 12;
	discsiz = DISCSIZ_5 - discswap;
    }

    p = GET_size(ptr);	    /* get total size of evt */
    if (native) {
	sz = *(long *)p;
    } else {
	sc_little_endian_long(p, (u_long *)&sz);
    }
    *sizep = sz;
    if (sz >= maxdata) {
	return -2;
    }

    p = GET_vidsize(ptr);   /* get video size (bytes) */
    if (native) {
	vidsz = *(u_long *)p;
    } else {
	sc_little_endian_long(p, (u_long *)&vidsz);
    }
    *vidsizep = vidsz;
    if (vidsz >= maxdata) {
	return -3;
    }
    
    p = GET_vidsize2(ptr);   /* get video size (bytes) */
    if (native) {
	vidsz2 = *(u_long *)p;
    } else {
	sc_little_endian_long(p, (u_long *)&vidsz2);
    }
    *vidsizep2 = vidsz2;
    if (vidsz2 >= maxdata) {
	return -4;
    }
    
    p = srcp = GET_PHA_HDR(ptr);
    dp = swapbuf;
    swab(p, dp, phasiz);	/* phas + HDRs */
    dp += phasiz;
    p = GET_DISC_HDR(ptr, phasiz);
    swab(p, dp, 2 * sizeof(u_short));	/* HDR_HDR, DISC_HDR */
    p += 2 * sizeof(u_short);
    dp += 2 * sizeof(u_short);

    /* insert the portion of disc that should remain unswapped; it is
     * already byte ordered */
    memcpy(dp, p, discsiz - (2 * sizeof(u_short)));
    dp += discsiz - (2 * sizeof(u_short));
    p  += discsiz - (2 * sizeof(u_short));

    /* ...and the part that should be swapped */
    swab(p, dp, discswap);
    dp += discswap;

    /* do the HDR_HDR, VID_HDR, and video data */
    p = GET_VID_HDR(ptr, phasiz, discsiz+discswap);
    swab(p, dp, 4 + vidsz);
    p += 4 + vidsz;
    dp += 4 + vidsz;
    if (eid >= 5) {
	/* should be pointing at the HDR_HDR, VID_HDR2 */
	swab(p, dp, 4 + vidsz2);
	p += 4 + vidsz2;
	dp += 4 + vidsz2;
    }
    /* should be pointing at the HDR_HDR, BLOB_HDR, but they may not be
     * there if we were not in doblobs mode. Check the 2nd word following to
     make sure it is a blob header. */
    if (native) {
	blob_hdr = BLOB_HDR;
    } else {
	blob_hdr = b_BLOB_HDR;
    }
    if (*(u_short *)(p+2) == blob_hdr) {
    	if (eid >= 5) {
	    ntimes = 2;
	} else {
	    ntimes = 1;
	}
	for (j=0; j<ntimes; j++) {
	    swab(p, dp, 3 * sizeof(u_short));	/* HDR_HDR, BLOB_HDR, nblobs */
	    p += 2 * sizeof(u_short);
	    dp += 2 * sizeof(u_short);
	    if (native) {
		nblobs = *(u_short *)p;
	    } else {
		nblobs = *(u_short *)dp;
	    }
	    p += sizeof(u_short);
	    dp += sizeof(u_short);
	    for (i=0; i<nblobs; i++) {
		swab(p, dp, 2 * sizeof(u_short));   /* x, y */
		p += 2 * sizeof(u_short);
		dp += 2 * sizeof(u_short);
		if (eid >= 3) {
		    /* pre msu fall 94 or later - get intensities */
		    sc_little_endian_long(p, (u_long *)dp);	/* I */
		    p += sizeof(u_long);
		    dp += sizeof(u_long);
		}
	    }
	}
    }
    
    swab(p, dp, 2 * sizeof(u_short));	/* HDR_HDR, EVT_END */

    /* There may be alignment bytes at the end. Put them into the swapbuf. */
    p += 2 * sizeof(u_short);
    dp += 2 * sizeof(u_short);
    nalignbytes = sz - STDSIZ - (dp - swapbuf);
    swab(p, dp, nalignbytes);
    return 0;
}

/* sc_msu_reorder_spill() - swap the bytes as needed in the spill stuff. See
 * the other reorder routines above */
void
sc_msu_reorder_spill(ptr, swapbuf)
    u_char *ptr;
    u_char *swapbuf;
{
    u_char *dp, *p; /*work pointers */
    u_char *srcp;   /* location of stuff of interest relative to start of
		     * evt pointed to by ptr */

    p = srcp = GET_SPILL_HDR(ptr);
    dp = swapbuf;
    swab(p, dp, 2 * sizeof(u_short));	    /* HDR_HDR, SPILL_HDR */
    p += 2 * sizeof(u_short);
    dp += 2 * sizeof(u_short);
    sc_little_endian_long(p, (u_long *)dp);    /* spill no. */
}

/* sc_msu_reorder_pha_only() - swap the bytes as needed in the pha portion of
 * the pointed-to event. See the reorder_std() routine above.  As a help to the
 * caller, this routine places the size of the evt into the long pointed to by
 * sizep. Returns -2 for a too-big evt size, else 0. */
int
sc_msu_reorder_pha_only(ptr, swapbuf, sizep)
    u_char *ptr;
    u_char *swapbuf;
    long *sizep;
{
    u_char *dp, *p; /*work pointers */
    u_short eid;
    int maxdata;
    int n;
    int native;
    int phasiz;
    u_short *sp;
    u_char *srcp;   /* location of stuff of interest relative to start of
		     * evt pointed to by ptr */
    u_long sz;	    /* size (bytes) */

    sp = (u_short *)GET_EVT_HDR(ptr);
    p = GET_exp_id(ptr);	/* get experiment id */
    if (*sp == b_EVT_HDR) {
	/* non-native endian - use swapped value */
	swab(p, (char *)&eid, sizeof(u_short));
	native = 0;
    } else if (*sp == EVT_HDR) {
	/* native - use unswapped value */
	eid = *(u_short *)p;
	native = 1;
    } else {
	/* bad data */
	return -1;
    }

    if (eid < 5) {
	phasiz = PHASIZ;
	maxdata = MAXDATA;
    } else {
	phasiz = PHASIZ_5;
	maxdata = MAXDATA2;
    }

    p = GET_size(ptr);	    /* get total size of evt */
    if (native) {
	sz = *(long *)p;
    } else {
	sc_little_endian_long(p, &sz);	/* already swapped above! */
    }
    *sizep = sz;
    if (sz >= maxdata) {
	return -2;
    }

    p = srcp = GET_PHA_HDR(ptr);
    dp = swapbuf;
    swab(p, dp, phasiz);	/* phas + HDRs */
    dp += phasiz;
    /* do the enders */
    p += phasiz;
    swab(p, dp, 2*sizeof(u_short));
    return 0;
}

/* sc_msu_reorder_dl() - swap bytes as needed for an (obsolete) 
 * datalink buffer header */
void
sc_msu_reorder_dl(ptr, swapbuf)
    u_char *ptr;
    u_char *swapbuf;
{
    u_char *dp, *p; /* work pointers */
    u_char *srcp;   /* location of stuff of interest relative to start of
		     * evt pointed to by ptr */

    p = srcp = GET_DL_HDR(ptr);
    dp = swapbuf;
    swab(p, dp, sizeof(u_short));
}

/* sc_little_endian_long() - convert a long from a little-endian
 * machine.  We have to swap the short words and the bytes within the words.
 * Hidden assumption that sizeof(u_long) = 2 * sizeof(u_short) here. */
void
sc_little_endian_long(srcp, destp)
    u_char *srcp;
    u_long *destp;
{
    u_char *p;
    p = (u_char *)destp;
    swab(srcp + sizeof(u_short), p, sizeof(u_short));
    swab(srcp, p + sizeof(u_short), sizeof(u_short));
}

/* sc_reorder_soft - swab the bytes in a SOFT image. ptr should point to
 * the START_HDR at the beginning of the image. The swapped data will be
 * stored in the memory pointed to by swapbuf. Processing ends when we 
 * encounter an END_HDR or max words have been processed. To disable
 * stopping after max words, set max to -1 or some other negative value. */
int
sc_reorder_soft(ptr, swapbuf, max)
    u_short *ptr;
    u_short *swapbuf;
    int max;
{
    u_short *dp, *p;  /* work pointers */
    int i;
    u_short target;

    p = ptr;
    dp = swapbuf;

    if (*p == SC_START_HDR) {
	target = SC_END_HDR;
    } else {
	target = SC_ALT_END_HDR;
    }

    for (i=0; i<max; i++) {
	swab((u_char *)p, (u_char *)dp, sizeof(u_short));
	if (*p == target) {
	    break;
	}
	p++;
	dp++;
    }

    if (i >= max && max >= 0) {
	return -1;
    }
    return 0;
}

/* sc_reorder_cris_subset - swab the bytes in a sc_crisSubset. 
 * The swapped data will be stored in the memory pointed to by swapbuf. */
void
sc_reorder_cris_subset(ptr, swapbuf)
    sc_crisSubset *ptr;
    sc_crisSubset *swapbuf;
{
    int amt;
    u_char *p, *swapp;
    int remainder;

    remainder = sizeof(sc_crisSubset);

    p = (u_char *)ptr;
    swapp = (u_char *)swapbuf;

    /* PacketSecondaryHeader */
    amt = sizeof(u_long);
    sc_little_endian_long(p, (u_long *)swapp);
    p += amt;
    swapp += amt;
    remainder -= amt;

    /* PacketSecondaryHeader */
    amt = sizeof(u_long);
    sc_little_endian_long(p, (u_long *)swapp);
    p += amt;
    swapp += amt;
    remainder -= amt;

    /* year, ErrorFlags, PhaseAngle */
    amt = 3 * sizeof(u_short);
    swab(p, swapp, amt);
    p += amt;
    swapp += amt;
    remainder -= amt;

    /* remainder should stay unswabbed */
    memcpy(swapp, p, remainder);
}
