/* fibermap.c - routines for reading SOFT pixel-to-fiber maps
 *
 *	Marty Olevitch, March 1996
 */

static char cvsid[] = "$Id: fibermap.c,v 1.5 1999/04/28 18:30:26 marty Exp $";

#include <stdio.h>
#include <errno.h>

#include "misc.h"
#include "softcris.h"

#define FILENAMELEN	500
#define CAM1_FILE	"fibermap.cam1"
#define CAM2_FILE	"fibermap.cam2"

static struct sc_fmapElem *fmapp1[SC_FMAPX];
static struct sc_fmapElem *fmapp2[SC_FMAPX];

static int Init = 0;

static int
centr_to_fiber_init(int cam)
    /* cam must be either SC_CAM1 or SC_CAM2 */
{
    int i;
    int j;
    static int Did_allocate_1 = 0;
    static int Did_allocate_2 = 0;

    if (cam != SC_CAM1 && cam != SC_CAM2) {
	fprintf(stderr, "centr_to_fiber_init: cam not SC_CAM1 or SC_CAM2\n"); 
	return -1;
    }

    if (cam == SC_CAM1) {
	if (!Did_allocate_1) {
	    for (i=0; i<SC_FMAPX; i++) {
		fmapp1[i] = (struct sc_fmapElem *)
		    malloc(SC_FMAPY*sizeof(struct sc_fmapElem));
		if (fmapp1[i] == (struct sc_fmapElem *)NULL) {
		    fprintf(stderr,
			"centr_to_fiber_init: bad malloc cam 1 tab %d\n", i);
		    for (j=0; j<i; j++) {
			/* release all memory we managed to allocate */
			free(fmapp1[j]);
		    }
		    return -1;
		}
	    }
	    Did_allocate_1 = 1;
	}

	i = sc_read_fmap(1, &fmapp1[0]);
	if (i) {
	    fprintf(stderr, "centr_to_fiber_init: bad sc_read_fmap cam 1\n");
	    return -1;
	}
    }

    if (cam == SC_CAM2) {
	if (!Did_allocate_2) {
	    for (i=0; i<SC_FMAPX; i++) {
		fmapp2[i] = (struct sc_fmapElem *)malloc(
		    SC_FMAPY*sizeof(struct sc_fmapElem));
		if (fmapp2[i] == (struct sc_fmapElem *)NULL) {
		    fprintf(stderr,
			"centr_to_fiber_init: bad malloc cam 2 tab %d\n", i);
		    for (j=0; j<i; j++) {
			/* release all memory we managed to allocate */
			free(fmapp2[j]);
		    }
		    return -1;
		}
	    }
	    Did_allocate_2 = 1;
	}

	i = sc_read_fmap(2, &fmapp2[0]);
	if (i) {
	    fprintf(stderr,"centr_to_fiber_init: bad sc_read_fmap cam 2\n");
	    return -1;
	}
    }

    Init |= (1 << cam);
    return 0;
}

struct sc_fmapElem *
sc_centr_to_fiber(int cam, int x, int y)
{
    struct sc_fmapElem *fmapp = (struct sc_fmapElem *)NULL;

    if (!(Init & (1 << cam))) {
	if (centr_to_fiber_init(cam)) {
	    return (NULL);
	}
    }

    if (x >= SC_FMAPX) {
#ifdef SOFT_VERBOSE
	fprintf(stderr, "sc_centr_to_fiber: need 0 <= x < %d, not %d\n", 
	    SC_FMAPX, x);
#endif
	return (NULL);
    }
    if (y >= SC_FMAPY) {
#ifdef SOFT_VERBOSE
	fprintf(stderr, "sc_centr_to_fiber: need 0 <= y < %d, not %d\n", 
	    SC_FMAPY, y);
#endif
	return (NULL);
    }

    if (cam == SC_CAM1) {
	fmapp = &fmapp1[x][y];
    } else if (cam == SC_CAM2) {
	fmapp = &fmapp2[x][y];
    } 

    return(fmapp);
}

int
sc_read_fmap(int cam, struct sc_fmapElem *fmapp[SC_FMAPX]) {
    FILE *fp;
    char filename[FILENAMELEN];
    int i;
    int nread;

    if (cam == 1) {
	sprintf(filename, "%s/%s", MAPDIR, CAM1_FILE);
    } else if (cam == 2) {
	sprintf(filename, "%s/%s", MAPDIR, CAM2_FILE);
    } else {
	fprintf(stderr, "sc_read_fmap: cam must be 1 or 2, not %d\n", cam);
	return -1;
    }

    fp = fopen(filename, "rb");
    if (fp == (FILE *)NULL) {
	fprintf(stderr, "sc_read_fmap: can't open '%s' (%s)\n", filename,
		strerror(errno));
	return -1;
    }

    for (i=0; i<SC_FMAPX; i++) {
	nread = fread((char *)fmapp[i], sizeof(struct sc_fmapElem), SC_FMAPY, fp);
	if (nread != SC_FMAPY) {
	    fprintf(stderr, "sc_read_fmap: bad read from file '%s' (%s)\n",
		filename, strerror(errno));
	    fclose(fp);
	    return -1;
	}
    }

    fclose(fp);
    return 0;
}

int
sc_read_fmap_by_name(char *filename, struct sc_fmapElem *fmapp[SC_FMAPX]) {
    FILE *fp;
    int i;
    int nread;

    fp = fopen(filename, "rb");
    if (fp == (FILE *)NULL) {
	fprintf(stderr, "sc_read_fmap_by_name: can't open '%s' (%s)\n", 
		filename, strerror(errno));
	return -1;
    }

    for (i=0; i<SC_FMAPX; i++) {
	nread = fread((char *)fmapp[i], sizeof(struct sc_fmapElem), SC_FMAPY, fp);
	if (nread != SC_FMAPY) {
	    fprintf(stderr, "sc_read_fmap_by_name: bad read from file '%s' (%s)\n",
		filename, strerror(errno));
	    fclose(fp);
	    return -1;
	}
    }

    fclose(fp);
    return 0;
}
