/* ace.h - some ace specific stuff */

#ifndef _MSU_H
#define _MSU_H

static char msu_h_cvsid[] = "$Id: msu.h,v 1.1.1.1 1997/03/14 15:53:04 marty Exp $";

/* this is to get the u_char, etc typedefs. */
#include <sys/types.h>

#define MAXDATA	    32700	/* max size of data buffers - should match
				 * DATALINKSIZ in the ace-pc software */
#define MAXDATA2    65400	/* max size of data buffers for exp id 5 -
				 * should match DATABUFSIZ in the ace-pc
				 * software */

#ifndef CHOP
/* CHOP - replace last character of a string */
#define CHOP(s, c)	((s)[strlen(s)-1] = (c))
#endif

/* big-endian headers */
#define b_HDR_HDR	0xffff
#define b_EVT_HDR	0xf9ff
#define b_SPILL_HDR	0xeeff		/* beam spill number follows	*/
#define b_BLOB_HDR	0xbcff

/* regular headers (after byte reordering) */
#define HDR_HDR		0xffff
#define EVT_HDR		0xfff9
#define SPILL_HDR	0xffee		/* beam spill number follows	*/
#define CLK_HDR		0xfffd		/* clock word (PC time-of-day)	*/
#define PHA_HDR		0xfff8		/* start of pha's		*/
#define HSK_HDR		0xfffb		/* start of housekeeping data	*/
#define VID_HDR		0xffec		/* start of video data		*/
#define VID_HDR2	0xffed		/* start of video data vid sys 2*/
#define CAM1_HDR	0xffd0		/* camera 1 video data		*/
#define CAM2_HDR	0xffd1		/* camera 2 video data		*/
#define CAM3_HDR	0xffd2		/* camera 3 video data		*/
#define CAM4_HDR	0xffd3		/* camera 4 video data		*/
#define CAM5_HDR	0xffd4		/* camera 5 video data		*/
#define CAM6_HDR	0xffd5		/* camera 6 video data		*/
#define CAM7_HDR	0xffd6		/* camera 7 video data		*/
#define CAM8_HDR	0xffd7		/* camera 8 video data		*/
#define CAM9_HDR	0xffd8		/* camera 9 video data		*/
#define CAM10_HDR	0xffd9		/* camera 10 video data		*/
#define CAM11_HDR	0xffda		/* camera 11 video data		*/
#define CAM12_HDR	0xffdb		/* camera 12 video data		*/
#define CNT_HDR		0xffea		/* elec cal count/level hdr	*/
#define LEV_HDR(x)	(0xff00 + (x))	/* elec cal level hdr ( <= c0 )	*/
#define GAIN_HDR	0xffe3		/* cam cal gains		*/
#define REPLY_HDR	0xffe1		/* reply data			*/
#define EVT_END		0xfff2		/* end of the event		*/
#define ABS_HDR		0xffe8		/* absolute event number	*/
#define DISC_HDR	0xffe6		/* introduces camera discrims	*/
#define MCA_HDR		0xffb0		/* begin of mca calibration	*/
#define TAPERD_HDR	0xffb3		/* start of flight exabyte tape data */
#define BLOB_HDR	0xffbc		/* blob stuff header		*/
#define SEQ_HDR(x)	(0xffa0 + (x))	/* bigvid sequencing header */

/* evt type codes */
#define NTYPE		11	/* number of different event types	*/
#define T_REAL		0	/* a "real" event			*/
#define T_CAL_E		1	/* electronic calibration		*/
#define T_CAL_V		2	/* video calibration			*/
#define T_CAL_C		3	/* coincidence calibration		*/
#define T_REPLY		4	/* reply				*/
#define T_HSK		5	/* housekeeping event			*/
#define T_SCALER	6	/* scalers event			*/
#define T_CAL_M		7	/* Multi-chan analyzer calibration	*/
#define T_CAL_B		8	/* Big video frame calibration		*/
#define T_PHA_ONLY	9	/* real evt with phas only, no video	*/
#define T_TAPERD	10	/* data that was put on the flight exabyte */

/* changing these values may require that you change other stuff elsewhere. */
#define N12PHA	    (7+1)   /* # of 12-bit phas + parity word	*/
#define N12PHA_5    (10+1)  /* # of 12-bit phas + parity word (exp id 5) */
#define N16PHA	    0	    /* # of 16-bit phas			*/
#define NCAM	    2	    /* # of actual cameras		*/
#define NLEV	    20	    /* # of calibration levels (elec cal)*/
#define NTHRESH	    3	    /* number of threshold discrims.	*/
#define NTRIG	    2	    /* number of trigger discrims	*/
#define NGAIN	    32	    /* number of gain values - there are really
			     * only 28 values. */

/* The following data stream sizes includes the associated headers which
 * begin the part. I.e., STDSIZ includes HDR_HDR & EVT_HDR; PHASIZ includes
 * HDR_HDR & PHA_HDR; and HSKSIZ includes HDR_HDR & HSK_HDR.  Likewise
 * REPLYSIZ includes HDR_HDR & REPLY_HDR; and GAINSIZ includes HDR_HDR &
 * GAIN_HDR. The sizes DO NOT include the HDR_HDR & EVT_END at the end of the
 * pha part and hsk part! */
#define STDSIZ		36	/* size (bytes) of standard evt info	*/
#define STDSIZh		24	/* std stuff size not counting HDRs	*/
#define PHASIZ		(((2 + N12PHA) + (2*N16PHA)) * 2)
				/* size (bytes) of pha data		*/
#define PHASIZ_5	(((2 + N12PHA_5) + (2*N16PHA)) * 2)
				/* size (bytes) of pha data		*/
#define REPLYSIZ	84	/* size (bytes) of reply body		*/
#define REPLYSIZ_5	124	/* size (bytes) of reply body		*/
#define ECALSIZ(x)	(((((x)*PHASIZ)+4)*NLEV)+6)	/* bytes/eleccal*/
#define DISCSIZ		18	/* size (bytes) of discrim, gain, enables,
				 * and threshold values */
#define DISCSIZ_5	28	/* size (bytes) of discrim, gain, enables,
				 * and threshold values */
#define SEQSIZ		8	/* size (bytes) of bigvid sequence part	*/
#define ENDSIZ		4	/* size (bytes) of event ender		*/
#define SPILLSIZ	8	/* size (bytes) of spill size sequence -
				 * note that this is NOT the size of the
				 * spill, just the headers and spill no. */

#define REPLY_FILL	0xda	/* Fill byte for replies that don't use all
				 * 78 characters available to them. We want to
				 * avoid having a "long" series of only 0 bits
				 * or only 1 bits. */


/* Macros to return pointers to certain items in the data stream. Assumes
 * that the source pointer p is pointing at the beginning of an event. If any
 * of the *SIZ constants above, change, then these may have to change also */
#define SKIPUNIT	(sizeof(u_short))
#define GET_HDR_HDR(p)	(p)
#define GET_EVT_HDR(p)	(p + SKIPUNIT)
#define GET_type(p)	(p + (2 * SKIPUNIT))	/* evt type */
#define GET_exp_id(p)	(p + (3 * SKIPUNIT))	/* experiment ID */
#define GET_size(p)	(p + (4 * SKIPUNIT))
#define GET_evtno(p)	(p + (6 * SKIPUNIT))
#define GET_corr_abs(p)	(p + (8 * SKIPUNIT))
#define GET_clock(p)	(p + (10 * SKIPUNIT))
#define GET_vidsize2(p)	(p + (12 * SKIPUNIT))
#define GET_absevtno(p)	(p + (14 * SKIPUNIT))
#define GET_vidsize(p)	(p + (16 * SKIPUNIT))
#define GET_REPLY_HDR(p) (p + ((STDSIZ/2) * SKIPUNIT))	    /* -> REPLY_HDR */
#define GET_reply(p)	(p + (((STDSIZ/2)+3) * SKIPUNIT))   /* -> reply mesg */
#define GET_PHA_HDR(p)	(p + ((STDSIZ/2) * SKIPUNIT))	    /* -> PHA_HDR */
#define GET_phas(p)	(p + (((STDSIZ/2)+2) * SKIPUNIT))   /* -> pha data */
#define GET_DISC_HDR(p,p2)	(p + (((STDSIZ/2)+((p2)/2)) * SKIPUNIT))
							    /* -> DISC_HDR */
#define GET_disc(p,p2)	(p + ((((STDSIZ/2)+((p2)/2))+2) * SKIPUNIT))
							    /* -> disc data */
#define GET_VID_HDR(p,p2,d)	(p + (((STDSIZ/2)+((p2)/2)+((d)/2)) * SKIPUNIT))
							    /* -> VID_HDR */
#define GET_vid(p,p2,d) (p + ((((STDSIZ/2)+((p2)/2)+((d)/2))+2) * SKIPUNIT))
							    /* -> vid data */
#define GET_SPILL_HDR(p) (p)				    /* -> SPILL_HDR */
#define GET_spill(p)	(p + (2 * SKIPUNIT))		    /* -> spill no */
#define GET_DL_HDR(p)	(p + SKIPUNIT)			    /* -> DL_HDR */

#define ALIGN_FILL  0xdd    /* same as in ace_pc data.c */

#endif /* _MSU_H */
